@@ -11,8 +11,6 @@
 #ifndef _GASNET_EXTENDED_HELP_H
 #define _GASNET_EXTENDED_HELP_H
 
-GASNETI_BEGIN_EXTERNC
-
 #include <gasnet_help.h>
 
 /* ------------------------------------------------------------------------------------ */
@@ -57,20 +55,40 @@ GASNETI_BEGIN_EXTERNC
 #endif
 /* returns the runtime size of the thread table (always <= GASNETI_MAX_THREADS) */
 extern uint64_t gasneti_max_threads(void);
-extern void gasneti_fatal_threadoverflow(const char *subsystem);
+extern void gasneti_fatal_threadoverflow(const char *_subsystem);
 
 #ifndef _GASNETE_MYTHREAD
-  struct _gasnete_threaddata_t;
+  struct _gasneti_threaddata_t;
   #if GASNETI_MAX_THREADS <= 256
-    extern struct _gasnete_threaddata_t *gasnete_threadtable[GASNETI_MAX_THREADS];
+    extern struct _gasneti_threaddata_t *gasnete_threadtable[GASNETI_MAX_THREADS];
   #else
-    extern struct _gasnete_threaddata_t **gasnete_threadtable;
+    extern struct _gasneti_threaddata_t **gasnete_threadtable;
   #endif
   #if GASNETI_MAX_THREADS > 1
-    extern struct _gasnete_threaddata_t *gasnete_mythread(void) GASNETI_CONST;
-    GASNETI_CONSTP(gasnete_mythread)
+    #if GASNETI_COMPILER_IS_CC
+      GASNETI_THREADKEY_DECLARE(gasnete_threaddata);
+      extern void * gasnete_new_threaddata(void);
+      GASNETI_INLINE(_gasneti_mythread_slow) GASNETI_CONST
+      struct _gasneti_threaddata_t *_gasneti_mythread_slow(void) {
+        void *_threaddata = gasneti_threadkey_get(gasnete_threaddata);
+        GASNETI_STAT_EVENT(C, DYNAMIC_THREADLOOKUP); /* tracing here can cause inf recursion */
+        if_pf (!_threaddata) { /* first time we've seen this thread - need to set it up */
+          // NOTE: DON'T use _gasnete_mythread_slow_slow to initially populate TLS, because it's annotated const
+          // so the optimizer won't understand it modifies the TLS "global" directly accessed above
+          _threaddata = gasnete_new_threaddata();
+        }
+        gasneti_memcheck(_threaddata);
+        return _threaddata;
+      }
+      GASNETI_CONSTP(_gasneti_mythread_slow)
+    #else // !GASNETI_COMPILER_IS_CC
+      // threadkey-get currently incurs a fncall on !CC anyhow, so nothing to save here
+      extern struct _gasneti_threaddata_t *_gasnete_mythread_slow_slow(void) GASNETI_CONST;
+      GASNETI_CONSTP(_gasnete_mythread_slow_slow)
+      #define _gasneti_mythread_slow() _gasnete_mythread_slow_slow()
+    #endif
   #else
-    #define gasnete_mythread() (gasnete_threadtable[0])
+    #define _gasneti_mythread_slow() (gasnete_threadtable[0])
   #endif
 #endif
 
@@ -79,7 +97,7 @@ extern void gasneti_fatal_threadoverflow(const char *subsystem);
    run for dynamic thread exits when the process is continuing.
    Cleanups will run in reverse order of registration
  */
-extern void gasnete_register_threadcleanup(void (*cleanupfn)(void *), void *context);
+extern void gasnete_register_threadcleanup(void (*_cleanupfn)(void *), void *_context);
 
 /* free list of valget cells */
 #ifdef GASNETE_VALGET_CUSTOM
@@ -89,29 +107,15 @@ extern void gasnete_register_threadcleanup(void (*cleanupfn)(void *), void *cont
 #endif
 
 typedef struct _gasnete_thread_cleanup {
-    struct _gasnete_thread_cleanup *next;
-    void (*cleanupfn)(void *);
-    void *context;
+    struct _gasnete_thread_cleanup *_next;
+    void (*_cleanupfn)(void *);
+    void *_context;
 } gasnete_thread_cleanup_t; /* thread exit cleanup function LIFO */
 
 
-/* fields that should appear first in the threaddata struct for all conduits */
-#define GASNETE_COMMON_THREADDATA_FIELDS                                      \
-  void *gasnetc_threaddata;     /* ptr reserved for use by the core */        \
-  void *gasnete_coll_threaddata;/* ptr reserved for use by the collectives */ \
-  void *gasnete_vis_threaddata; /* ptr reserved for use by the VIS */         \
-                                                                              \
-  gasnete_threadidx_t threadidx;                                              \
-                                                                              \
-  gasnete_thread_cleanup_t *thread_cleanup; /* thread cleanup function LIFO */\
-  int thread_cleanup_delay;                                                   \
-                                                                              \
-  GASNETE_VALGET_FIELDS
-
 /* high-water mark on highest thread index allocated thus far */
 extern int gasnete_maxthreadidx;
 #define gasnete_assert_valid_threadid(threadidx) do {   \
-    GASNETI_UNUSED_UNLESS_DEBUG                         \
     int _thid = (threadidx);                            \
     gasneti_assert(_thid <= gasnete_maxthreadidx);      \
     gasneti_assert(gasnete_threadtable[_thid] != NULL); \
@@ -123,14 +127,28 @@ extern int gasnete_maxthreadidx;
    the local node id, but clients can override this to remove the check overhead
    by defining either GASNETE_PUTGET_ALWAYSLOCAL or GASNETE_PUTGET_ALWAYSREMOTE
  */
-#if defined(GASNETE_PUTGET_ALWAYSLOCAL)
-  #define gasnete_islocal(nodeid) (1) /* always local */
-#elif defined(GASNETE_PUTGET_ALWAYSREMOTE)
-  #define gasnete_islocal(nodeid) (0) /* always remote */
-#else
-  #define gasnete_islocal(nodeid) (nodeid == gasneti_mynode)
+#if GASNET_CONDUIT_SMP
+  #if GASNET_PSHM // smp w/pshm: the PSHM support handles smp loopback
+    #define gasnete_islocal(nodeid) (gasneti_assert(nodeid < gasneti_nodes),0) 
+  #else           // smp nopshm: single-process loopback handled in header
+    #define gasnete_islocal(nodeid) (gasneti_assert(nodeid == 0),1)
+  #endif
+#elif defined(GASNETE_PUTGET_ALWAYSLOCAL)  // always local
+  #define gasnete_islocal(nodeid) (gasneti_assert(nodeid == gasneti_mynode),1)
+#elif defined(GASNETE_PUTGET_ALWAYSREMOTE) // always remote
+  #define gasnete_islocal(nodeid) (gasneti_assert(nodeid != gasneti_mynode),0)
+#else // general case
+  #define gasnete_islocal(nodeid) (gasneti_assert(nodeid < gasneti_nodes),(nodeid == gasneti_mynode))
 #endif
 
+#if PLATFORM_COMPILER_CLANG // workaround bug 3763
+  GASNETI_INLINE(_gasnete_islocal)
+  int _gasnete_islocal(gasnet_node_t _nodeid) {
+    return gasnete_islocal(_nodeid);
+  }
+  #undef gasnete_islocal
+  #define gasnete_islocal(nodeid) _gasnete_islocal(nodeid)
+#endif
 /* ------------------------------------------------------------------------------------ */
 /* bug 1389: need to prevent bad optimizations on GASNETE_FAST_ALIGNED_MEMCPY due to
    ansi-aliasing rules added in C99 that foolishly outlaw type-punning. 
@@ -252,40 +270,35 @@ typedef union {
   } while(0)
 #else
 #define GASNETE_FAST_ALIGNED_MEMCPY(dest, src, nbytes) do { \
-  const void * const _src = (src);                          \
-  void * const _dst = (dest);                               \
-  switch(nbytes) {                                          \
+  const void * const _fam_src = (src);                      \
+  void * const _fam_dst = (dest);                           \
+  size_t const _fam_nbytes = (nbytes);                      \
+  switch(_fam_nbytes) {                                     \
     case 0:                                                 \
       break;                                                \
-    case 1:  *((gasnete_anytype8_t *)_dst) =                \
-             *((gasnete_anytype8_t *)_src);                 \
+    case 1:  *((gasnete_anytype8_t *)_fam_dst) =            \
+             *((gasnete_anytype8_t *)_fam_src);             \
       break;                                                \
   GASNETE_OMIT_WHEN_MISSING_16BIT(                          \
-    case 2:  *((gasnete_anytype16_t *)_dst) =               \
-             *((gasnete_anytype16_t *)_src);                \
+    case 2:  *((gasnete_anytype16_t *)_fam_dst) =           \
+             *((gasnete_anytype16_t *)_fam_src);            \
       break;                                                \
   )                                                         \
-    case 4:  *((gasnete_anytype32_t *)_dst) =               \
-             *((gasnete_anytype32_t *)_src);                \
+    case 4:  *((gasnete_anytype32_t *)_fam_dst) =           \
+             *((gasnete_anytype32_t *)_fam_src);            \
       break;                                                \
-    case 8:  *((gasnete_anytype64_t *)_dst) =               \
-             *((gasnete_anytype64_t *)_src);                \
+    case 8:  *((gasnete_anytype64_t *)_fam_dst) =           \
+             *((gasnete_anytype64_t *)_fam_src);            \
       break;                                                \
     default:                                                \
-      memcpy(_dst, _src, nbytes);                           \
+      memcpy(_fam_dst, _fam_src, _fam_nbytes);              \
   }                                                         \
 } while(0)
 #endif /* GASNETI_BUG1389_WORKAROUND */
 
-#define GASNETE_FAST_UNALIGNED_MEMCPY(dest, src, nbytes) memcpy(dest, src, nbytes)
-
-/* Wrapper around GASNETE_FAST_UNALIGNED_MEMCPY which becomes a no-op if src == dst */
-#define GASNETE_FAST_UNALIGNED_MEMCPY_CHECK(dest, src, nbytes) do { \
-    void *_dest = (dest);                                           \
-    const void *_src = (src);                                       \
-    if_pt (_dest != _src)                                           \
-        GASNETE_FAST_UNALIGNED_MEMCPY(_dest, _src, (nbytes));       \
-  } while (0)
+// TODO-EX: remove these if/when all uses are updated
+#define GASNETE_FAST_UNALIGNED_MEMCPY(d,s,n)       GASNETI_MEMCPY(d,s,n)
+#define GASNETE_FAST_UNALIGNED_MEMCPY_CHECK(d,s,n) GASNETI_MEMCPY_SAFE_IDENTICAL(d,s,n)
 
 /* given the address of a gasnet_register_value_t object and the number of
    significant bytes, return the byte address where significant bytes begin */
@@ -330,11 +343,11 @@ typedef union {
 /* interpret *src as a ptr to an nbytes type,
    and return the value as a gasnet_register_value_t */
 #ifdef GASNETI_BUG1389_WORKAROUND
-  #define GASNETE_VALUE_RETURN(src, nbytes) do {              \
-    gasnet_register_value_t result = 0;                       \
-    gasneti_compiler_fence();                                 \
-    memcpy(GASNETE_STARTOFBITS(&result,nbytes), src, nbytes); \
-    return result;                                            \
+  #define GASNETE_VALUE_RETURN(src, nbytes) do {               \
+    gasnet_register_value_t _result = 0;                       \
+    gasneti_compiler_fence();                                  \
+    memcpy(GASNETE_STARTOFBITS(&_result,nbytes), src, nbytes); \
+    return _result;                                            \
   } while(0)
 #else
 #define GASNETE_VALUE_RETURN(src, nbytes) do {                               \
@@ -347,9 +360,9 @@ typedef union {
       case 4: return (gasnet_register_value_t)GASNETE_ANYTYPE_LVAL(src,32);  \
       case 8: return (gasnet_register_value_t)GASNETE_ANYTYPE_LVAL(src,64);  \
       default: { /* no such native nbytes integral type */                   \
-          gasnet_register_value_t result = 0;                                \
-          memcpy(GASNETE_STARTOFBITS(&result,nbytes), src, nbytes);          \
-          return result;                                                     \
+          gasnet_register_value_t _result = 0;                               \
+          memcpy(GASNETE_STARTOFBITS(&_result,nbytes), src, nbytes);         \
+          return _result;                                                    \
       }                                                                      \
     }                                                                        \
   } while (0)
@@ -369,21 +382,21 @@ typedef union {
     #define gasnete_aligncheck(ptr,nbytes) do {                                         \
         uint8_t *_gasnete_alignbuf =                                                    \
           (uint8_t *)(((uintptr_t)&(_gasnete_aligncheck[0x100])) & ~((uintptr_t)0xFF)); \
-        uintptr_t offset = ((uintptr_t)(ptr)) & 0xFF;                                   \
-        uint8_t *p = _gasnete_alignbuf + offset;                                        \
-        gasneti_assert(p >= _gasnete_aligncheck &&                                      \
-              (p + 8) < (_gasnete_aligncheck+sizeof(_gasnete_aligncheck)));             \
+        uintptr_t _offset = ((uintptr_t)(ptr)) & 0xFF;                                  \
+        uint8_t *_p = _gasnete_alignbuf + _offset;                                      \
+        gasneti_assert(_p >= _gasnete_aligncheck &&                                     \
+              (_p + 8) < (_gasnete_aligncheck+sizeof(_gasnete_aligncheck)));            \
         /* NOTE: a runtime bus error in this code indicates the relevant pointer        \
             was not "properly aligned for accessing objects of size nbytes", as         \
             required by the GASNet spec for src/dest addresses in non-bulk puts/gets    \
          */                                                                             \
         switch (nbytes) {                                                               \
-          case 1: *(uint8_t *)p = 0; break;                                             \
+          case 1: *(uint8_t *)_p = 0; break;                                            \
         GASNETE_OMIT_WHEN_MISSING_16BIT(                                                \
-          case 2: *(uint16_t *)p = 0; break;                                            \
+          case 2: *(uint16_t *)_p = 0; break;                                           \
         )                                                                               \
-          case 4: *(uint32_t *)p = 0; break;                                            \
-          case 8: *(uint64_t *)p = 0; break;                                            \
+          case 4: *(uint32_t *)_p = 0; break;                                           \
+          case 8: *(uint64_t *)_p = 0; break;                                           \
         }                                                                               \
       } while (0)
   #endif
@@ -416,95 +429,71 @@ typedef union {
   #define gasnete_loopbackget_memsync()
 #endif
 
-/* ------------------------------------------------------------------------------------ */
-/* thread-id optimization support */
-#ifdef GASNETI_THREADINFO_OPT
-  #if GASNETI_RESTRICT_MAY_QUALIFY_TYPEDEFS
-    #define GASNETE_THREAD_FARG_ALONE   gasnet_threadinfo_t const GASNETI_RESTRICT _threadinfo
-  #else
-    #define GASNETE_THREAD_FARG_ALONE   void * const GASNETI_RESTRICT _threadinfo
-  #endif
-  #define GASNETE_THREAD_FARG         , GASNETE_THREAD_FARG_ALONE
-  #define GASNETE_THREAD_GET_ALONE    GASNET_GET_THREADINFO()
-  #define GASNETE_THREAD_GET          , GASNETE_THREAD_GET_ALONE
-  #define GASNETE_THREAD_PASS_ALONE   (_threadinfo)
-  #define GASNETE_THREAD_PASS         , GASNETE_THREAD_PASS_ALONE
-  #define GASNETE_THREAD_LOOKUP       GASNETE_THREAD_FARG_ALONE = GASNETE_THREAD_GET_ALONE;
-  #define GASNETE_THREAD_SWALLOW(x)
-  #define GASNETE_TISTARTOFBITS(ptr,nbytes,ti) GASNETE_STARTOFBITS(ptr,nbytes)
-  #define GASNETE_MYTHREAD            ((struct _gasnete_threaddata_t *)_threadinfo)
-#else
-  #define GASNETE_THREAD_FARG_ALONE   void
-  #define GASNETE_THREAD_FARG         
-  #define GASNETE_THREAD_GET_ALONE   
-  #define GASNETE_THREAD_GET         
-  #define GASNETE_THREAD_PASS_ALONE   
-  #define GASNETE_THREAD_PASS         
-  #define GASNETE_THREAD_LOOKUP
-  #define GASNETE_THREAD_SWALLOW(x)
-  #define GASNETE_TISTARTOFBITS       GASNETE_STARTOFBITS
-  #define GASNETE_MYTHREAD            (gasnete_mythread())
-#endif
-/* ------------------------------------------------------------------------------------ */
 
 /* helper macros */
 #define _GASNETI_RETURN_V  return
 #define _GASNETI_RETURN_H  return GASNET_INVALID_HANDLE
-#define GASNETI_CHECKZEROSZ_GET(variety, rt) do {            \
-    if_pf (nbytes == 0) {                                    \
-      GASNETI_TRACE_GET_LOCAL(variety,dest,node,src,nbytes); \
-      _GASNETI_RETURN_##rt;                                  \
+#define GASNETI_CHECKZEROSZ_GET(variety,rt,dest,node,src,nbytes) do {    \
+    if_pf (nbytes == 0) {                                                \
+      GASNETI_TRACE_GET_LOCAL(variety,dest,node,src,nbytes);             \
+      _GASNETI_RETURN_##rt;                                              \
     } } while(0)
-#define GASNETI_CHECKZEROSZ_PUT(variety, rt) do {            \
-    if_pf (nbytes == 0) {                                    \
-      GASNETI_TRACE_PUT_LOCAL(variety,node,dest,src,nbytes); \
-      _GASNETI_RETURN_##rt;                                  \
+#define GASNETI_CHECKZEROSZ_PUT(variety,rt,node,dest,src,nbytes) do {    \
+    if_pf (nbytes == 0) {                                                \
+      GASNETI_TRACE_PUT_LOCAL(variety,node,dest,src,nbytes);             \
+      _GASNETI_RETURN_##rt;                                              \
     } } while(0)
-#define GASNETI_CHECKZEROSZ_MEMSET(variety, rt) do {            \
-    if_pf (nbytes == 0) {                                       \
-      GASNETI_TRACE_MEMSET_LOCAL(variety,node,dest,val,nbytes); \
-      _GASNETI_RETURN_##rt;                                     \
+#define GASNETI_CHECKZEROSZ_MEMSET(variety,rt,node,dest,val,nbytes) do { \
+    if_pf (nbytes == 0) {                                                \
+      GASNETI_TRACE_MEMSET_LOCAL(variety,node,dest,val,nbytes);          \
+      _GASNETI_RETURN_##rt;                                              \
     } } while(0)
-#define GASNETI_CHECKZEROSZ_NAMED(tracecall, rt) do { \
-    if_pf (nbytes == 0) {                             \
-      tracecall;                                      \
-      _GASNETI_RETURN_##rt;                           \
+#define GASNETI_CHECKZEROSZ_NAMED(tracecall,rt,nbytes) do {              \
+    if_pf (nbytes == 0) {                                                \
+      tracecall;                                                         \
+      _GASNETI_RETURN_##rt;                                              \
     } } while(0)
 #if GASNET_PSHM
-  #define GASNETI_CHECKPSHM_GET(align, rt) do { \
-    if (gasneti_pshm_in_supernode(node)) {      \
+  #define GASNETI_CHECKPSHM_GET(align,rt,dest,node,src,nbytes) do {                    \
+    if (gasneti_pshm_in_supernode(node)) {                                             \
       GASNETE_FAST_##align##_MEMCPY(dest, gasneti_pshm_addr2local(node, src), nbytes); \
-      gasnete_loopbackget_memsync();            \
-      _GASNETI_RETURN_##rt;                     \
+      gasnete_loopbackget_memsync();                                                   \
+      _GASNETI_RETURN_##rt;                                                            \
     }} while(0)
-  #define GASNETI_CHECKPSHM_PUT(align, rt) do { \
-    if (gasneti_pshm_in_supernode(node)) {      \
+  #define GASNETI_CHECKPSHM_PUT(align, rt,node,dest,src,nbytes) do {                   \
+    if (gasneti_pshm_in_supernode(node)) {                                             \
       GASNETE_FAST_##align##_MEMCPY(gasneti_pshm_addr2local(node, dest), src, nbytes); \
-      gasnete_loopbackput_memsync();            \
-      _GASNETI_RETURN_##rt;                     \
+      gasnete_loopbackput_memsync();                                                   \
+      _GASNETI_RETURN_##rt;                                                            \
     }} while(0)
-  #define GASNETI_CHECKPSHM_GETVAL() do {     \
-    if (gasneti_pshm_in_supernode(node)) {      \
-      GASNETE_VALUE_RETURN(gasneti_pshm_addr2local(node, src), nbytes); \
+  #define GASNETI_CHECKPSHM_GETVAL(node,src,nbytes) do {                               \
+    if (gasneti_pshm_in_supernode(node)) {                                             \
+      GASNETE_VALUE_RETURN(gasneti_pshm_addr2local(node, src), nbytes);                \
     }} while(0)
-  #define GASNETI_CHECKPSHM_PUTVAL(rt) do {     \
-    if (gasneti_pshm_in_supernode(node)) {      \
-      GASNETE_VALUE_ASSIGN(gasneti_pshm_addr2local(node, dest), value, nbytes); \
-      gasnete_loopbackput_memsync();            \
-      _GASNETI_RETURN_##rt;                     \
+  #define GASNETI_CHECKPSHM_PUTVAL(rt,node,dest,value,nbytes) do {                     \
+    if (gasneti_pshm_in_supernode(node)) {                                             \
+      GASNETE_VALUE_ASSIGN(gasneti_pshm_addr2local(node, dest), value, nbytes);        \
+      gasnete_loopbackput_memsync();                                                   \
+      _GASNETI_RETURN_##rt;                                                            \
     }} while(0)
-  #define GASNETI_CHECKPSHM_MEMSET(rt) do {     \
-    if (gasneti_pshm_in_supernode(node)) {      \
-      memset(gasneti_pshm_addr2local(node, dest), val, nbytes); \
-      gasnete_loopbackput_memsync();            \
-      _GASNETI_RETURN_##rt;                     \
+  #define GASNETI_CHECKPSHM_MEMSET(rt,node,dest,val,nbytes) do {                       \
+    if (gasneti_pshm_in_supernode(node)) {                                             \
+      memset(gasneti_pshm_addr2local(node, dest), val, nbytes);                        \
+      gasnete_loopbackput_memsync();                                                   \
+      _GASNETI_RETURN_##rt;                                                            \
     }} while(0)
+  #define GASNETI_SUPERNODE_LOCAL(node) gasneti_pshm_in_supernode(node) 
 #else
-  #define GASNETI_CHECKPSHM_GET(align, rt) ((void)0)
-  #define GASNETI_CHECKPSHM_PUT(align, rt) ((void)0)
-  #define GASNETI_CHECKPSHM_GETVAL()       ((void)0)
-  #define GASNETI_CHECKPSHM_PUTVAL(rt)     ((void)0)
-  #define GASNETI_CHECKPSHM_MEMSET(rt)     ((void)0)
+  #define GASNETI_CHECKPSHM_GET(align,rt,dest,node,src,nbytes) ((void)0)
+  #define GASNETI_CHECKPSHM_PUT(align,rt,node,dest,src,nbytes) ((void)0)
+  #define GASNETI_CHECKPSHM_GETVAL(node,src,nbytes)            ((void)0)
+  #define GASNETI_CHECKPSHM_PUTVAL(rt,node,dest,value,nbytes)  ((void)0)
+  #define GASNETI_CHECKPSHM_MEMSET(rt,node,dest,val,nbytes)    ((void)0)
+  #if GASNET_CONDUIT_SMP
+    #define GASNETI_SUPERNODE_LOCAL(node)    (1)
+  #else 
+    #define GASNETI_SUPERNODE_LOCAL(node)    ((node) == gasnet_mynode()) 
+  #endif
 #endif
 
 /* ------------------------------------------------------------------------------------ */
@@ -513,6 +502,4 @@ typedef union {
   #include <gasnet_extended_help_extra.h>
 #endif
 
-GASNETI_END_EXTERNC
-
 #endif
