@@ -16,6 +16,10 @@
   #error Internal GASNet code should not directly include gasnet.h, just gasnet_internal.h
 #endif
 
+#ifdef __cplusplus
+  extern "C" { // cannot use GASNETI_BEGIN_EXTERNC here due to a header dependency cycle
+#endif
+
 /* Usage:
    see the GASNet specification and top-level README for details on how to use the GASNet interface
    clients should use the automatically-generated Makefile *.mak fragments to get the correct compile settings
@@ -113,6 +117,8 @@
 /* basic utilities used in the headers */
 #include <gasnet_basic.h>
 
+GASNETI_BEGIN_NOWARN
+
 /* ------------------------------------------------------------------------------------ */
 /* check segment configuration */
 
@@ -140,8 +146,11 @@
 #endif
 
 /* additional safety check, in case a very smart linker removes all of the checks at the end of this file */
-#define gasnet_init _CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT( \
+#define gasnet_init _CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT(_CONCAT( \
                     gasnet_init_GASNET_,                             \
+                    GASNET_RELEASE_VERSION_MAJOR),                   \
+                    GASNET_RELEASE_VERSION_MINOR),                   \
+                    GASNET_RELEASE_VERSION_PATCH),                   \
                     GASNETI_THREAD_MODEL),                           \
                     GASNETI_PSHM_CONFIG_ENABLED),                    \
                     GASNETI_SEGMENT_CONFIG),                         \
@@ -151,6 +160,18 @@
                     GASNETI_MALLOC_CONFIG),                          \
                     GASNETI_SRCLINES_CONFIG)
 
+/* ------------------------------------------------------------------------------------ */
+// Default (read-only) shared-memory MaxMedium value "recommended" to conduits.
+// Either GASNETC_MAX_MEDIUM_PSHM or gasnet_AMMaxMedium determines the actual maximum.
+// See template-conduit/gasnet_core.h for more info.
+#if !GASNETI_PSHM_ENABLED
+  #define GASNETC_MAX_MEDIUM_PSHM_DFLTMAX 65536
+#elif PLATFORM_ARCH_64
+  #define GASNETC_MAX_MEDIUM_PSHM_DFLTMAX 65416
+#else
+  #define GASNETC_MAX_MEDIUM_PSHM_DFLTMAX 65436
+#endif
+
 /* ------------------------------------------------------------------------------------ */
 /* GASNet forward definitions, which may override some of the defaults below */
 #include <gasnet_core_fwd.h>
@@ -229,6 +250,10 @@
   /* UNNAMED includes ANONYMOUS to yield a trivial default implementation: */
   #define GASNETE_BARRIERFLAG_UNNAMED 8
   #define GASNET_BARRIERFLAG_UNNAMED (GASNET_BARRIERFLAG_ANONYMOUS|GASNETE_BARRIERFLAG_UNNAMED)
+
+  /* reserve bits for use by conduit-specific implementations */
+  #define GASNETE_BARRIERFLAG_CONDUIT0 0x80000000
+  #define GASNETE_BARRIERFLAG_CONDUIT1 0x40000000
 #endif
 
 /* Errors: GASNET_OK must be zero */
@@ -244,15 +269,8 @@
   #define GASNET_ERR_BARRIER_MISMATCH     (_GASNET_ERR_BASE+5)
 #endif
 
-/* Largest Medium supported by AMPSHM */
-#ifndef GASNETI_MAX_MEDIUM_PSHM
-  #define GASNETI_MAX_MEDIUM_PSHM 65000
-#endif
-
-GASNETI_BEGIN_EXTERNC
 extern const char *gasnet_ErrorName(int);
 extern const char *gasnet_ErrorDesc(int);
-GASNETI_END_EXTERNC
 
 /* ------------------------------------------------------------------------------------ */
 /* core types */
@@ -286,11 +304,7 @@ GASNETI_END_EXTERNC
   /*  struct type used to negotiate handler registration in gasnet_init() */
   typedef struct gasneti_handlerentry_s {
     gasnet_handler_t index; /*  == 0 for don't care  */
-   #ifdef GASNET_USE_STRICT_PROTOTYPES
-    void *fnptr;    
-   #else
     void (*fnptr)();    
-   #endif
   } gasnet_handlerentry_t;
 #endif
 
@@ -357,8 +371,20 @@ extern void (*gasnet_client_attach_hook)(void *, uintptr_t);
 /* Main extended header */
 #include <gasnet_extended.h>
 
+#if GASNETI_THREADINFO_OPT
+  #define GASNETI_TIOPT_CONFIG tiopt
+#else
+  #define GASNETI_TIOPT_CONFIG notiopt
+#endif
+
 /* ------------------------------------------------------------------------------------ */
 
+#if !defined(GASNET_NULL_ARGV_OK) || \
+    (defined(GASNET_NULL_ARGV_OK) && GASNET_NULL_ARGV_OK != 0 && GASNET_NULL_ARGV_OK != 1)
+  /*  defined to be 1 if gasnet_init(NULL,NULL) is supported. defined to 0 otherwise */
+  #error GASNet core failed to define GASNET_NULL_ARGV_OK to 0 or 1
+#endif
+
 #ifndef GASNET_BEGIN_FUNCTION
   #error GASNet extended API failed to define GASNET_BEGIN_FUNCTION
 #endif
@@ -427,8 +453,12 @@ extern void (*gasnet_client_attach_hook)(void *, uintptr_t);
  * all objects in a given executable (client and library) must agree on all
  * of the following configuration settings, otherwise MANY things break,
  * often in very subtle and confusing ways (eg GASNet mutexes, threadinfo, etc.)
+ * DO NOT REMOVE ANYTHING FROM THIS LIST!!!!
  */
 #define GASNETI_LINKCONFIG_IDIOTCHECK(name) _CONCAT(gasneti_linkconfig_idiotcheck_,name)
+extern int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MAJOR_,GASNET_RELEASE_VERSION_MAJOR));
+extern int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MINOR_,GASNET_RELEASE_VERSION_MINOR));
+extern int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_PATCH_,GASNET_RELEASE_VERSION_PATCH));
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_THREAD_MODEL);
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_SEGMENT_CONFIG);
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_DEBUG_CONFIG);
@@ -444,16 +474,23 @@ extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_MEMBAR_CONFIG);
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC_CONFIG);
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC32_CONFIG);
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC64_CONFIG);
+extern int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_TIOPT_CONFIG);
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(CORE_,GASNET_CORE_NAME));
 extern int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(EXTENDED_,GASNET_EXTENDED_NAME));
 
 static int *gasneti_linkconfig_idiotcheck(void);
-/* use of void* here avoids a tinyc bug */
-static void *_gasneti_linkconfig_idiotcheck = (void *)&gasneti_linkconfig_idiotcheck;
+#if !PLATFORM_COMPILER_TINY /* avoid a tinyc bug */
+  #define GASNETI_IDIOTCHECK_RECURSIVE_REFERENCE 1
+  static int *(*_gasneti_linkconfig_idiotcheck)(void) = &gasneti_linkconfig_idiotcheck;
+#endif
 GASNETI_USED
 static int *gasneti_linkconfig_idiotcheck(void) {
-  static int val;
-  val +=  GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_THREAD_MODEL)
+  static int _val;
+  _val +=
+        + GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MAJOR_,GASNET_RELEASE_VERSION_MAJOR))
+        + GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MINOR_,GASNET_RELEASE_VERSION_MINOR))
+        + GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_PATCH_,GASNET_RELEASE_VERSION_PATCH))
+        + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_THREAD_MODEL)
         + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_SEGMENT_CONFIG)
         + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_DEBUG_CONFIG)
         + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_TRACE_CONFIG)
@@ -468,23 +505,26 @@ static int *gasneti_linkconfig_idiotcheck(void) {
         + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC_CONFIG)
         + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC32_CONFIG)
         + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC64_CONFIG)
+        + GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_TIOPT_CONFIG)
         + GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(CORE_,GASNET_CORE_NAME))
         + GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(EXTENDED_,GASNET_EXTENDED_NAME))
         ;
-  if (_gasneti_linkconfig_idiotcheck != (void*)&gasneti_linkconfig_idiotcheck)
-    val += ((int(*)(void))_gasneti_linkconfig_idiotcheck)();
-  return &val;
+  #if GASNETI_IDIOTCHECK_RECURSIVE_REFERENCE
+  if (_gasneti_linkconfig_idiotcheck == &gasneti_linkconfig_idiotcheck)
+    _val += *(*_gasneti_linkconfig_idiotcheck)();
+  #endif
+  return &_val;
 }
-extern int gasneti_internal_idiotcheck(gasnet_handlerentry_t *table, int numentries,
-                                       uintptr_t segsize, uintptr_t minheapoffset);
+extern int gasneti_internal_idiotcheck(gasnet_handlerentry_t *_table, int _numentries,
+                                       uintptr_t _segsize, uintptr_t _minheapoffset);
 
 #if defined(GASNET_DEBUG) && (defined(__OPTIMIZE__) || defined(NDEBUG))
-  #ifndef GASNET_ALLOW_OPTIMIZED_DEBUG
     #error Tried to compile GASNet client code with optimization enabled but also GASNET_DEBUG (which seriously hurts performance). Reconfigure/rebuild GASNet without --enable-debug
-  #endif
 #endif
 
 /* ------------------------------------------------------------------------------------ */
+GASNETI_END_NOWARN
+GASNETI_END_EXTERNC
 
 #undef _IN_GASNET_H
 #endif
