@@ -33,10 +33,14 @@ int gasneti_VerboseErrors = 1;
   #ifdef GASNETI_ATOMIC_LOCK_TBL_DEFNS
     #define _gasneti_atomic_lock_initializer	GASNET_HSL_INITIALIZER
     #define _gasneti_atomic_lock_init(x)	gasnet_hsl_init(x)
+    #define _gasneti_atomic_lock_lock(x)	gasnet_hsl_lock(x)
+    #define _gasneti_atomic_lock_unlock(x)	gasnet_hsl_unlock(x)
     #define _gasneti_atomic_lock_malloc		gasneti_malloc
     GASNETI_ATOMIC_LOCK_TBL_DEFNS(gasneti_hsl_atomic_, gasnet_hsl_)
     #undef _gasneti_atomic_lock_initializer
     #undef _gasneti_atomic_lock_init
+    #undef _gasneti_atomic_lock_lock
+    #undef _gasneti_atomic_lock_unlock
     #undef _gasneti_atomic_lock_malloc
   #endif
   #ifdef GASNETI_GENATOMIC32_DEFN
@@ -95,6 +99,9 @@ void (*gasnet_client_attach_hook)(void *, uintptr_t) = NULL;
 
 int gasneti_wait_mode = GASNET_WAIT_SPIN;
 
+int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MAJOR_,GASNET_RELEASE_VERSION_MAJOR)) = 1;
+int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MINOR_,GASNET_RELEASE_VERSION_MINOR)) = 1;
+int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_PATCH_,GASNET_RELEASE_VERSION_PATCH)) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_THREAD_MODEL) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_SEGMENT_CONFIG) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_DEBUG_CONFIG) = 1;
@@ -122,16 +129,14 @@ extern int gasneti_internal_idiotcheck(gasnet_handlerentry_t *table, int numentr
   return GASNET_ERR_NOT_INIT;
 }
 
+/* global definitions of GASNet-wide internal variables
+   not subject to override */
+gasnet_node_t gasneti_mynode = (gasnet_node_t)-1;
+gasnet_node_t gasneti_nodes = 0;
+
 /* Default global definitions of GASNet-wide internal variables
    if conduits override one of these, they must
    still provide variable or macro definitions for these tokens */
-#ifdef _GASNET_MYNODE_DEFAULT
-  gasnet_node_t gasneti_mynode = (gasnet_node_t)-1;
-#endif
-#ifdef _GASNET_NODES_DEFAULT
-  gasnet_node_t gasneti_nodes = 0;
-#endif
-
 #if defined(_GASNET_GETMAXSEGMENTSIZE_DEFAULT) && !GASNET_SEGMENT_EVERYTHING
   uintptr_t gasneti_MaxLocalSegmentSize = 0;
   uintptr_t gasneti_MaxGlobalSegmentSize = 0;
@@ -222,6 +227,8 @@ extern void gasneti_check_config_preinit(void) {
 }
 
 static void gasneti_check_portable_conduit(void);
+static void gasneti_check_architecture(void);
+int gasneti_malloc_munmap_disabled = 0;
 extern void gasneti_check_config_postattach(void) {
   gasneti_check_config_preinit();
 
@@ -237,11 +244,15 @@ extern void gasneti_check_config_postattach(void) {
     if (firstcall) { /* miscellaneous conduit-independent initializations */
       firstcall = 0;
 
-      if (gasneti_getenv_yesno_withdefault("GASNET_DISABLE_MUNMAP",0)) {
+      #ifndef GASNET_DISABLE_MUNMAP_DEFAULT
+      #define GASNET_DISABLE_MUNMAP_DEFAULT 0
+      #endif
+      if (gasneti_getenv_yesno_withdefault("GASNET_DISABLE_MUNMAP",GASNET_DISABLE_MUNMAP_DEFAULT)) {
         #if HAVE_PTMALLOC                                        
           mallopt(M_TRIM_THRESHOLD, -1);
           mallopt(M_MMAP_MAX, 0);
           GASNETI_TRACE_PRINTF(I,("Setting mallopt M_TRIM_THRESHOLD=-1 and M_MMAP_MAX=0"));
+          gasneti_malloc_munmap_disabled = 1;
         #else
           GASNETI_TRACE_PRINTF(I,("WARNING: GASNET_DISABLE_MUNMAP set on an unsupported platform"));
           if (gasneti_verboseenv()) 
@@ -250,10 +261,13 @@ extern void gasneti_check_config_postattach(void) {
       }
       #if GASNET_NDEBUG
         gasneti_check_portable_conduit();
+        gasneti_check_architecture();
       #endif
     }
   }
   gasneti_memcheck_all();
+
+  gasneti_flush_streams();  // flush above messages, and ensure FS_SYNC envvar is initted
 }
 
 /* ------------------------------------------------------------------------------------ */
@@ -299,10 +313,75 @@ extern void gasneti_defaultAMHandler(gasnet_token_t token) {
                      (int)gasnet_mynode(), (int)gasnet_nodes(), (int)srcnode);
 }
 /* ------------------------------------------------------------------------------------ */
+#if GASNETC_AMREGISTER
+  /* Use conduit-specific impl */
+  extern int gasnetc_amregister(gasnet_handler_t, gasneti_handler_fn_t);
+#else
+  /* Use default/recommended impl */
+  extern gasneti_handler_fn_t gasnetc_handler[];
+  static int gasnetc_amregister(gasnet_handler_t index, gasneti_handler_fn_t fnptr) {
+    /* register a single handler */
+    gasneti_assert(gasnetc_handler[index] == gasneti_defaultAMHandler);
+    gasnetc_handler[index] = fnptr;
+    return GASNET_OK;
+  }
+#endif
+extern int gasneti_amregister(gasnet_handlerentry_t *table, int numentries,
+                               int lowlimit, int highlimit,
+                               int dontcare, int *numregistered) {
+  static char checkuniqhandler[256] = { 0 };
+  int i;
+  *numregistered = 0;
+  for (i = 0; i < numentries; i++) {
+    int newindex;
+
+    if ((table[i].index == 0 && !dontcare) ||
+        (table[i].index && dontcare)) continue;
+    else if (table[i].index) newindex = table[i].index;
+    else { /* deterministic assignment of dontcare indexes */
+      for (newindex = lowlimit; newindex <= highlimit; newindex++) {
+        if (!checkuniqhandler[newindex]) break;
+      }
+      if (newindex > highlimit) {
+        char s[255];
+        snprintf(s, sizeof(s), "Too many handlers. (limit=%i)", highlimit - lowlimit + 1);
+        GASNETI_RETURN_ERRR(BAD_ARG, s);
+      }
+    }
+
+    /*  ensure handlers fall into the proper range of pre-assigned values */
+    if (newindex < lowlimit || newindex > highlimit) {
+      char s[255];
+      snprintf(s, sizeof(s), "handler index (%i) out of range [%i..%i]", newindex, lowlimit, highlimit);
+      GASNETI_RETURN_ERRR(BAD_ARG, s);
+    }
+
+    /* discover duplicates */
+    if (checkuniqhandler[newindex] != 0)
+      GASNETI_RETURN_ERRR(BAD_ARG, "handler index not unique");
+    checkuniqhandler[newindex] = 1;
+
+    /* register the handler */
+    int rc = gasnetc_amregister((gasnet_handler_t)newindex, (gasneti_handler_fn_t)table[i].fnptr);
+    if (GASNET_OK != rc) return rc;
+
+    /* The check below for !table[i].index is redundant and present
+     * only to defeat the over-aggressive optimizer in pathcc 2.1
+     */
+    if (dontcare && !table[i].index) table[i].index = newindex;
+
+    (*numregistered)++;
+  }
+  return GASNET_OK;
+}
+/* ------------------------------------------------------------------------------------ */
 
 #ifndef GASNETC_FATALSIGNAL_CALLBACK
 #define GASNETC_FATALSIGNAL_CALLBACK(sig)
 #endif
+#ifndef GASNETC_FATALSIGNAL_CLEANUP_CALLBACK
+#define GASNETC_FATALSIGNAL_CLEANUP_CALLBACK(sig)
+#endif
 
 static void do_raise(int sig) {
 #if defined(PTHREAD_MUTEX_INITIALIZER) && !GASNET_SEQ && HAVE_PTHREAD_KILL && 0 
@@ -328,23 +407,38 @@ void gasneti_defaultSignalHandler(int sig) {
     case SIGILL:
     case SIGSEGV:
     case SIGBUS:
-    case SIGFPE:
+    case SIGFPE: {
       oldsigpipe = gasneti_reghandler(SIGPIPE, SIG_IGN);
 
       GASNETC_FATALSIGNAL_CALLBACK(sig); /* give conduit first crack at it */
-      fprintf(stderr,"*** Caught a fatal signal: %s(%i) on node %i/%i\n",
-        signame, sig, (int)gasnet_mynode(), (int)gasnet_nodes()); 
-      fflush(stderr);
+
+      FILE * streams[] = { stderr, GASNETI_MAYBE_TRACEFILE };
+      for (int s = 0; s < sizeof(streams)/sizeof(streams[0]); s++) {
+        FILE *stream = streams[s];
+        if (stream) {
+          fprintf(stream, "*** Caught a fatal signal: %s(%i) on node %i/%i\n", 
+                        signame, sig, (int)gasnet_mynode(), (int)gasnet_nodes());
+          fflush(stream);
+        }
+      }
 
       gasnett_freezeForDebuggerErr(); /* allow freeze */
 
       gasneti_print_backtrace_ifenabled(STDERR_FILENO); /* try to print backtrace */
 
+      // Try to flush I/O (especially the tracefile) before crashing
+      signal(SIGALRM, _exit); alarm(5); 
+      gasneti_flush_streams();
+      alarm(0);
+
       (void) gasneti_reghandler(SIGPIPE, oldsigpipe);
 
+      GASNETC_FATALSIGNAL_CLEANUP_CALLBACK(sig); /* conduit hook to kill the job */
+
       signal(sig, SIG_DFL); /* restore default core-dumping handler and re-raise */
       do_raise(sig);
       break;
+    }
     default: 
       /* translate signal to SIGQUIT */
       { static int sigquit_raised = 0;
@@ -637,6 +731,52 @@ extern void gasneti_decode_args(int *argc, char ***argv) {
   }
 }
 
+/* Propagate requested env vars from GASNet global env to the local env */
+
+void (*gasneti_propagate_env_hook)(const char *, int) = NULL; // spawner- or conduit-specific hook
+
+extern void gasneti_propagate_env_helper(const char *environ, const char * keyname, int flags) {
+  const int is_prefix = flags & GASNETI_PROPAGATE_ENV_PREFIX;
+  const char *p = environ;
+
+  gasneti_assert(environ);
+  gasneti_assert(keyname && !strchr(keyname,'='));
+
+  int keylen = strlen(keyname);
+  while (*p) {
+    if (!strncmp(keyname, p, keylen) && (is_prefix || p[keylen] == '=')) {
+      gasneti_assert(NULL != strchr(p+keylen, '='));
+      char *var = gasneti_strdup(p);
+      char *val = strchr(var, '=');
+      *(val++) = '\0';
+      if (gasnett_decode_envval_fn) {
+        val = (char *)((*gasnett_decode_envval_fn)(val));
+      }
+      gasnett_setenv(var, val);
+      GASNETI_TRACE_PRINTF(I,("gasneti_propagate_env(%s) => '%s'", var, val));
+      gasneti_free(var);
+      if (!is_prefix) break;
+    }
+    p += strlen(p) + 1;
+  }
+}
+
+extern void gasneti_propagate_env(const char * keyname, int flags) {
+  gasneti_assert(keyname);
+  gasneti_assert(NULL == strchr(keyname, '='));
+
+  // First look for matches in gasneti_globalEnv (if any)
+  if (gasneti_globalEnv) {
+    gasneti_propagate_env_helper(gasneti_globalEnv, keyname, flags);
+  }
+
+  // Next allow conduit-specific getenv (if any) to overwrite
+  if (gasneti_propagate_env_hook) {
+    gasneti_propagate_env_hook(keyname, flags);
+  }
+}
+
+
 /* Process environment for exittimeout.
  * If (GASNET_EXITTIMEOUT is set), it is returned
  * else return = min(GASNET_EXITTIMEOUT_MAX,
@@ -675,7 +815,11 @@ extern double gasneti_get_exittimeout(double dflt_max, double dflt_min, double d
   #endif
 
   int gasneti_pthread_create(gasneti_pthread_create_fn_t *create_fn, pthread_t *thread, const pthread_attr_t *attr, void *(*start_routine)(void *), void *arg) {
-    GASNETI_TRACE_PRINTF(I, ("gasneti_pthread_create(%p, %p, %p, %p, %p)", create_fn, thread, attr, start_routine, arg));
+    // There is no portable way to printf a pointer-to-function. This way avoids warnings with -pedantic
+    union { void *vp; gasneti_pthread_create_fn_t *cfp; void *(*sfp)(void *); } ucreate, ustart; 
+    ucreate.cfp = create_fn;
+    ustart.sfp = start_routine;
+    GASNETI_TRACE_PRINTF(I, ("gasneti_pthread_create(%p, %p, %p, %p, %p)", ucreate.vp, (void *)thread, (void *)attr, ustart.vp, arg));
     return GASNETC_PTHREAD_CREATE_OVERRIDE(create_fn, thread, attr, start_routine, arg);
   }
 #endif
@@ -691,8 +835,10 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
   m = myext; while (*mn) { *m = tolower(*mn); m++; mn++; }
   *m = '\0';
   if ( /* is a portable network conduit */
-      (!strcmp("mpi",mycore) && !strcmp("reference",myext)) || 
-      (!strcmp("udp",mycore) && !strcmp("reference",myext))
+         (!strcmp("mpi",mycore) && !strcmp("reference",myext))
+      || (!strcmp("udp",mycore) && !strcmp("reference",myext))
+      || (!strcmp("ofi",mycore) && !strcmp("ofi",myext))
+      || (!strcmp("portals4",mycore) && !strcmp("portals4",myext))
       ) {
     const char *p = GASNETI_CONDUITS;
     char natives[255];
@@ -710,13 +856,11 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
         p += len;
         p += strspn(p,GASNETI_CONDUITS_DELIM);
         /* Ignore the portable conduits */
+        if (!strcmp(name,"smp")) continue;
         if (!strcmp(name,"mpi")) continue;
         if (!strcmp(name,"udp")) continue;
-        if (!strcmp(name,"smp")) continue;
-      #if !GASNET_SEQ
-        /* Ignore conduits that lack thread safety */
-        if (!strcmp(name,"shmem")) continue;
-      #endif
+        if (!strcmp(name,"ofi")) continue;
+        if (!strcmp(name,"portals4")) continue;
         if (strlen(natives)) strcat(natives,", ");
         strcat(natives,name);
       }
@@ -782,6 +926,34 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
   }
 }
 
+static void gasneti_check_architecture(void) { // check for bad build configurations
+  #if PLATFORM_OS_CNL && PLATFORM_ARCH_X86_64 // bug 3743, verify correct processor tuning
+  { FILE *fp = fopen("/proc/cpuinfo","r");
+    char model[255];
+    if (!fp) gasneti_fatalerror("*** ERROR: Failure in fopen('/proc/cpuinfo','r')=%s",strerror(errno));
+    while (!feof(fp) && fgets(model, sizeof(model), fp)) {
+      if (strstr(model,"model name")) break;
+    }
+    fclose(fp);
+    GASNETI_TRACE_PRINTF(I,("CPU %s",model));
+    int isKNL = !!strstr(model, "Phi");
+    #ifdef __CRAY_MIC_KNL  // module craype-mic-knl that tunes for AVX512
+      const char *warning = isKNL ? 0 :
+      "WARNING: This executable was optimized for MIC KNL (module craype-mic-knl) but run on another processor!\n";
+    #else // some other x86 tuning mode
+      const char *warning = isKNL ? 
+      "WARNING: This executable is running on a MIC KNL architecture, but was not optimized for MIC KNL.\n"
+      "WARNING: This often has a MAJOR impact on performance. Please re-build with module craype-mic-knl!\n"
+      : 0;
+    #endif
+    if (warning && gasnet_mynode() == 0) {
+      fprintf(stderr, warning);
+      fflush(stderr);
+    }
+  }
+  #endif
+}
+
 /* ------------------------------------------------------------------------------------ */
 /* Nodemap handling
  */
@@ -881,7 +1053,9 @@ static void gasneti_nodemap_helper_qsort(const char *ids, size_t sz, size_t stri
 GASNETI_NEVER_INLINE(gasneti_nodemap_helper,
 static void gasneti_nodemap_helper(const void *ids, size_t sz, size_t stride)) {
   #ifndef GASNETC_DEFAULT_NODEMAP_EXACT
-    #define GASNETC_DEFAULT_NODEMAP_EXACT 0
+    // Default to slow-but-steady (it wins the race)
+    // However, see Bug 3770 - RFE: restore default linear-time nodemap behavior
+    #define GASNETC_DEFAULT_NODEMAP_EXACT 1
   #endif
   gasneti_assert(ids);
   gasneti_assert(sz > 0);
@@ -1258,6 +1432,79 @@ ssize_t gasneti_getline(char **buf_p, size_t *n_p, FILE *fp) {
 }
 #endif /* gasneti_getline */
 
+/* ------------------------------------------------------------------------------------ */
+// Internal conduit interface to spawner
+
+#if HAVE_SSH_SPAWNER
+  extern gasneti_spawnerfn_t const *gasneti_bootstrapInit_ssh(int *argc, char ***argv, gasnet_node_t *nodes, gasnet_node_t *mynode);
+#endif
+#if HAVE_MPI_SPAWNER
+  extern gasneti_spawnerfn_t const *gasneti_bootstrapInit_mpi(int *argc, char ***argv, gasnet_node_t *nodes, gasnet_node_t *mynode);
+#endif
+#if HAVE_PMI_SPAWNER
+  extern gasneti_spawnerfn_t const *gasneti_bootstrapInit_pmi(int *argc, char ***argv, gasnet_node_t *nodes, gasnet_node_t *mynode);
+#endif
+
+extern gasneti_spawnerfn_t const *gasneti_spawnerInit(int *argc_p, char ***argv_p,
+                                  const char *force_spawner,
+                                  gasnet_node_t *nodes_p, gasnet_node_t *mynode_p) {
+  gasneti_spawnerfn_t const *res = NULL;
+  const char *not_set = "(not set)";
+  const char *spawner;
+  char *tmp = NULL;
+  if (force_spawner) spawner = force_spawner;
+  else { 
+    // Purposely hide this variable from verbose output, since it's only for use as an internal hand-off
+    // from gasnetrun scripts. End users should set GASNET_<conduit>_SPAWNER
+    spawner = gasneti_getenv("GASNET_SPAWN_CONTROL");
+    if (!spawner) spawner = not_set;
+  }
+
+  if (spawner != not_set) { // upper-case
+    tmp = gasneti_strdup(spawner);
+    for (char *p = tmp; *p; p++) *p = toupper(*p);
+    spawner = tmp;
+  }
+
+#if HAVE_MPI_SPAWNER
+  /* bug 3406: Try MPI-based spawn first, EVEN if the var is not set.
+   * This is a requirement for spawning using bare mpirun
+   */
+  if (!res && (spawner == not_set || !strcmp(spawner, "MPI"))) {
+    res = gasneti_bootstrapInit_mpi(argc_p, argv_p, nodes_p, mynode_p);
+  }
+#endif
+
+#if HAVE_SSH_SPAWNER
+  /* GASNET_SPAWN_CONTROL=ssh is set by gasnetrun for the ssh spawn master,
+   * and by the ssh command line for other processes (ie all normal uses).
+   * We no longer claim to support ssh-based launch without gasnetrun.
+   * TODO: should we remove the "spawner == not_set" case?
+   */
+  if (!res && (spawner == not_set || !strcmp(spawner, "SSH"))) {
+    res = gasneti_bootstrapInit_ssh(argc_p, argv_p, nodes_p, mynode_p);
+  }
+#endif
+
+#if HAVE_PMI_SPAWNER
+  /* GASNET_SPAWN_CONTROL=pmi is set by gasnetrun for the pmi spawn case.
+   * We no longer claim to support direct launch with srun, yod, etc.
+   * TODO: should we remove the "spawner == not_set" case?
+   */
+  if (!res && (spawner == not_set || !strcmp(spawner, "PMI"))) {
+    res = gasneti_bootstrapInit_pmi(argc_p, argv_p, nodes_p, mynode_p);
+  }
+#endif
+
+  if (!res) {
+    gasneti_fatalerror("Requested spawner \"%s\" is unknown or not supported in this build", spawner);
+  }
+
+  gasneti_free(tmp);
+
+  return res;
+}
+
 /* ------------------------------------------------------------------------------------ */
 /* Debug memory management
    debug memory format:
@@ -1313,10 +1560,10 @@ ssize_t gasneti_getline(char **buf_p, size_t *n_p, FILE *fp) {
   static int gasneti_memalloc_envisinit = 0;
   static gasneti_mutex_t gasneti_memalloc_lock = GASNETI_MUTEX_INITIALIZER;
   static gasneti_memalloc_desc_t *gasneti_memalloc_pos = NULL;
-  #define GASNETI_MEM_BEGINPOST   ((uint64_t)0xDEADBABEDEADBABEllu)
-  #define GASNETI_MEM_LEAKMARK    ((uint64_t)0xBABEDEADCAFEBEEFllu)
-  #define GASNETI_MEM_ENDPOST     ((uint64_t)0xCAFEDEEDCAFEDEEDllu)
-  #define GASNETI_MEM_FREEMARK    ((uint64_t)0xBEEFEFADBEEFEFADllu)
+  #define GASNETI_MEM_BEGINPOST   ((uint64_t)0xDEADBABEDEADBABEULL)
+  #define GASNETI_MEM_LEAKMARK    ((uint64_t)0xBABEDEADCAFEBEEFULL)
+  #define GASNETI_MEM_ENDPOST     ((uint64_t)0xCAFEDEEDCAFEDEEDULL)
+  #define GASNETI_MEM_FREEMARK    ((uint64_t)0xBEEFEFADBEEFEFADULL)
   #define GASNETI_MEM_HEADERSZ    (sizeof(gasneti_memalloc_desc_t))
   #define GASNETI_MEM_TAILSZ      8     
   #define GASNETI_MEM_EXTRASZ     (GASNETI_MEM_HEADERSZ+GASNETI_MEM_TAILSZ)     
@@ -1329,8 +1576,8 @@ ssize_t gasneti_getline(char **buf_p, size_t *n_p, FILE *fp) {
        Quiet NaN: any bit pattern between 0x7ff8000000000000 and 0x7fffffffffffffff 
                or any bit pattern between 0xfff8000000000000 and 0xffffffffffffffff
     */
-    uint64_t sNAN = 0x7ff7ffffffffffffllu; 
-    uint64_t qNAN = 0x7fffffffffffffffllu;
+    uint64_t sNAN = ((uint64_t)0x7ff7ffffffffffffULL); 
+    uint64_t qNAN = ((uint64_t)0x7fffffffffffffffULL);
     uint64_t val = 0;
     const char *envval = gasneti_getenv_withdefault(name, deflt);
     const char *p = envval;
@@ -1601,16 +1848,17 @@ ssize_t gasneti_getline(char **buf_p, size_t *n_p, FILE *fp) {
     gasneti_assert_always((((uintptr_t)ret) & 0x3) == 0); /* should have at least 4-byte alignment */
     if_pf (ret == NULL) {
       char curlocstr[GASNETI_MAX_LOCSZ];
+      strcpy(curlocstr, "\n   at: %s");
       if (allowfail) {
         if_pt (gasneti_attach_done) { gasnet_resume_interrupts(); }
-        GASNETI_TRACE_PRINTF(I,("Warning: returning NULL for a failed gasneti_malloc(%lu): %s",
-                                (unsigned long)nbytes, _gasneti_format_curloc(curlocstr,curloc)));
+        GASNETI_TRACE_PRINTF(I,("Warning: returning NULL for a failed gasneti_malloc(%"PRIuPTR")%s",
+                                (uintptr_t)nbytes, _gasneti_format_curloc(curlocstr,curloc)));
         return NULL;
       }
-      gasneti_fatalerror("Debug malloc(%lu) failed (%lu bytes in use, in %lu objects): %s", 
-                     (unsigned long)nbytes, 
-                     (unsigned long)(gasneti_memalloc_allocatedbytes - gasneti_memalloc_freedbytes),
-                     (unsigned long)(gasneti_memalloc_allocatedobjects - gasneti_memalloc_freedobjects),
+      gasneti_fatalerror("Debug malloc(%"PRIuPTR") failed (%"PRIu64" bytes in use, in %"PRIu64" objects)%s", 
+                     (uintptr_t)nbytes, 
+                     (gasneti_memalloc_allocatedbytes - gasneti_memalloc_freedbytes),
+                     (gasneti_memalloc_allocatedobjects - gasneti_memalloc_freedobjects),
                      _gasneti_format_curloc(curlocstr,curloc));
     } else {
       uint64_t gasneti_endpost_ref = GASNETI_MEM_ENDPOST;
@@ -1802,15 +2050,37 @@ extern char *_gasneti_extern_strndup(const char *s, size_t n GASNETI_CURLOCFARG)
 }
 
 #if GASNET_DEBUGMALLOC
-  extern void *(*gasnett_debug_malloc_fn)(size_t sz GASNETI_CURLOCFARG);
-  extern void *(*gasnett_debug_calloc_fn)(size_t N, size_t S GASNETI_CURLOCFARG);
-  extern void (*gasnett_debug_free_fn)(void *ptr GASNETI_CURLOCFARG);
-  void *(*gasnett_debug_malloc_fn)(size_t sz GASNETI_CURLOCFARG) =
+  extern void *(*gasnett_debug_malloc_fn)(size_t sz, const char *curloc);
+  extern void *(*gasnett_debug_calloc_fn)(size_t N, size_t S, const char *curloc);
+  extern void *(*gasnett_debug_realloc_fn)(void *ptr, size_t sz, const char *curloc);
+  extern void (*gasnett_debug_free_fn)(void *ptr, const char *curloc);
+  extern char *(*gasnett_debug_strdup_fn)(const char *ptr, const char *curloc);
+  extern char *(*gasnett_debug_strndup_fn)(const char *ptr, size_t n, const char *curloc);
+  void *(*gasnett_debug_malloc_fn)(size_t sz, const char *curloc) =
          &_gasneti_extern_malloc;
-  void *(*gasnett_debug_calloc_fn)(size_t N, size_t S GASNETI_CURLOCFARG) =
+  void *(*gasnett_debug_calloc_fn)(size_t N, size_t S, const char *curloc) =
          &_gasneti_extern_calloc;
-  void (*gasnett_debug_free_fn)(void *ptr GASNETI_CURLOCFARG) =
+  void *(*gasnett_debug_realloc_fn)(void *ptr, size_t sz, const char *curloc) =
+        &_gasneti_extern_realloc;
+  void (*gasnett_debug_free_fn)(void *ptr, const char *curloc) =
          &_gasneti_extern_free;
+  char *(*gasnett_debug_strdup_fn)(const char *s, const char *curloc) =
+         &_gasneti_extern_strdup;
+  char *(*gasnett_debug_strndup_fn)(const char *s, size_t sz, const char *curloc) =
+         &_gasneti_extern_strndup;
+  /* these only exist with debug malloc */
+  extern void (*gasnett_debug_memcheck_fn)(void *ptr, const char *curloc);
+  extern void (*gasnett_debug_memcheck_one_fn)(const char *curloc);
+  extern void (*gasnett_debug_memcheck_all_fn)(const char *curloc);
+  extern void _gasneti_extern_memcheck(void *ptr, const char *curloc) {
+    _gasneti_memcheck(ptr, curloc, 0);
+  }
+  void (*gasnett_debug_memcheck_fn)(void *ptr, const char *curloc) = 
+        &_gasneti_extern_memcheck;
+  void (*gasnett_debug_memcheck_one_fn)(const char *curloc) = 
+        &_gasneti_memcheck_one;
+  void (*gasnett_debug_memcheck_all_fn)(const char *curloc) =
+        &_gasneti_memcheck_all;
 #endif
 
 /* don't put anything here - malloc stuff must come last */
