@@ -54,53 +54,39 @@ static struct gasneti_pshm_info {
 #define pshmnet_get_struct_addr_from_field_addr(structname, fieldname, fieldaddr) \
         ((structname*)(((uintptr_t)fieldaddr) - offsetof(structname,fieldname)))
 
-void *gasneti_pshm_init(gasneti_bootstrapExchangefn_t exchangefn, size_t aux_sz) {
+static void (*gasnetc_pshm_abort_callback)(void);
+
+void *gasneti_pshm_init(gasneti_bootstrapBroadcastfn_t snodebcastfn, size_t aux_sz) {
   size_t vnetsz, mmapsz;
   int discontig = 0;
-  gasneti_pshm_rank_t *pshm_max_nodes;
   gasnet_node_t i;
 #if !GASNET_CONDUIT_SMP
   gasnet_node_t j;
 #endif
 
-  gasneti_assert(exchangefn != NULL);  /* NULL exchangefn no longer supported */
+  gasneti_assert(snodebcastfn != NULL);  /* NULL snodebcastfn no longer supported */
 
-  /* Testing if the number of PSHM nodes is always smaller than GASNETI_PSHM_MAX_NODES */
-  pshm_max_nodes = gasneti_calloc(gasneti_nodes, sizeof(gasneti_pshm_rank_t));
-  for(i=0; i<gasneti_nodes; i++){
-    /* Note combination of post-increment and == guard against overflow */
-    if ((pshm_max_nodes[gasneti_nodemap[i]]++) == GASNETI_PSHM_MAX_NODES){
-      if (gasneti_mynode == gasneti_nodemap[i]){
-        gasneti_fatalerror("PSHM nodes requested on node '%s' exceeds maximum (%d)\n", 
-                        gasneti_gethostname(), GASNETI_PSHM_MAX_NODES);
-      } else {
-        gasneti_fatalerror("PSHM nodes requested on some node exceeds maximum (%d)\n", 
-                        GASNETI_PSHM_MAX_NODES);
-      }
-    }
-  }
-  gasneti_free(pshm_max_nodes);
+  gasneti_assert_always(gasneti_nodemap_local_count <= GASNETI_PSHM_MAX_NODES);
     
   gasneti_pshm_nodes = gasneti_nodemap_local_count;
-  gasneti_pshm_firstnode = gasneti_nodemap[gasneti_mynode];
   gasneti_pshm_mynode = gasneti_nodemap_local_rank;
+  gasneti_pshm_firstnode = gasneti_nodemap_local[0];
 
 #if GASNET_CONDUIT_SMP
   gasneti_assert(gasneti_pshm_nodes == gasneti_nodes);
-  gasneti_assert(gasneti_pshm_firstnode == 0);
   gasneti_assert(gasneti_pshm_mynode == gasneti_mynode);
+  gasneti_assert(gasneti_pshm_firstnode == 0);
 #else
-  gasneti_assert(gasneti_nodemap[0] == 0);
-  for (i=1; i<gasneti_nodes; ++i) {
-    /* Determine if supernode members are numbered contiguously */
-    if (gasneti_nodemap[i-1] > gasneti_nodemap[i]) {
+  /* Determine if local supernode members are numbered contiguously */
+  for (i = 1; i < gasneti_nodemap_local_count; ++i) {
+    if (gasneti_nodemap_local[i] != gasneti_pshm_firstnode + i) {
       discontig = 1;
       break;
     }
   }
 #endif
 
-  gasneti_assert(gasneti_pshm_supernodes > 0);
+  gasneti_assert(gasneti_nodemap_global_count > 0);
 
   /* compute size of vnet shared memory region */
   vnetsz = gasneti_pshmnet_memory_needed(gasneti_pshm_nodes); 
@@ -108,7 +94,7 @@ void *gasneti_pshm_init(gasneti_bootstrapExchangefn_t exchangefn, size_t aux_sz)
   { /* gasneti_pshm_info contains multiple variable-length arrays in the same space */
     size_t info_sz;
     /* space for gasneti_pshm_firsts: */
-    info_sz = gasneti_pshm_supernodes * sizeof(gasnet_node_t);
+    info_sz = gasneti_nodemap_global_count * sizeof(gasnet_node_t);
     /* optional space for gasneti_pshm_rankmap: */
     if (discontig) {
       info_sz = GASNETI_ALIGNUP(info_sz, sizeof(gasneti_pshm_rank_t));
@@ -128,7 +114,7 @@ void *gasneti_pshm_init(gasneti_bootstrapExchangefn_t exchangefn, size_t aux_sz)
 
   /* setup vnet shared memory region for AM infrastructure and supernode barrier.
    */
-  gasnetc_pshmnet_region = gasneti_mmap_vnet(mmapsz, exchangefn);
+  gasnetc_pshmnet_region = gasneti_mmap_vnet(mmapsz, snodebcastfn);
   gasneti_assert_always((((uintptr_t)gasnetc_pshmnet_region) % GASNETI_PSHMNET_PAGESIZE) == 0);
   if (gasnetc_pshmnet_region == NULL) {
     const int save_errno = errno;
@@ -166,9 +152,9 @@ void *gasneti_pshm_init(gasneti_bootstrapExchangefn_t exchangefn, size_t aux_sz)
   gasneti_pshmnet_bootstrapBarrier();
   {
     uintptr_t addr = (uintptr_t)&gasneti_pshm_info->early_barrier;
-    /* gasneti_pshm_firsts, an array of gasneti_pshm_supernodes*sizeof(gasnet_node_t): */
+    /* gasneti_pshm_firsts, an array of gasneti_nodemap_global_count*sizeof(gasnet_node_t): */
     gasneti_pshm_firsts = (gasnet_node_t *)addr;
-    addr += gasneti_pshm_supernodes * sizeof(gasnet_node_t);
+    addr += gasneti_nodemap_global_count * sizeof(gasnet_node_t);
     /* optional rankmap: */
     if (discontig) {
       addr = GASNETI_ALIGNUP(addr, sizeof(gasneti_pshm_rank_t));
@@ -189,14 +175,14 @@ void *gasneti_pshm_init(gasneti_bootstrapExchangefn_t exchangefn, size_t aux_sz)
     if (gasneti_nodemap[i] == i) {
       if (!gasneti_pshm_mynode) gasneti_pshm_firsts[j] = i;
       j += 1;
-      gasneti_assert(j <= gasneti_pshm_supernodes);
+      gasneti_assert(j <= gasneti_nodemap_global_count);
     }
   }
 #endif
 #if GASNET_DEBUG
   /* Validate gasneti_pshm_firsts[] */
   if (gasneti_pshm_mynode == 0) {
-    for (i=0; i<gasneti_pshm_supernodes; ++i) {
+    for (i=0; i<gasneti_nodemap_global_count; ++i) {
       gasneti_assert(!i || (gasneti_pshm_firsts[i] > gasneti_pshm_firsts[i-1]));
       gasneti_assert(gasneti_nodemap[gasneti_pshm_firsts[i]] == gasneti_pshm_firsts[i]);
     }
@@ -312,6 +298,12 @@ gasnet_node_t *gasneti_pshm_firsts = NULL;
  * These come early because their sizes influence allocation
  ******************************************************************************/
 
+// NOTE:
+// If sizes of these types changes, such that gasneti_pshmnet_allocator_block_t
+// also changes size, one must update GASNETC_MAX_MEDIUM_PSHM_DFLTMAX in gasnet.h
+// to ensure a tight fit in 64KB:
+//   sizeof(gasneti_pshmnet_allocator_block_t) == 65536
+
 /* TODO: Could/should we squeeze unused args out of a Medium.*/
 /* TODO: Pack category and numargs together (makes assumtion about ranges) */
 
@@ -326,12 +318,8 @@ typedef gasneti_AMPSHM_msg_t gasneti_AMPSHM_shortmsg_t;
 
 typedef struct {
   gasneti_AMPSHM_msg_t msg;
-#if (GASNETI_MAX_MEDIUM_PSHM < 65536) /* GASNET<C>_MAX_MEDIUM_PSHM often not a preprocess-time constant */
-  uint16_t numbytes;
-#else
   uint32_t numbytes;
-#endif
-  uint8_t  mediumdata[6 + GASNETC_MAX_MEDIUM_PSHM]; /* Is 2, 4 or 8-byte aligned */
+  uint8_t  mediumdata[4 + GASNETC_MAX_MEDIUM_PSHM]; /* +4 to deal with 4 or 8-byte alignment */
 } gasneti_AMPSHM_medmsg_t;
 
 typedef struct {
@@ -501,7 +489,7 @@ struct gasneti_pshmnet {
   gasneti_pshmnet_queue_t *my_queue;
   /* only need to see one's own allocator */
   gasneti_pshmnet_allocator_t *my_allocator;
-#if GASNET_PAR
+#if GASNET_PAR || GASNETI_CONDUIT_THREADS
   /* serializes dequeue operations */
   gasneti_mutex_t lock;
 #endif
@@ -517,7 +505,7 @@ struct gasneti_pshmnet {
                 (gasneti_atomic_val_t)((uintptr_t)(addr) - (uintptr_t)gasnetc_pshmnet_region))
 #if PLATFORM_COMPILER_PGI && PLATFORM_COMPILER_VERSION_LT(10,0,0)
 /* PGI 9.0-0 truncates the macro version to 32-bits! (older than 9.0 not tested) */
-GASNETI_ALWAYS_INLINE(gasneti_pshm_addr)
+GASNETI_INLINE(gasneti_pshm_addr)
 void * gasneti_pshm_addr(uintptr_t offset) {
   gasneti_assert(offset);
   return (void*)(offset + (uintptr_t)gasnetc_pshmnet_region);
@@ -589,17 +577,32 @@ gasneti_pshmnet_init(void *region, size_t regionlen, gasneti_pshm_rank_t pshmnod
   /* make sure that our max buffer size fits all possible AMs */
   gasneti_assert(sizeof(gasneti_AMPSHM_maxmsg_t) <= GASNETI_PSHMNET_MAX_PAYLOAD);
 
+  gasneti_assert((offsetof(gasneti_AMPSHM_medmsg_t, mediumdata) % 4) == 0);
+
+  // Check GASNETC_MAX_MEDIUM_PSHM_DFLTMAX (if used) provides tight fit
+  #if (PLATFORM_OS_LINUX || PLATFORM_OS_DARWIN) && \
+      (PLATFORM_ARCH_X86 || PLATFORM_ARCH_X86_64)
+    // Arbitrary choice of frequently-tested ABIs known to provide tight fit
+    gasneti_assert((GASNETC_MAX_MEDIUM_PSHM != GASNETC_MAX_MEDIUM_PSHM_DFLTMAX) || \
+                   (sizeof(gasneti_pshmnet_allocator_block_t) == 65536));
+  #else
+    // Other ABIs may have less restrictive alignments (allow 16-byte slack)
+    gasneti_assert((GASNETC_MAX_MEDIUM_PSHM != GASNETC_MAX_MEDIUM_PSHM_DFLTMAX) || \
+                   ((sizeof(gasneti_pshmnet_allocator_block_t) <= 65536) && \
+                    (sizeof(gasneti_pshmnet_allocator_block_t) >= 65536 - 16)));
+  #endif
+
   szpernode = gasneti_pshmnet_memory_needed_pernode(pshmnodes);
   szonce = gasneti_pshmnet_memory_needed_once(pshmnodes);
 
   if (regionlen < (szonce + szpernode * pshmnodes))
     gasneti_fatalerror("Internal error: not enough memory for pshmnet: \n"
-                       " given %lu effective bytes, but need %lu", 
-                       (unsigned long)regionlen, (unsigned long)(szonce + szpernode * pshmnodes));
+                       " given %"PRIuPTR" effective bytes, but need %"PRIuPTR, 
+                       (uintptr_t)regionlen, (uintptr_t)(szonce + szpernode * pshmnodes));
 
   vnet = gasneti_malloc(sizeof(gasneti_pshmnet_t));
   vnet->nodecount = pshmnodes;
-#if GASNET_PAR
+#if GASNET_PAR || GASNETI_CONDUIT_THREADS
   gasneti_mutex_init(&vnet->lock);
 #endif
 
@@ -661,7 +664,7 @@ void gasneti_pshmnet_deliver_send_buffer(gasneti_pshmnet_t *vnet, void *buf,
   }
 }
 
-GASNETI_ALWAYS_INLINE(gasneti_pshmnet_queue_peek)
+GASNETI_INLINE(gasneti_pshmnet_queue_peek)
 int gasneti_pshmnet_queue_peek(const gasneti_pshmnet_queue_t * const q)
 {
   return q->shead || q->head;
@@ -674,7 +677,7 @@ int gasneti_pshmnet_recv(gasneti_pshmnet_t *vnet, void **pbuf, size_t *psize,
   gasneti_pshmnet_payload_t *p = NULL;
   gasneti_pshmnet_queue_t *q = vnet->my_queue;
 
-#if GASNET_PAR
+#if GASNET_PAR || GASNETI_CONDUIT_THREADS
   if (gasneti_pshmnet_queue_peek(q)) {
     gasneti_mutex_lock(&vnet->lock);
 #endif
@@ -699,7 +702,7 @@ int gasneti_pshmnet_recv(gasneti_pshmnet_t *vnet, void **pbuf, size_t *psize,
       }
       q->shead = next;
     }
-#if GASNET_PAR
+#if GASNET_PAR || GASNETI_CONDUIT_THREADS
     gasneti_mutex_unlock(&vnet->lock);
   }
 #endif
@@ -755,7 +758,10 @@ void gasneti_pshmnet_bootstrapBarrier(void)
   gasneti_assert_always(target < GASNETI_PSHM_BSB_LIMIT); /* Die if we were ever to reach the limit */
 
   gasneti_waitwhile((curr = gasneti_atomic_read(&gasneti_pshm_info->bootstrap_barrier_gen, 0)) < target);
-  if_pf (curr >= GASNETI_PSHM_BSB_LIMIT) gasnet_exit(1);
+  if_pf (curr >= GASNETI_PSHM_BSB_LIMIT) {
+    if (gasnetc_pshm_abort_callback) gasnetc_pshm_abort_callback();
+    gasnet_exit(1);
+  }
 
   generation = target;
 }
@@ -765,30 +771,95 @@ void gasneti_pshmnet_bootstrapBarrier(void)
  * they are potentially blocked in gasneti_pshmnet_bootstrapBarrier().
  * These DO NOT nest (but there is no checking to ensure that).
  ******************************************************************************/
-static gasneti_sighandlerfn_t gasneti_prev_abort_handler = NULL;
+static struct {
+  int sig;
+  gasneti_sighandlerfn_t old_hand;
+} gasneti_pshm_catch_signals[] = {
+#ifdef SIGABRT
+  { SIGABRT, NULL },
+#endif
+#ifdef SIGILL
+  { SIGILL,  NULL },
+#endif
+#ifdef SIGSEGV
+  { SIGSEGV, NULL },
+#endif
+#ifdef SIGBUS
+  { SIGBUS,  NULL },
+#endif
+#ifdef SIGFPE
+  { SIGFPE,  NULL },
+#endif
+#ifdef SIGINT
+  { SIGINT,  NULL },
+#endif
+#ifdef SIGTERM
+  { SIGTERM, NULL },
+#endif
+#ifdef SIGQUIT
+  { SIGQUIT, NULL },
+#endif
+#ifdef SIGPIPE
+  { SIGPIPE, NULL },
+#endif
+#ifdef SIGHUP
+  { SIGHUP,  NULL },
+#endif
+  { 0, NULL }
+};
 
 static void gasneti_pshm_abort_handler(int sig) {
+  // Invoke callback, if any
+  if (gasnetc_pshm_abort_callback) gasnetc_pshm_abort_callback();
+
+  // Force others to exit from barrier:
   gasneti_atomic_set(&gasneti_pshm_info->bootstrap_barrier_gen, GASNETI_PSHM_BSB_LIMIT, 0);
 
-  gasneti_reghandler(SIGABRT, gasneti_prev_abort_handler);
+  // Best-effort message if this is not due to gasneti_fatalerror()
+  if (sig != SIGABRT) {
+    const char msg1[] = "*** FATAL ERROR: fatal ";
+    const char msg2[] = " while mapping shared memory\n";
+    const char *signame = gasnett_signame_fromval(sig);
+    if (!signame) signame = "signal";
+    char msg[128] = { '\0', };
+    gasneti_assert(strlen(msg1) + strlen(signame) + strlen(msg2) + 1 <= sizeof(msg));
+    strcat(strcat(strcat(msg, msg1), signame), msg2);
+    int ignore = write(STDERR_FILENO, msg, strlen(msg));
+  }
+
+  // Reraise the signal
+  for (int i = 0; gasneti_pshm_catch_signals[i].sig; ++i) {
+    if (gasneti_pshm_catch_signals[i].sig != sig) continue;
+    gasneti_reghandler(sig, gasneti_pshm_catch_signals[i].old_hand);
+    break;
+  }
 #if HAVE_SIGPROCMASK /* Is this ever NOT the case? */
   { sigset_t new_set, old_set;
     sigemptyset(&new_set);
-    sigaddset(&new_set, SIGABRT);
+    sigaddset(&new_set, sig);
     sigprocmask(SIG_UNBLOCK, &new_set, &old_set);
   }
 #endif
-  raise(SIGABRT);
+  raise(sig);
 }
 
-void gasneti_pshm_cs_enter(void)
+void gasneti_pshm_cs_enter(void (*callback)(void))
 {
-  gasneti_prev_abort_handler = gasneti_reghandler(SIGABRT, &gasneti_pshm_abort_handler);
+  gasnetc_pshm_abort_callback = callback;
+  for (int i = 0; gasneti_pshm_catch_signals[i].sig; ++i) {
+    gasneti_pshm_catch_signals[i].old_hand =
+      gasneti_reghandler(gasneti_pshm_catch_signals[i].sig,
+                         &gasneti_pshm_abort_handler);
+  }
 }
 
 void gasneti_pshm_cs_leave(void)
 {
-  gasneti_reghandler(SIGABRT, gasneti_prev_abort_handler);
+  gasnetc_pshm_abort_callback = NULL;
+  for (int i = 0; gasneti_pshm_catch_signals[i].sig; ++i) {
+    gasneti_reghandler(gasneti_pshm_catch_signals[i].sig,
+                       gasneti_pshm_catch_signals[i].old_hand);
+  }
 }
 
 /******************************************************************************
@@ -1204,6 +1275,7 @@ int gasnetc_AMPSHM_ReqRepGeneric(int category, int isReq, gasnet_node_t dest,
     if_pf (msg == NULL) {
       /* Grow the free pool with buffers sized and aligned for the largest Medium */
       void *tmp = gasneti_malloc(sizeof(gasneti_AMPSHM_medmsg_t)+7);
+      gasneti_leak(tmp);
       uintptr_t offset = (uintptr_t)GASNETI_AMPSHM_MSG_MED_DATA(tmp) & 7;
       /* Align the (macro-adjusted) Medium payload field, not the msg itself */
       msg = (void*)((uintptr_t)tmp + (offset ? (8-offset) : 0));
