@@ -44,7 +44,7 @@
 /* In gasnet_mmap.c */
 #define GASNETI_PSHM_UNIQUE_LEN 6
 
-extern void *gasneti_mmap_vnet(uintptr_t segsize, gasneti_bootstrapExchangefn_t exchangefn);
+extern void *gasneti_mmap_vnet(uintptr_t segsize, gasneti_bootstrapBroadcastfn_t localbcastfn);
 extern void gasneti_unlink_vnet(void);
 
 /* Virtual network between processes within a shared
@@ -57,7 +57,10 @@ typedef struct gasneti_pshmnet gasneti_pshmnet_t;
 
 /* Initialize pshm request and reply networks given a conduit-specific exchange function.
    Returns pointer to shared memory of length "aux_sz" available for conduit-specific use */
-extern void *gasneti_pshm_init(gasneti_bootstrapExchangefn_t exchangefn, size_t aux_sz);
+extern void *gasneti_pshm_init(gasneti_bootstrapBroadcastfn_t localbcastfn, size_t aux_sz);
+
+/* Unmap all pshm segments */
+extern void gasneti_pshm_fini(void);
 
 /*  PSHMnets needed for PSHM active messages.
  *
@@ -208,7 +211,7 @@ void gasneti_pshmnet_bootstrapGather(gasneti_pshmnet_t *vnet, void *src,
 /* "critical sections" in which we notify peers if we abort() while
  * they are potentially blocked in gasneti_pshmnet_bootstrapBarrier().
   */
-extern void gasneti_pshm_cs_enter(void);
+extern void gasneti_pshm_cs_enter(void (*callback)(void));
 extern void gasneti_pshm_cs_leave(void);
 
 /* returns the maximum size payload that pshmnet can offer.  This is the
@@ -295,7 +298,7 @@ int gasneti_AMPSHM_ReplyGeneric(int category, gasnet_token_t token,
                                 va_list argptr) 
 {
   int retval;
-  gasnet_node_t sourceid;
+  gasnet_node_t sourceid = 0; // init to avoid a maybe-uninit warning on gcc -O3 -Wall
   gasneti_assert(gasnetc_token_is_pshm(token));
   gasnetc_AMGetMsgSource(token, &sourceid);
   gasneti_assert(gasneti_pshm_in_supernode(sourceid));
