@@ -11,8 +11,6 @@
 #ifndef _GASNET_SYNCOPS_H
 #define _GASNET_SYNCOPS_H
 
-GASNETI_BEGIN_EXTERNC
-
 /* ------------------------------------------------------------------------------------ */
 
 /* 
@@ -688,6 +686,9 @@ gasneti_atomic_val_t gasneti_semaphore_trydown_partial_SEQ(gasneti_semaphore_t_S
  * pointer, we prohibit any use of the LIFO in which a former element becomes unmapped
  * unless one can ensure that no partially-completed POP could be referencing the
  * object.
+ *
+ * NOTE: Because of the specifics of the representation, it is not permitted to copy
+ * a gasneti_lifo_head_t by value.
  */
 
 
@@ -696,11 +697,12 @@ gasneti_atomic_val_t gasneti_semaphore_trydown_partial_SEQ(gasneti_semaphore_t_S
   /* No threads, so we use the mutex code that compiles away. */
 #elif defined(GASNETI_USE_GENERIC_ATOMICOPS)
   /* If using mutexes, then just use the mutex code */
-#elif PLATFORM_ARCH_POWERPC && GASNETI_HAVE_ATOMIC_PTR_CAS
+#elif PLATFORM_ARCH_POWERPC && GASNETI_HAVE_ATOMIC_PTR_CAS && \
+      (GASNETI_HAVE_GCC_ASM || GASNETI_HAVE_XLC_ASM)
   /* Among the platforms we currently support, PPC is unique in having an LL/SC
    * construct which allows a load between the LL and the SC.
    */
-  #if PLATFORM_COMPILER_GNU
+  #if GASNETI_HAVE_GCC_ASM && !(PLATFORM_COMPILER_XLC && GASNETI_HAVE_XLC_ASM)
     typedef struct {
       /* Ensure list head pointer is the only item on its cache line.
        * This prevents a live-lock which would result if a list element fell
@@ -775,7 +777,7 @@ gasneti_atomic_val_t gasneti_semaphore_trydown_partial_SEQ(gasneti_semaphore_t_S
     }
     #define GASNETI_LIFO_INITIALIZER_PAR {{0,}, gasneti_atomic_ptr_init(0), {0,}}
     #define GASNETI_HAVE_ARCH_LIFO	1
-  #elif PLATFORM_COMPILER_XLC
+  #elif GASNETI_HAVE_XLC_ASM
     typedef struct {
       /* Ensure list head pointer is the only item on its cache line.
        * This prevents a live-lock which would result if a list element fell
@@ -899,7 +901,12 @@ gasneti_atomic_val_t gasneti_semaphore_trydown_partial_SEQ(gasneti_semaphore_t_S
 
     GASNETI_INLINE(_gasneti_lifo_push)
     void _gasneti_lifo_push(gasneti_lifo_head_t_PAR *p, void **newhead, void **tail) {
-      uintptr_t tag, oldhead;
+    #if (PLATFORM_COMPILER_PGI && PLATFORM_COMPILER_VERSION_GE(16,4,0)) /* XXX: no end version */
+      volatile uintptr_t tag; /* See GASNet bug #3324 */
+    #else
+      uintptr_t tag;
+    #endif
+      uintptr_t oldhead;
       gasneti_atomic64_t *head = (gasneti_atomic64_t *)_GASNETI_LIFO_ALIGN(p);
       gasneti_atomic128_t *head_and_tag = (gasneti_atomic128_t *)head;
       do {
@@ -1195,6 +1202,5 @@ void *gasneti_lifo_next(void *elem) {
 #define gasneti_lifo_push_many       gasneti_cons_lifo(_GASNETI_PARSEQ,push_many)
 
 /* ------------------------------------------------------------------------------------ */
-GASNETI_END_EXTERNC
 
 #endif
