@@ -13,6 +13,10 @@
          include gasnet.h before gasnet_tools.h 
 #endif
 
+#ifdef __cplusplus
+  extern "C" { // cannot use GASNETI_BEGIN_EXTERNC here due to a header dependency cycle
+#endif
+
 /* Recognized definitions:
    GASNETT_THREAD_SAFE - may be defined by client to enable thread-safety. Thread-safety is also
       auto-enabled by _REENTRANT, _THREAD_SAFE and pthread.h
@@ -47,13 +51,16 @@
 #include <gasnet_config.h>
 
 /* public spec version numbers */
-#define GASNETT_SPEC_VERSION_MAJOR GASNETIT_SPEC_VERSION_MAJOR
-#define GASNETT_SPEC_VERSION_MINOR GASNETIT_SPEC_VERSION_MINOR
+#define GASNETT_SPEC_VERSION_MAJOR GASNETI_TOOLS_SPEC_VERSION_MAJOR
+#define GASNETT_SPEC_VERSION_MINOR GASNETI_TOOLS_SPEC_VERSION_MINOR
 #define GASNETT_RELEASE_VERSION_MAJOR GASNET_RELEASE_VERSION_MAJOR
 #define GASNETT_RELEASE_VERSION_MINOR GASNET_RELEASE_VERSION_MINOR
 #define GASNETT_RELEASE_VERSION_PATCH GASNET_RELEASE_VERSION_PATCH
 
 #include <gasnet_basic.h>
+
+GASNETI_BEGIN_NOWARN
+
 #include <gasnet_toolhelp.h>
 
 /* allow conduit-specific tool helpers (eg elan timers) */
@@ -71,8 +78,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-GASNETI_BEGIN_EXTERNC
-
 /* ------------------------------------------------------------------------------------ */
 /* basic purely syntactic features */
 #define GASNETT_IDENT(identName, identText) GASNETI_IDENT(identName, identText)
@@ -91,8 +96,8 @@ GASNETI_BEGIN_EXTERNC
 #define GASNETT_SYSTEM_TUPLE     GASNETI_SYSTEM_TUPLE
 #define GASNETT_CONFIGURE_ARGS   GASNETI_CONFIGURE_ARGS
 
-#ifdef GASNETI_CONFIGURE_MISMATCH
-#define GASNETT_CONFIGURE_MISMATCH 1  /* indicate configure and build compilers mismatch */
+#if !GASNETI_COMPILER_IS_CC
+#define GASNETT_CONFIGURE_MISMATCH 1  /* indicate build compiler mismatches configure $CC */
 #endif
 
 /* platform identifiers for the compiler detected at *configure*-time */
@@ -103,8 +108,6 @@ GASNETI_BEGIN_EXTERNC
 
 /* various configure-detected C compiler features available in only some compilers */
 #define GASNETT_INLINE                  GASNETI_INLINE
-#define GASNETT_ALWAYS_INLINE           GASNETI_ALWAYS_INLINE
-#define GASNETT_PLEASE_INLINE           GASNETI_PLEASE_INLINE
 #define GASNETT_NEVER_INLINE            GASNETI_NEVER_INLINE
 #define GASNETT_RESTRICT                GASNETI_RESTRICT
 #define GASNETT_USED                    GASNETI_USED
@@ -116,6 +119,8 @@ GASNETI_BEGIN_EXTERNC
 #define GASNETT_PUREP                   GASNETI_PUREP
 #define GASNETT_CONST                   GASNETI_CONST
 #define GASNETT_CONSTP                  GASNETI_CONSTP
+#define GASNETT_HOT                     GASNETI_HOT
+#define GASNETT_COLD                    GASNETI_COLD
 #define GASNETT_DEPRECATED              GASNETI_DEPRECATED
 #define GASNETT_WARN_UNUSED_RESULT      GASNETI_WARN_UNUSED_RESULT
 #define GASNETT_FORMAT_PRINTF           GASNETI_FORMAT_PRINTF
@@ -126,11 +131,12 @@ GASNETI_BEGIN_EXTERNC
 #define GASNETT_BEGIN_EXTERNC           GASNETI_BEGIN_EXTERNC
 #define GASNETT_END_EXTERNC             GASNETI_END_EXTERNC
 #define GASNETT_EXTERNC                 GASNETI_EXTERNC
-#define GASNETT_COMMON_EXTERN           GASNETI_COMMON_EXTERN
 #define GASNETT_TENTATIVE_EXTERN        GASNETI_TENTATIVE_EXTERN
 
 #define gasnett_constant_p              gasneti_constant_p
 
+#define gasnett_unreachable             gasneti_unreachable
+
 /* ------------------------------------------------------------------------------------ */
 /* portable memory barriers */
 
@@ -143,6 +149,11 @@ GASNETI_BEGIN_EXTERNC
 #define gasnett_compiler_fence()     gasneti_compiler_fence()
 #define GASNETT_MEMBAR_CONFIG        GASNETI_MEMBAR_CONFIG
 
+/* ------------------------------------------------------------------------------------ */
+/* microsecond-resolution sleep */
+
+#define gasnett_nsleep(ns_delay)     gasneti_nsleep(ns_delay)
+
 /* ------------------------------------------------------------------------------------ */
 /* portable high-performance, low-overhead timers */
 
@@ -382,6 +393,20 @@ extern const char *gasnett_performance_warning_str(void);
 #define gasnett_mutex_assertlocked    gasneti_mutex_assertlocked
 #define gasnett_mutex_assertunlocked  gasneti_mutex_assertunlocked
 
+#define gasnett_rwlock_t              gasneti_rwlock_t
+#define gasnett_rwlock_init           gasneti_rwlock_init
+#define gasnett_rwlock_destroy        gasneti_rwlock_destroy
+#define gasnett_rwlock_rdlock         gasneti_rwlock_rdlock
+#define gasnett_rwlock_wrlock         gasneti_rwlock_wrlock
+#define gasnett_rwlock_tryrdlock      gasneti_rwlock_tryrdlock
+#define gasnett_rwlock_trywrlock      gasneti_rwlock_trywrlock
+#define gasnett_rwlock_unlock         gasneti_rwlock_unlock
+#define GASNETT_RWLOCK_INITIALIZER    GASNETI_RWLOCK_INITIALIZER
+#define gasnett_rwlock_assertlocked   gasneti_rwlock_assertlocked 
+#define gasnett_rwlock_assertrdlocked gasneti_rwlock_assertrdlocked
+#define gasnett_rwlock_assertwrlocked gasneti_rwlock_assertwrlocked
+#define gasnett_rwlock_assertunlocked gasneti_rwlock_assertunlocked
+
 #define gasnett_cond_t               gasneti_cond_t
 #define gasnett_cond_init            gasneti_cond_init
 #define gasnett_cond_destroy         gasneti_cond_destroy
@@ -434,9 +459,9 @@ static void _gasnett_trace_printf_noop(const char *_format, ...)) {
 }
 #ifdef GASNET_TRACE
   GASNETT_FORMAT_PRINTF_FUNCPTR(_gasnett_trace_printf,1,2,
-  GASNETT_TENTATIVE_EXTERN void (*_gasnett_trace_printf)(const char *format, ...));
+  GASNETT_TENTATIVE_EXTERN void (*_gasnett_trace_printf)(const char *_format, ...));
   GASNETT_FORMAT_PRINTF_FUNCPTR(_gasnett_trace_printf_force,1,2,
-  GASNETT_TENTATIVE_EXTERN void (*_gasnett_trace_printf_force)(const char *format, ...));
+  GASNETT_TENTATIVE_EXTERN void (*_gasnett_trace_printf_force)(const char *_format, ...));
   #if PLATFORM_COMPILER_PGI /* bug 1703 - workaround a PGI bug using Gnu-style variadic macros which PGI supports */
     #define GASNETT_TRACE_PRINTF(args...) \
             (_gasnett_trace_printf ? _gasnett_trace_printf(args) : _gasnett_trace_printf_noop(args))
@@ -456,7 +481,7 @@ static void _gasnett_trace_printf_noop(const char *_format, ...)) {
     #define GASNETT_TRACE_GET_TRACELOCAL()        GASNETI_TRACE_GET_TRACELOCAL()
     #define GASNETT_TRACE_SET_TRACELOCAL(newval)  GASNETI_TRACE_SET_TRACELOCAL(newval)
   #else
-    GASNETT_TENTATIVE_EXTERN int (*_gasnett_trace_enabled)(char tracecat);
+    GASNETT_TENTATIVE_EXTERN int (*_gasnett_trace_enabled)(char _tracecat);
     #define GASNETT_TRACE_ENABLED       (_gasnett_trace_enabled?_gasnett_trace_enabled('H'):0)
     #define GASNETT_TRACE_GETMASK()               ""
     #define GASNETT_TRACE_SETMASK(mask)           ((void)0)
@@ -493,14 +518,14 @@ static void _gasnett_trace_printf_noop(const char *_format, ...)) {
 #if defined(_INCLUDED_GASNET_H) 
   /* these tools ONLY available when linking a libgasnet.a */
   #ifdef HAVE_MMAP
-    extern void *gasneti_mmap(uintptr_t segsize);
+    extern void *gasneti_mmap(uintptr_t _segsize);
     #define gasnett_mmap(sz) gasneti_mmap(sz)
   #else
     #define gasnett_mmap(sz) gasnett_fatalerror("gasnett_mmap not available")
   #endif
-  extern int gasneti_run_diagnostics(int iters, int threadcnt, 
-                                     const char *testsections, gasnet_seginfo_t const *seginfo);
-  extern void gasneti_diagnostic_gethandlers(gasnet_handlerentry_t **htable, int *htable_cnt);
+  extern int gasneti_run_diagnostics(int _iters, int _threadcnt, 
+                                     const char *_testsections, gasnet_seginfo_t const *_seginfo);
+  extern void gasneti_diagnostic_gethandlers(gasnet_handlerentry_t **_htable, int *_htable_cnt);
   #define gasnett_run_diagnostics gasneti_run_diagnostics
   #define gasnett_diagnostic_gethandlers gasneti_diagnostic_gethandlers
 
@@ -525,9 +550,6 @@ static void _gasnett_trace_printf_noop(const char *_format, ...)) {
     #define gasnett_getheapstats(pstat)   gasneti_getheapstats(pstat)
   #endif
 
-  #define gasnett_malloc_aligned(align,sz) gasneti_malloc_aligned((align),(sz))
-  #define gasnett_free_aligned(ptr)        gasneti_free_aligned(ptr)
-
   /* VIS string formatting */
   #define gasnett_format_memveclist_bufsz gasneti_format_memveclist_bufsz 
   #define gasnett_format_memveclist       gasneti_format_memveclist       
@@ -583,6 +605,9 @@ static void _gasnett_trace_printf_noop(const char *_format, ...)) {
 #endif
 
 #define GASNETT_LINKCONFIG_IDIOTCHECK(name) _CONCAT(gasnett_linkconfig_idiotcheck_,name)
+extern int GASNETT_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MAJOR_,GASNET_RELEASE_VERSION_MAJOR));
+extern int GASNETT_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MINOR_,GASNET_RELEASE_VERSION_MINOR));
+extern int GASNETT_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_PATCH_,GASNET_RELEASE_VERSION_PATCH));
 extern int GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_THREAD_MODEL);
 extern int GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_DEBUG_CONFIG);
 extern int GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_PTR_CONFIG);
@@ -594,12 +619,19 @@ extern int GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_ATOMIC32_CONFIG);
 extern int GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_ATOMIC64_CONFIG);
 #endif
 static int *gasnett_linkconfig_idiotcheck(void);
-static void *_gasnett_linkconfig_idiotcheck = (void *)&gasnett_linkconfig_idiotcheck;
+#if !PLATFORM_COMPILER_TINY /* avoid a tinyc bug */
+  #define GASNETI_IDIOTCHECK_RECURSIVE_REFERENCE 1
+  static int *(*_gasnett_linkconfig_idiotcheck)(void) = &gasnett_linkconfig_idiotcheck;
+#endif
 GASNETT_USED
 static int *gasnett_linkconfig_idiotcheck(void) 
 {
-  static int val;
-  val +=  GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_THREAD_MODEL)
+  static int _val;
+  _val +=  
+        + GASNETT_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MAJOR_,GASNET_RELEASE_VERSION_MAJOR))
+        + GASNETT_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_MINOR_,GASNET_RELEASE_VERSION_MINOR))
+        + GASNETT_LINKCONFIG_IDIOTCHECK(_CONCAT(RELEASE_PATCH_,GASNET_RELEASE_VERSION_PATCH))
+        + GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_THREAD_MODEL)
         + GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_DEBUG_CONFIG)
         + GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_PTR_CONFIG)
         + GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_TIMER_CONFIG)
@@ -610,13 +642,16 @@ static int *gasnett_linkconfig_idiotcheck(void)
         + GASNETT_LINKCONFIG_IDIOTCHECK(GASNETT_ATOMIC64_CONFIG)
       #endif
         ;
-  if (_gasnett_linkconfig_idiotcheck != (void *)&gasnett_linkconfig_idiotcheck)
-    val += ((int(*)(void))_gasnett_linkconfig_idiotcheck)();
-  return &val;
+  #if GASNETI_IDIOTCHECK_RECURSIVE_REFERENCE
+  if (_gasnett_linkconfig_idiotcheck == &gasnett_linkconfig_idiotcheck)
+    _val += *(*_gasnett_linkconfig_idiotcheck)();
+  #endif
+  return &_val;
 }
 
 /* ------------------------------------------------------------------------------------ */
 
+GASNETI_END_NOWARN
 GASNETI_END_EXTERNC
 
 #undef _IN_GASNET_TOOLS_H
