@@ -7,6 +7,7 @@
 #include <gasnet_internal.h>
 #include <gasnet_extended_internal.h>
 #include <gasnet_gemini.h>
+#include <gasnet_coll.h>
 
 static const gasnete_eopaddr_t EOPADDR_NIL = { { 0xFF, 0xFF } };
 extern void _gasnete_iop_check(gasnete_iop_t *iop) { gasnete_iop_check(iop); }
@@ -48,7 +49,7 @@ extern void _gasnete_iop_check(gasnete_iop_t *iop) { gasnete_iop_check(iop); }
 
 /*  allocate more eops */
 GASNETI_NEVER_INLINE(gasnete_eop_alloc,
-static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
+static void gasnete_eop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_eopaddr_t addr;
     int bufidx = thread->eop_num_bufs;
     gasnete_eop_t *buf;
@@ -134,7 +135,7 @@ static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  allocate a new iop */
 GASNETI_NEVER_INLINE(gasnete_iop_alloc,
-static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
+static gasnete_iop_t *gasnete_iop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_iop_t *iop = (gasnete_iop_t *)gasneti_malloc(sizeof(gasnete_iop_t));
     gasneti_leak(iop);
     #if GASNET_DEBUG
@@ -151,7 +152,7 @@ static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  get a new op */
 static
-gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *_gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eopaddr_t head = thread->eop_free;
   if_pf (gasnete_eopaddr_isnil(head)) {
     gasnete_eop_alloc(thread);
@@ -174,7 +175,7 @@ gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new op AND mark it in flight */
 GASNETI_INLINE(gasnete_eop_new)
-gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eop_t *eop = _gasnete_eop_new(thread);
 #if GASNETE_EOP_COUNTED
   eop->initiated_cnt++;
@@ -184,7 +185,7 @@ gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new iop */
 static
-gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
+gasnete_iop_t *gasnete_iop_new(gasneti_threaddata_t * const thread) {
   gasnete_iop_t *iop = thread->iop_free;
   if_pt (iop) {
     thread->iop_free = iop->next;
@@ -209,7 +210,7 @@ gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
 /*  query an eop for completeness */
 static
 int gasnete_eop_isdone(gasnete_eop_t *eop) {
-  gasneti_assert(eop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(eop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_eop_check(eop);
   return GASNETE_EOP_DONE(eop);
 }
@@ -217,7 +218,7 @@ int gasnete_eop_isdone(gasnete_eop_t *eop) {
 /*  query an iop for completeness - this means both puts and gets */
 static
 int gasnete_iop_isdone(gasnete_iop_t *iop) {
-  gasneti_assert(iop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(iop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_iop_check(iop);
   return (GASNETE_IOP_CNTDONE(iop,get) && GASNETE_IOP_CNTDONE(iop,put));
 }
@@ -240,9 +241,9 @@ void gasnete_op_markdone(gasnete_op_t *op, int isget) {
 /*  free an eop */
 static
 void gasnete_eop_free(gasnete_eop_t *eop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
+  gasneti_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
   gasnete_eopaddr_t addr = eop->addr;
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_eop_check(eop);
   gasneti_assert(GASNETE_EOP_DONE(eop));
 #if GASNET_DEBUG
@@ -255,8 +256,8 @@ void gasnete_eop_free(gasnete_eop_t *eop) {
 /*  free an iop */
 static
 void gasnete_iop_free(gasnete_iop_t *iop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_iop_check(iop);
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,get));
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,put));
@@ -279,7 +280,7 @@ void gasnete_iop_free(gasnete_iop_t *iop) {
 
 /* ensure thread cleanup uses our custom for valget handles */
 #define GASNETE_VALGET_FREEALL(thread) gasnete_valget_freeall(thread)
-static void gasnete_valget_freeall(gasnete_threaddata_t *thread);
+static void gasnete_valget_freeall(gasneti_threaddata_t *thread);
 
 #include "gasnet_extended_common.c"
 
@@ -297,7 +298,6 @@ static void gasnete_check_config(void) {
 }
 
 extern void gasnete_init(void) {
-  GASNETI_UNUSED_UNLESS_DEBUG
   static int firstcall = 1;
   GASNETI_TRACE_PRINTF(C,("gasnete_init()"));
   gasneti_assert(firstcall); /*  make sure we haven't been called before */
@@ -307,11 +307,11 @@ extern void gasnete_init(void) {
 
   gasneti_assert(gasneti_nodes >= 1 && gasneti_mynode < gasneti_nodes);
 
-  { gasnete_threaddata_t *threaddata = NULL;
+  { gasneti_threaddata_t *threaddata = NULL;
     gasnete_eop_t *eop = NULL;
     #if GASNETI_MAX_THREADS > 1
       /* register first thread (optimization) */
-      threaddata = gasnete_mythread(); 
+      threaddata = _gasneti_mythread_slow(); 
     #else
       /* register only thread (required) */
       threaddata = gasnete_new_threaddata();
@@ -332,12 +332,12 @@ extern void gasnete_init(void) {
 
 /* ------------------------------------------------------------------------------------ */
 /* GASNET-Internal OP Interface */
-gasneti_eop_t *gasneti_eop_create(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_eop_t *op = gasnete_eop_new(GASNETE_MYTHREAD);
+gasneti_eop_t *gasneti_eop_create(GASNETI_THREAD_FARG_ALONE) {
+  gasnete_eop_t *op = gasnete_eop_new(GASNETI_MYTHREAD);
   return (gasneti_eop_t *)op;
 }
-gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t * const op = mythread->current_iop;
   gasnete_iop_check(op);
   if (isget) op->initiated_get_cnt += noperations;
@@ -422,23 +422,7 @@ gasnete_get_bulk_inner(void *dest, gasnet_node_t node, void *src, size_t nbytes,
 {
   size_t chunksz;
 
-#if GASNETC_GNI_FIREHOSE
-  /* Let registration boundaries drive the xfer for out-of-segment case */
-  if (gasnetc_use_firehose && !gasneti_in_segment(gasneti_mynode, dest, nbytes)) {
-    gasneti_assert(nbytes);
-    do {
-      gasnetc_post_descriptor_t *gpd = gasnete_cntr_gpd(initiated_p, completed_p GASNETC_DIDX_PASS);
-      chunksz = gasnetc_rdma_get_fh(node, dest, src, nbytes, gpd);
-      dest = (char *) dest + chunksz;
-      src  = (char *) src  + chunksz;
-      nbytes -= chunksz;
-    } while (nbytes);
-    return;
-  }
-  chunksz = GC_MAXRDMA_IN;
-#else
   chunksz = gasneti_in_segment(gasneti_mynode, dest, nbytes) ? GC_MAXRDMA_IN : GC_MAXRDMA_OUT;
-#endif
 
   if (nbytes > 2*chunksz) {
     /* If need more than 2 chunks, then size first one to achieve page alignment of remainder */
@@ -455,7 +439,6 @@ retry:
 
     if_pf (tmp != xfer_len) { /* MemRegister failed */
       gasneti_assert(chunksz == GC_MAXRDMA_OUT); /* out-of-seg and not looping */
-      gasneti_assert(!gasnetc_use_firehose);
       chunksz = tmp; /* Will avoid more MemRegister failures */
       goto retry;
     }
@@ -528,6 +511,52 @@ gasnete_get_bulk_unaligned(void *dest, gasnet_node_t node, void *src, size_t nby
   }
 }
 
+static void /* XXX: Inlining left to compiler's discretion */
+gasnete_put_inner(gasnet_node_t node, void *dest, void *src, size_t nbytes,
+                  unsigned int *initiated_lc,
+                  volatile unsigned int *completed_lc,
+                  gasneti_weakatomic_val_t * const initiated_p,
+                  gasneti_weakatomic_t * const completed_p
+                  GASNETC_DIDX_FARG)
+{
+  gasnetc_post_descriptor_t *gpd;
+  size_t chunksz;
+
+  chunksz = gasneti_in_segment(gasneti_mynode, src, nbytes) ? GC_MAXRDMA_IN : GC_MAXRDMA_OUT;
+
+  if (nbytes > 2*chunksz) {
+    /* If need more than 2 chunks, then size first one to achieve page alignment of remainder */
+    size_t tmp, xfer_len;
+retry:
+    xfer_len = chunksz - ((uintptr_t)src & (GASNETI_PAGESIZE-1));
+    gasneti_assert(xfer_len != 0);
+    gasneti_assert(xfer_len < nbytes);
+    gpd = gasnete_cntr_gpd(initiated_p, completed_p GASNETC_DIDX_PASS);
+    gpd->gpd_put_lc = (uint64_t) completed_lc;
+    tmp = gasnetc_rdma_put_lc(node, dest, src, xfer_len, initiated_lc, gpd);
+    dest = (char *) dest + tmp;
+    src  = (char *) src  + tmp;
+    nbytes -= tmp;
+
+    if_pf (tmp != xfer_len) { /* MemRegister failed */
+      gasneti_assert(chunksz == GC_MAXRDMA_OUT); /* out-of-seg and not looping */
+      chunksz = tmp; /* Will avoid more MemRegister failures */
+      goto retry;
+    }
+  }
+
+  gasneti_assert(nbytes);
+  do {
+    const size_t xfer_len = MIN(nbytes, chunksz);
+    gpd = gasnete_cntr_gpd(initiated_p, completed_p GASNETC_DIDX_PASS);
+    gpd->gpd_put_lc = (uint64_t) completed_lc;
+    chunksz = gasnetc_rdma_put_lc(node, dest, src, xfer_len, initiated_lc, gpd);
+    dest = (char *) dest + chunksz;
+    src  = (char *) src  + chunksz;
+    nbytes -= chunksz;
+  } while (nbytes);
+}
+
 static void /* XXX: Inlining left to compiler's discretion */
 gasnete_put_bulk_inner(gasnet_node_t node, void *dest, void *src, size_t nbytes,
                        gasneti_weakatomic_val_t * const initiated_p,
@@ -536,23 +565,7 @@ gasnete_put_bulk_inner(gasnet_node_t node, void *dest, void *src, size_t nbytes,
 {
   size_t chunksz;
 
-#if GASNETC_GNI_FIREHOSE
-  /* Let registration boundaries drive the xfer for out-of-segment case */
-  if (gasnetc_use_firehose && !gasneti_in_segment(gasneti_mynode, src, nbytes)) {
-    gasneti_assert(nbytes);
-    do {
-      gasnetc_post_descriptor_t *gpd = gasnete_cntr_gpd(initiated_p, completed_p GASNETC_DIDX_PASS);
-      chunksz = gasnetc_rdma_put_fh(node, dest, src, nbytes, gpd);
-      dest = (char *) dest + chunksz;
-      src  = (char *) src  + chunksz;
-      nbytes -= chunksz;
-    } while (nbytes);
-    return;
-  }
-  chunksz = GC_MAXRDMA_IN;
-#else
   chunksz = gasneti_in_segment(gasneti_mynode, src, nbytes) ? GC_MAXRDMA_IN : GC_MAXRDMA_OUT;
-#endif
 
   if (nbytes > 2*chunksz) {
     /* If need more than 2 chunks, then size first one to achieve page alignment of remainder */
@@ -569,7 +582,6 @@ retry:
 
     if_pf (tmp != xfer_len) { /* MemRegister failed */
       gasneti_assert(chunksz == GC_MAXRDMA_OUT); /* out-of-seg and not looping */
-      gasneti_assert(!gasnetc_use_firehose);
       chunksz = tmp; /* Will avoid more MemRegister failures */
       goto retry;
     }
@@ -600,10 +612,10 @@ retry:
      gasnete_memset_nb
 */
 
-extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,H);
+extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,H,dest,node,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_eop_t *eop = _gasnete_eop_new(mythread);
     GASNETC_DIDX_POST(mythread->domain_idx);
     gasneti_suspend_spinpollers();
@@ -617,46 +629,28 @@ extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void
   }
 }
 
-extern gasnet_handle_t gasnete_put_nb (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  gasnet_handle_t head_op = GASNET_INVALID_HANDLE;
-  gasnete_eop_t *tail_op;
-  const size_t max_tail = gasnetc_max_put_lc;
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
-  GASNETC_DIDX_POST(mythread->domain_idx);
-
-  GASNETI_CHECKPSHM_PUT(ALIGNED,H);
-
-  gasneti_suspend_spinpollers();
-
-  /* Non-blocking bulk put of "head" portion */
-  if (nbytes > max_tail) {
-    const size_t head_len = nbytes - max_tail;
-    gasnete_eop_t * const eop = _gasnete_eop_new(mythread);
-    gasnete_put_bulk_inner(node, dest, src, head_len, GASNETE_EOP_CNTRS(eop) GASNETC_DIDX_PASS);
-    head_op = (gasnet_handle_t) eop;
-    dest = (char *) dest + head_len;
-    src  = (char *) src  + head_len;
-    nbytes = max_tail;
+extern gasnet_handle_t gasnete_put_nb (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(ALIGNED,H,node,dest,src,nbytes);
+  {
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
+    gasnete_eop_t *eop = _gasnete_eop_new(mythread);
+    GASNETC_DIDX_POST(mythread->domain_idx);
+    unsigned int initiated_lc = 0;
+    volatile unsigned int completed_lc = 0;
+    gasneti_suspend_spinpollers();
+    gasnete_put_inner(node, dest, src, nbytes,
+                      &initiated_lc, &completed_lc,
+                      GASNETE_EOP_CNTRS(eop) GASNETC_DIDX_PASS);
+    gasneti_resume_spinpollers();
+    gasneti_polluntil(initiated_lc == completed_lc);
+    return (gasnet_handle_t) eop;
   }
-
-  /* Non-blocking non-bulk put of "tail" portion */
-  tail_op = _gasnete_eop_new(mythread);
-  gasnetc_rdma_put_lc(node, dest, src, nbytes,
-                      gasnete_cntr_gpd(GASNETE_EOP_CNTRS(tail_op) GASNETC_DIDX_PASS));
-
-  gasneti_resume_spinpollers();
-
-  /* Block for completion of head, if any */
-  gasnete_wait_syncnb(head_op);
-
-  /* return the tail_op */
-  return (gasnet_handle_t)tail_op;
 }
 
-extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
+extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_eop_t *eop = _gasnete_eop_new(mythread);
     GASNETC_DIDX_POST(mythread->domain_idx);
     gasneti_suspend_spinpollers();
@@ -679,7 +673,7 @@ GASNETI_INLINE(gasnete_op_try_free)
 int gasnete_op_try_free(gasnet_handle_t handle) {
   gasnete_op_t *op = (gasnete_op_t *)handle;
 
-  gasneti_assert(op->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(op->threadidx == _gasneti_mythread_slow()->threadidx);
   if_pt (OPTYPE(op) == OPTYPE_EXPLICIT) {
     gasnete_eop_t *eop = (gasnete_eop_t*)op;
 
@@ -777,10 +771,10 @@ extern int  gasnete_try_syncnb_all (gasnet_handle_t *phandle, size_t numhandles)
      gasnete_memset_nbi
 */
 
-extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const iop = mythread->current_iop;
     GASNETC_DIDX_POST(mythread->domain_idx);
     gasneti_suspend_spinpollers();
@@ -793,42 +787,27 @@ extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, siz
   }
 }
 
-extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
-  gasnete_iop_t * const tail_op = mythread->current_iop;
-  gasnet_handle_t head_op = GASNET_INVALID_HANDLE;
-  const size_t max_tail = gasnetc_max_put_lc;
-  GASNETC_DIDX_POST(mythread->domain_idx);
-
-  GASNETI_CHECKPSHM_PUT(ALIGNED,V);
-
-  gasneti_suspend_spinpollers();
-
-  /* Non-blocking bulk put of "head" portion */
-  if (nbytes > max_tail) {
-    const size_t head_len = nbytes - max_tail;
-    gasnete_eop_t * const eop = _gasnete_eop_new(GASNETE_MYTHREAD);
-    gasnete_put_bulk_inner(node, dest, src, head_len, GASNETE_EOP_CNTRS(eop) GASNETC_DIDX_PASS);
-    head_op = (gasnet_handle_t) eop;
-    dest = (char *) dest + head_len;
-    src  = (char *) src  + head_len;
-    nbytes = max_tail;
+extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(ALIGNED,V,node,dest,src,nbytes);
+  {
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
+    gasnete_iop_t * const iop = mythread->current_iop;
+    GASNETC_DIDX_POST(mythread->domain_idx);
+    unsigned int initiated_lc = 0;
+    volatile unsigned int completed_lc = 0;
+    gasneti_suspend_spinpollers();
+    gasnete_put_inner(node, dest, src, nbytes,
+                      &initiated_lc, &completed_lc,
+                      GASNETE_IOP_CNTRS(iop, put) GASNETC_DIDX_PASS);
+    gasneti_resume_spinpollers();
+    gasneti_polluntil(initiated_lc == completed_lc);
   }
-
-  /* Non-blocking non-bulk put of "tail" portion */
-  gasnetc_rdma_put_lc(node, dest, src, nbytes,
-                      gasnete_cntr_gpd(GASNETE_IOP_CNTRS(tail_op, put) GASNETC_DIDX_PASS));
-
-  gasneti_resume_spinpollers();
-
-  /* Block for completion of head, if any */
-  gasnete_wait_syncnb(head_op);
 }
 
-extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,V);
+extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,V,node,dest,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const iop = mythread->current_iop;
     GASNETC_DIDX_POST(mythread->domain_idx);
     gasneti_suspend_spinpollers();
@@ -843,13 +822,13 @@ extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, siz
   ===========================================================
 */
 
-extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_gets(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(OPTYPE(iop) == OPTYPE_IMPLICIT);
@@ -865,13 +844,13 @@ extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
   }
 }
 
-extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_puts(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(iop->next == NULL);
@@ -896,8 +875,8 @@ extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
 */
 /*  This implementation allows recursive access regions, although the spec does not require that */
 /*  operations are associated with the most immediately enclosing access region */
-extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = gasnete_iop_new(mythread); /*  push an iop  */
   GASNETI_TRACE_PRINTF(S,("BEGIN_NBI_ACCESSREGION"));
   #if GASNET_DEBUG
@@ -908,8 +887,8 @@ extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE
   mythread->current_iop = iop;
 }
 
-extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETI_THREAD_FARG_ALONE) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop; /*  pop an iop */
   GASNETI_TRACE_EVENT_VAL(S,END_NBI_ACCESSREGION,iop->initiated_get_cnt + iop->initiated_put_cnt);
   #if GASNET_DEBUG
@@ -927,53 +906,50 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
 */
 /* ------------------------------------------------------------------------------------ */
 
-#define gasnete_val_assign(_dst, _val) \
-    (*(gasnet_register_value_t *)(_dst) = (gasnet_register_value_t)(_val))
-
-extern void gasnete_put_val(gasnet_node_t node, void *dest, gasnet_register_value_t value, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUTVAL(V);
+extern void gasnete_put_val(gasnet_node_t node, void *dest, gasnet_register_value_t value, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUTVAL(V,node,dest,value,nbytes);
   {
-    GASNETC_DIDX_POST(GASNETE_MYTHREAD->domain_idx);
+    GASNETC_DIDX_POST(GASNETI_MYTHREAD->domain_idx);
     gasnetc_post_descriptor_t *gpd;
     volatile int done = 0;
     gasneti_suspend_spinpollers();
     gpd = gasnetc_alloc_post_descriptor(GASNETC_DIDX_PASS_ALONE);
     gpd->gpd_completion = (uintptr_t) &done;
     gpd->flags = GC_POST_COMPLETION_FLAG;
-    gasnete_val_assign(gpd->u.immediate, value);
-    gasnetc_rdma_put_buff(node, dest, GASNETE_STARTOFBITS(gpd->u.immediate, nbytes), nbytes, gpd);
+    gpd->u.put_val = value;
+    gasnetc_rdma_put_buff(node, dest, GASNETE_STARTOFBITS(&gpd->u.put_val, nbytes), nbytes, gpd);
     gasneti_resume_spinpollers();
     gasneti_polluntil(done);
   }
 }
 
-extern gasnet_handle_t gasnete_put_nb_val(gasnet_node_t node, void *dest, gasnet_register_value_t value, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUTVAL(H);
+extern gasnet_handle_t gasnete_put_nb_val(gasnet_node_t node, void *dest, gasnet_register_value_t value, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUTVAL(H,node,dest,value,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     GASNETC_DIDX_POST(mythread->domain_idx);
     gasnete_eop_t * const eop = _gasnete_eop_new(mythread);
     gasnetc_post_descriptor_t *gpd;
     gasneti_suspend_spinpollers();
     gpd = gasnete_cntr_gpd(GASNETE_EOP_CNTRS(eop) GASNETC_DIDX_PASS);
-    gasnete_val_assign(gpd->u.immediate, value);
-    gasnetc_rdma_put_buff(node, dest, GASNETE_STARTOFBITS(gpd->u.immediate, nbytes), nbytes, gpd);
+    gpd->u.put_val = value;
+    gasnetc_rdma_put_buff(node, dest, GASNETE_STARTOFBITS(&gpd->u.put_val, nbytes), nbytes, gpd);
     gasneti_resume_spinpollers();
     return((gasnet_handle_t) eop);
   }
 }
 
-extern void gasnete_put_nbi_val(gasnet_node_t node, void *dest, gasnet_register_value_t value, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUTVAL(V);
+extern void gasnete_put_nbi_val(gasnet_node_t node, void *dest, gasnet_register_value_t value, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUTVAL(V,node,dest,value,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     GASNETC_DIDX_POST(mythread->domain_idx);
     gasnete_iop_t * const iop = mythread->current_iop;
     gasnetc_post_descriptor_t *gpd;
     gasneti_suspend_spinpollers();
     gpd = gasnete_cntr_gpd(GASNETE_IOP_CNTRS(iop, put) GASNETC_DIDX_PASS);
-    gasnete_val_assign(gpd->u.immediate, value);
-    gasnetc_rdma_put_buff(node, dest, GASNETE_STARTOFBITS(gpd->u.immediate, nbytes), nbytes, gpd);
+    gpd->u.put_val = value;
+    gasnetc_rdma_put_buff(node, dest, GASNETE_STARTOFBITS(&gpd->u.put_val, nbytes), nbytes, gpd);
     gasneti_resume_spinpollers();
   }
 }
@@ -996,11 +972,11 @@ gasnet_register_value_t gasnete_get_val_help(void *src, size_t nbytes) {
 #endif
 }
  
-extern gasnet_register_value_t gasnete_get_val(gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GETVAL();
+extern gasnet_register_value_t gasnete_get_val(gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GETVAL(node,src,nbytes);
   {
     gasnet_register_value_t result;
-    GASNETC_DIDX_POST(GASNETE_MYTHREAD->domain_idx);
+    GASNETC_DIDX_POST(GASNETI_MYTHREAD->domain_idx);
     gasnetc_post_descriptor_t *gpd;
     volatile int done = 0;
     uint8_t *buffer;
@@ -1033,7 +1009,7 @@ typedef struct _gasnete_valget_op_t {
 } gasnete_valget_op_t;
 
 GASNETI_INLINE(gasnete_new_valget_handle)
-gasnet_valget_handle_t gasnete_new_valget_handle(gasnete_threaddata_t * const mythread) {
+gasnet_valget_handle_t gasnete_new_valget_handle(gasneti_threaddata_t * const mythread) {
   gasnet_valget_handle_t retval;
   if (mythread->valget_free) {
     retval = mythread->valget_free;
@@ -1048,8 +1024,8 @@ gasnet_valget_handle_t gasnete_new_valget_handle(gasnete_threaddata_t * const my
   return retval;
 }
 
-extern gasnet_valget_handle_t gasnete_get_nb_val(gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern gasnet_valget_handle_t gasnete_get_nb_val(gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnet_valget_handle_t retval = gasnete_new_valget_handle(mythread);
 
   gasneti_assert(nbytes > 0 && nbytes <= sizeof(gasnet_register_value_t));
@@ -1084,11 +1060,11 @@ extern gasnet_valget_handle_t gasnete_get_nb_val(gasnet_node_t node, void *src,
 
 extern gasnet_register_value_t gasnete_wait_syncnb_valget(gasnet_valget_handle_t handle) {
   gasnete_assert_valid_threadid(handle->threadidx);
-  { gasnete_threaddata_t * const thread = gasnete_threadtable[handle->threadidx];
+  { gasneti_threaddata_t * const thread = gasnete_threadtable[handle->threadidx];
     gasnet_register_value_t val;
     GASNETC_DIDX_POST(thread->domain_idx);
     GASNET_POST_THREADINFO(thread); /* for gasneti_poll() in multi-domain case */
-    gasneti_assert(thread == gasnete_mythread());
+    gasneti_assert(thread == _gasneti_mythread_slow());
     handle->next = thread->valget_free; /* free before the wait to save time after the wait, */
     thread->valget_free = handle;       /*  safe because this thread is under our control */
     gasneti_polluntil(handle->done);
@@ -1097,7 +1073,7 @@ extern gasnet_register_value_t gasnete_wait_syncnb_valget(gasnet_valget_handle_t
   }
 }
 
-static void gasnete_valget_freeall(gasnete_threaddata_t *thread) {
+static void gasnete_valget_freeall(gasneti_threaddata_t *thread) {
   gasnete_valget_op_t *vg = thread->valget_free;
   while (vg) {
     gasnete_valget_op_t *next = vg->next;
@@ -1117,9 +1093,9 @@ static void gasnete_valget_freeall(gasnete_threaddata_t *thread) {
 
 static gasnet_handle_t gasnete_fetchop_u64_nb(
         uint64_t *dest, gasnet_node_t node, uint64_t *src,
-        gni_fma_cmd_type_t cmd, uint64_t operand GASNETE_THREAD_FARG)
+        gni_fma_cmd_type_t cmd, uint64_t operand GASNETI_THREAD_FARG)
 {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   GASNETC_DIDX_POST(mythread->domain_idx);
   gasnete_eop_t * const eop = _gasnete_eop_new(mythread);
   gasnetc_post_descriptor_t *gpd;
@@ -1136,9 +1112,9 @@ static gasnet_handle_t gasnete_fetchop_u64_nb(
 
 static void gasnete_fetchop_u64_nbi(
         uint64_t *dest, gasnet_node_t node, uint64_t *src,
-        gni_fma_cmd_type_t cmd, uint64_t operand GASNETE_THREAD_FARG)
+        gni_fma_cmd_type_t cmd, uint64_t operand GASNETI_THREAD_FARG)
 {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   GASNETC_DIDX_POST(mythread->domain_idx);
   gasnete_iop_t * const iop = mythread->current_iop;
   gasnetc_post_descriptor_t *gpd;
@@ -1153,10 +1129,10 @@ static void gasnete_fetchop_u64_nbi(
 
 static uint64_t gasnete_fetchop_u64_val(
         gasnet_node_t node, void *src, gni_fma_cmd_type_t cmd,
-        uint64_t operand GASNETE_THREAD_FARG)
+        uint64_t operand GASNETI_THREAD_FARG)
 {
   uint64_t result;
-  GASNETC_DIDX_POST(GASNETE_MYTHREAD->domain_idx);
+  GASNETC_DIDX_POST(GASNETI_MYTHREAD->domain_idx);
   gasnetc_post_descriptor_t *gpd;
   volatile int done = 0;
 
@@ -1168,16 +1144,16 @@ static uint64_t gasnete_fetchop_u64_val(
   gasneti_resume_spinpollers();
 
   gasneti_polluntil(done);
-  result = * (uint64_t *) gpd->u.immediate;
+  result = gpd->u.u64;
   gasnetc_free_post_descriptor(gpd);
   return result;
 }
 
 static gasnet_valget_handle_t gasnete_fetchop_u64_nb_val(
         gasnet_node_t node, void *src, gni_fma_cmd_type_t cmd,
-        uint64_t operand GASNETE_THREAD_FARG)
+        uint64_t operand GASNETI_THREAD_FARG)
 {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   GASNETC_DIDX_POST(mythread->domain_idx);
   gasnetc_post_descriptor_t *gpd;
 
@@ -1200,38 +1176,38 @@ static gasnet_valget_handle_t gasnete_fetchop_u64_nb_val(
     extern void                                                           \
     _gasnetX_fetch##_op##_##_suff(                                        \
                 _type *dest, gasnet_node_t node, _type *src,              \
-                _type operand GASNETE_THREAD_FARG)                        \
+                _type operand GASNETI_THREAD_FARG)                        \
     {                                                                     \
-        *dest = gasnete_fetchop_##_suff##_val(node, src, _cmd, operand GASNETE_THREAD_PASS); \
+        *dest = gasnete_fetchop_##_suff##_val(node, src, _cmd, operand GASNETI_THREAD_PASS); \
         gasneti_sync_writes();                                            \
     }                                                                     \
     extern gasnet_handle_t                                                \
     _gasnetX_fetch##_op##_##_suff##_nb(                                   \
                 _type *dest, gasnet_node_t node, _type *src,              \
-                _type operand GASNETE_THREAD_FARG)                        \
+                _type operand GASNETI_THREAD_FARG)                        \
     {                                                                     \
-        return gasnete_fetchop_##_suff##_nb(dest, node, src, _cmd, operand GASNETE_THREAD_PASS); \
+        return gasnete_fetchop_##_suff##_nb(dest, node, src, _cmd, operand GASNETI_THREAD_PASS); \
     }                                                                     \
     extern void                                                           \
     _gasnetX_fetch##_op##_##_suff##_nbi(                                  \
                 _type *dest, gasnet_node_t node, _type *src,              \
-                _type operand GASNETE_THREAD_FARG)                        \
+                _type operand GASNETI_THREAD_FARG)                        \
     {                                                                     \
-        gasnete_fetchop_##_suff##_nbi(dest, node, src, _cmd, operand GASNETE_THREAD_PASS); \
+        gasnete_fetchop_##_suff##_nbi(dest, node, src, _cmd, operand GASNETI_THREAD_PASS); \
     }                                                                     \
     extern _type                                                          \
     _gasnetX_fetch##_op##_##_suff##_val(                                  \
                 gasnet_node_t node, _type *src,                           \
-                _type operand GASNETE_THREAD_FARG)                        \
+                _type operand GASNETI_THREAD_FARG)                        \
     {                                                                     \
-        return gasnete_fetchop_##_suff##_val(node, src, _cmd, operand GASNETE_THREAD_PASS); \
+        return gasnete_fetchop_##_suff##_val(node, src, _cmd, operand GASNETI_THREAD_PASS); \
     }                                                                     \
     extern gasnet_valget_handle_t                                         \
     _gasnetX_fetch##_op##_##_suff##_nb_val(                               \
                 gasnet_node_t node, _type *src,                           \
-                _type operand GASNETE_THREAD_FARG)                        \
+                _type operand GASNETI_THREAD_FARG)                        \
     {                                                                     \
-        return gasnete_fetchop_##_suff##_nb_val(node, src, _cmd, operand GASNETE_THREAD_PASS); \
+        return gasnete_fetchop_##_suff##_nb_val(node, src, _cmd, operand GASNETI_THREAD_PASS); \
     }                                                                     \
 
 /* protect against iso646.h */
@@ -1390,7 +1366,7 @@ void gasnete_gdbarrier_send(gasnete_coll_gdbarrier_t *barrier_data,
     } else
 #endif
     {
-      GASNETC_DIDX_POST((gasnete_mythread())->domain_idx);
+      GASNETC_DIDX_POST((_gasneti_mythread_slow())->domain_idx);
       gasnetc_post_descriptor_t * const gpd = gasnetc_alloc_post_descriptor(GASNETC_DIDX_PASS_ALONE);
       uint64_t * const src = (uint64_t *)GASNETE_STARTOFBITS(gpd->u.immediate, sizeof(uint64_t));
 
@@ -1785,7 +1761,7 @@ static void gasnete_gdbarrier_init(gasnete_coll_team_t team) {
 */
 
 /* use reference implementation of scatter/gather and strided */
-#include "gasnet_extended_refvis.h"
+#include "gasnet_refvis.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
@@ -1794,7 +1770,7 @@ static void gasnete_gdbarrier_init(gasnete_coll_team_t team) {
 */
 
 /* use reference implementation of collectives */
-#include "gasnet_extended_refcoll.h"
+#include "gasnet_refcoll.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
