@@ -7,23 +7,10 @@
 #ifndef _GASNET_CORE_INTERNAL_H
 #define _GASNET_CORE_INTERNAL_H
 
-#include <stddef.h>	/* for offsetof() */
-
 #include <gasnet_internal.h>
 #include <gasnet_handler.h>
 #include <firehose.h>
 
-
-#if HAVE_SSH_SPAWNER
-  #include <ssh-spawner/gasnet_bootstrap_internal.h>
-#endif
-#if HAVE_MPI_SPAWNER
-  #include <mpi-spawner/gasnet_bootstrap_internal.h>
-#endif
-#if HAVE_PMI_SPAWNER
-  #include <pmi-spawner/gasnet_bootstrap_internal.h>
-#endif
-
 #include <infiniband/verbs.h>
 
 /* TODO: flatten these? */
@@ -170,8 +157,7 @@ typedef enum {
 
 /* ------------------------------------------------------------------------------------ */
 
-#define GASNETC_ARGSEND_AUX(s,nargs) \
-	(offsetof(s,args)+((nargs)*sizeof(gasnet_handlerarg_t)))
+#define GASNETC_ARGSEND_AUX(s,nargs) gasneti_offsetof(s,args[nargs])
 
 typedef struct {
 #if GASNETI_STATS_OR_TRACE
@@ -251,7 +237,7 @@ typedef union {
 /* Define non-zero if we want to allow the mlock rlimit to bound the
  * amount of memory we will pin. */
 #ifndef GASNETC_HONOR_RLIMIT_MEMLOCK
-  #define GASNETC_HONOR_RLIMIT_MEMLOCK 0
+  #define GASNETC_HONOR_RLIMIT_MEMLOCK 1
 #endif
 
 /* Use alloca()?  (e.g. to work-around bug 2079) */
@@ -260,6 +246,9 @@ typedef union {
 #elif !PLATFORM_COMPILER_PGI
   #define GASNETI_USE_ALLOCA 1
 #endif
+#if GASNETI_USE_ALLOCA
+  #include <alloca.h>
+#endif
 
 /* Can one send a 0-byte payload?
  * TODO: autoconf or runtime probe if/when we can determine which systems need this
@@ -439,7 +428,7 @@ typedef char gasnetc_amrdma_buf_t[GASNETC_AMRDMA_SZ];
     /* Initialized by create_cq or spawn_progress_thread: */
     pthread_t               thread_id;
     uint64_t                prev_time;
-    uint64_t                min_us;
+    uint64_t                min_ns;
     struct ibv_cq *         cq;
     struct ibv_comp_channel *compl;
     volatile int            done;
@@ -506,7 +495,7 @@ typedef struct {
 #if GASNETC_USE_RCV_THREAD
   /* Rcv thread */
   gasnetc_progress_thread_t rcv_thread;
-  void                      *rcv_thread_priv;
+  struct gasnetc_rbuf_s *rcv_thread_priv;
 #endif
 
   /* AM-over-RMDA */
@@ -516,11 +505,12 @@ typedef struct {
     gasnetc_atomic_val_t max_peers;
     gasnetc_atomic_t	count;
     gasnetc_cep_t	**cep;
+    volatile int        prev;
   }	  amrdma_rcv;
   struct {
     gasnetc_atomic_t		count;
     gasnetc_atomic_val_t	mask;
-    gasneti_atomic_t		state;
+    gasnetc_atomic_t		state;
     gasnetc_atomic_val_t	floor;
     gasnetc_amrdma_balance_tbl_t *table;
   }	  amrdma_balance;
@@ -656,18 +646,18 @@ extern int gasnetc_ReplyGeneric(gasnetc_category_t category,
 				int numargs, gasnetc_counter_t *mem_oust,
 				gasnetc_atomic_t *completed, va_list argptr);
 #if GASNETC_PIN_SEGMENT
-  extern int gasnetc_rdma_put(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETE_THREAD_FARG);
+  extern int gasnetc_rdma_put(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETI_THREAD_FARG);
 #else
-  extern int gasnetc_rdma_put_fh(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed, gasnetc_counter_t *am_oust GASNETE_THREAD_FARG);
+  extern int gasnetc_rdma_put_fh(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed, gasnetc_counter_t *am_oust GASNETI_THREAD_FARG);
   GASNETI_INLINE(gasnetc_rdma_put)
-  int gasnetc_rdma_put(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETE_THREAD_FARG)
-  { return gasnetc_rdma_put_fh(epid,src_ptr,dst_ptr,nbytes,mem_oust,initiated,completed,NULL GASNETE_THREAD_PASS); }
+  int gasnetc_rdma_put(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETI_THREAD_FARG)
+  { return gasnetc_rdma_put_fh(epid,src_ptr,dst_ptr,nbytes,mem_oust,initiated,completed,NULL GASNETI_THREAD_PASS); }
 #endif
-extern int gasnetc_rdma_get(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETE_THREAD_FARG);
+extern int gasnetc_rdma_get(gasnetc_epid_t epid, void *src_ptr, void *dst_ptr, size_t nbytes, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETI_THREAD_FARG);
 
 #if 0 /* Putv and Getv are unused */
-extern int gasnetc_rdma_putv(gasnetc_epid_t epid, size_t srccount, gasnet_memvec_t const srclist[], void *dst_ptr, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETE_THREAD_FARG);
-extern int gasnetc_rdma_getv(gasnetc_epid_t epid, void *src_ptr, size_t dstcount, gasnet_memvec_t const dstlist[], gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETE_THREAD_FARG);
+extern int gasnetc_rdma_putv(gasnetc_epid_t epid, size_t srccount, gasnet_memvec_t const srclist[], void *dst_ptr, gasnetc_counter_t *mem_oust, gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETI_THREAD_FARG);
+extern int gasnetc_rdma_getv(gasnetc_epid_t epid, void *src_ptr, size_t dstcount, gasnet_memvec_t const dstlist[], gasnetc_atomic_val_t *initiated, gasnetc_atomic_t *completed GASNETI_THREAD_FARG);
 #endif
 
 /* Routines in gasnet_core_thread.c */
@@ -682,24 +672,17 @@ extern void gasnetc_unpin(gasnetc_hca_t *hca, gasnetc_memreg_t *reg);
 #define gasnetc_unmap(reg)	gasnetc_munmap((void *)((reg)->addr), (reg)->len)
 
 /* Bootstrap support */
-extern void (*gasneti_bootstrapFini_p)(void);
-extern void (*gasneti_bootstrapAbort_p)(int exitcode);
-extern void (*gasneti_bootstrapAlltoall_p)(void *src, size_t len, void *dest);
-extern void (*gasneti_bootstrapBroadcast_p)(void *src, size_t len, void *dest, int rootnode);
-extern void (*gasneti_bootstrapSNodeCast_p)(void *src, size_t len, void *dest, int rootnode);
-extern void (*gasneti_bootstrapCleanup_p)(void);
-#define gasneti_bootstrapFini           (*gasneti_bootstrapFini_p)
-#define gasneti_bootstrapAbort          (*gasneti_bootstrapAbort_p)
-#define gasneti_bootstrapAlltoall       (*gasneti_bootstrapAlltoall_p)
-#define gasneti_bootstrapBroadcast      (*gasneti_bootstrapBroadcast_p)
-#define gasneti_bootstrapSNodeBroadcast (*gasneti_bootstrapSNodeCast_p)
-#define gasneti_bootstrapCleanup        (*gasneti_bootstrapCleanup_p)
+extern gasneti_spawnerfn_t const *gasneti_spawner;
+
+/* This indirection allows us to drop-in a native implementation after init */
 extern void gasneti_bootstrapBarrier(void);
 extern void gasneti_bootstrapExchange(void *src, size_t len, void *dest);
-
-/* Explicitly native Barrier and Exchange: */
-extern void gasnetc_bootstrapBarrier_ib(void);
-extern void gasnetc_bootstrapExchange_ib(void *src, size_t len, void *dest);
+#define gasneti_bootstrapBroadcast      (*(gasneti_spawner->Broadcast))
+#define gasneti_bootstrapSNodeBroadcast (*(gasneti_spawner->SNodeBroadcast))
+#define gasneti_bootstrapAlltoall       (*(gasneti_spawner->Alltoall))
+#define gasneti_bootstrapAbort          (*(gasneti_spawner->Abort))
+#define gasneti_bootstrapCleanup        (*(gasneti_spawner->Cleanup))
+#define gasneti_bootstrapFini           (*(gasneti_spawner->Fini))
 
 /* Global configuration variables */
 extern int		gasnetc_op_oust_limit;
