@@ -7,6 +7,7 @@
 #include <gasnet_internal.h>
 #include <gasnet_extended_internal.h>
 #include <gasnet_handler.h>
+#include <gasnet_coll.h>
 
 static const gasnete_eopaddr_t EOPADDR_NIL = { { 0xFF, 0xFF } };
 extern void _gasnete_iop_check(gasnete_iop_t *iop) { gasnete_iop_check(iop); }
@@ -23,7 +24,7 @@ extern void _gasnete_iop_check(gasnete_iop_t *iop) { gasnete_iop_check(iop); }
 
 /*  allocate more eops */
 GASNETI_NEVER_INLINE(gasnete_eop_alloc,
-static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
+static void gasnete_eop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_eopaddr_t addr;
     int bufidx = thread->eop_num_bufs;
     gasnete_eop_t *buf;
@@ -109,7 +110,7 @@ static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  allocate a new iop */
 GASNETI_NEVER_INLINE(gasnete_iop_alloc,
-static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
+static gasnete_iop_t *gasnete_iop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_iop_t *iop = (gasnete_iop_t *)gasneti_malloc(sizeof(gasnete_iop_t));
     gasneti_leak(iop);
     #if GASNET_DEBUG
@@ -126,7 +127,7 @@ static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  get a new op */
 static
-gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *_gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eopaddr_t head = thread->eop_free;
   if_pf (gasnete_eopaddr_isnil(head)) {
     gasnete_eop_alloc(thread);
@@ -149,7 +150,7 @@ gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new op AND mark it in flight */
 GASNETI_INLINE(gasnete_eop_new)
-gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eop_t *eop = _gasnete_eop_new(thread);
 #if GASNETE_EOP_COUNTED
   eop->initiated_cnt++;
@@ -159,7 +160,7 @@ gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new iop */
 static
-gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
+gasnete_iop_t *gasnete_iop_new(gasneti_threaddata_t * const thread) {
   gasnete_iop_t *iop = thread->iop_free;
   if_pt (iop) {
     thread->iop_free = iop->next;
@@ -184,7 +185,7 @@ gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
 /*  query an eop for completeness */
 static
 int gasnete_eop_isdone(gasnete_eop_t *eop) {
-  gasneti_assert(eop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(eop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_eop_check(eop);
   return GASNETE_EOP_DONE(eop);
 }
@@ -192,7 +193,7 @@ int gasnete_eop_isdone(gasnete_eop_t *eop) {
 /*  query an iop for completeness - this means both puts and gets */
 static
 int gasnete_iop_isdone(gasnete_iop_t *iop) {
-  gasneti_assert(iop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(iop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_iop_check(iop);
   return (GASNETE_IOP_CNTDONE(iop,get) && GASNETE_IOP_CNTDONE(iop,put));
 }
@@ -215,9 +216,9 @@ void gasnete_op_markdone(gasnete_op_t *op, int isget) {
 /*  free an eop */
 static
 void gasnete_eop_free(gasnete_eop_t *eop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
+  gasneti_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
   gasnete_eopaddr_t addr = eop->addr;
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_eop_check(eop);
   gasneti_assert(GASNETE_EOP_DONE(eop));
 #if GASNET_DEBUG
@@ -230,8 +231,8 @@ void gasnete_eop_free(gasnete_eop_t *eop) {
 /*  free an iop */
 static
 void gasnete_iop_free(gasnete_iop_t *iop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_iop_check(iop);
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,get));
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,put));
@@ -266,7 +267,6 @@ static void gasnete_check_config(void) {
 }
 
 extern void gasnete_init(void) {
-  GASNETI_UNUSED_UNLESS_DEBUG
   static int firstcall = 1;
   GASNETI_TRACE_PRINTF(C,("gasnete_init()"));
   gasneti_assert(firstcall); /*  make sure we haven't been called before */
@@ -276,11 +276,11 @@ extern void gasnete_init(void) {
 
   gasneti_assert(gasneti_nodes >= 1 && gasneti_mynode < gasneti_nodes);
 
-  { gasnete_threaddata_t *threaddata = NULL;
+  { gasneti_threaddata_t *threaddata = NULL;
     gasnete_eop_t *eop = NULL;
     #if GASNETI_MAX_THREADS > 1
       /* register first thread (optimization) */
-      threaddata = gasnete_mythread(); 
+      threaddata = _gasneti_mythread_slow(); 
     #else
       /* register only thread (required) */
       threaddata = gasnete_new_threaddata();
@@ -301,12 +301,12 @@ extern void gasnete_init(void) {
 
 /* ------------------------------------------------------------------------------------ */
 /* GASNET-Internal OP Interface */
-gasneti_eop_t *gasneti_eop_create(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_eop_t *op = gasnete_eop_new(GASNETE_MYTHREAD);
+gasneti_eop_t *gasneti_eop_create(GASNETI_THREAD_FARG_ALONE) {
+  gasnete_eop_t *op = gasnete_eop_new(GASNETI_MYTHREAD);
   return (gasneti_eop_t *)op;
 }
-gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t * const op = mythread->current_iop;
   gasnete_iop_check(op);
   if (isget) op->initiated_get_cnt += noperations;
@@ -355,39 +355,39 @@ void gasneti_iop_markdone(gasneti_iop_t *iop, unsigned int noperations, int isge
 #define GASNETE_EOP_CNTRS(_eop) \
         &(_eop)->initiated_cnt, &(_eop)->completed_cnt
 
-extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,H);
+extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,H,dest,node,src,nbytes);
  {
-  gasnete_eop_t *op = _gasnete_eop_new(GASNETE_MYTHREAD);
+  gasnete_eop_t *op = _gasnete_eop_new(GASNETI_MYTHREAD);
 
   /* XXX check error returns */
-  gasnetc_rdma_get(node, src, dest, nbytes, GASNETE_EOP_CNTRS(op) GASNETE_THREAD_PASS);
+  gasnetc_rdma_get(node, src, dest, nbytes, GASNETE_EOP_CNTRS(op) GASNETI_THREAD_PASS);
 
   return (gasnet_handle_t)op;
  }
 }
 
-extern gasnet_handle_t gasnete_put_nb      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
+extern gasnet_handle_t gasnete_put_nb      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
  {
-  gasnete_eop_t *op = _gasnete_eop_new(GASNETE_MYTHREAD);
+  gasnete_eop_t *op = _gasnete_eop_new(GASNETI_MYTHREAD);
   gasnetc_counter_t mem_oust = GASNETC_COUNTER_INITIALIZER;
 
   /* XXX check error returns */
-  gasnetc_rdma_put(node, src, dest, nbytes, &mem_oust, GASNETE_EOP_CNTRS(op) GASNETE_THREAD_PASS);
+  gasnetc_rdma_put(node, src, dest, nbytes, &mem_oust, GASNETE_EOP_CNTRS(op) GASNETI_THREAD_PASS);
   gasnetc_counter_wait(&mem_oust, 0);
 
   return (gasnet_handle_t)op;
  }
 }
 
-extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
+extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
  {
-  gasnete_eop_t *op = _gasnete_eop_new(GASNETE_MYTHREAD);
+  gasnete_eop_t *op = _gasnete_eop_new(GASNETI_MYTHREAD);
 
   /* XXX check error returns */
-  gasnetc_rdma_put(node, src, dest, nbytes, NULL, GASNETE_EOP_CNTRS(op) GASNETE_THREAD_PASS);
+  gasnetc_rdma_put(node, src, dest, nbytes, NULL, GASNETE_EOP_CNTRS(op) GASNETI_THREAD_PASS);
 
   return (gasnet_handle_t)op;
  }
@@ -406,7 +406,7 @@ GASNETI_INLINE(gasnete_op_try_free)
 int gasnete_op_try_free(gasnet_handle_t handle) {
   gasnete_op_t *op = (gasnete_op_t *)handle;
 
-  gasneti_assert(op->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(op->threadidx == _gasneti_mythread_slow()->threadidx);
   if_pt (OPTYPE(op) == OPTYPE_EXPLICIT) {
     gasnete_eop_t *eop = (gasnete_eop_t*)op;
 
@@ -507,39 +507,39 @@ extern int  gasnete_try_syncnb_all (gasnet_handle_t *phandle, size_t numhandles)
 #define GASNETE_IOP_CNTRS(_iop,_putget) \
         &(_iop)->initiated_##_putget##_cnt, &(_iop)->completed_##_putget##_cnt
 
-extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
  {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *op = mythread->current_iop;
 
   /* XXX check error returns */ 
-  gasnetc_rdma_get(node, src, dest, nbytes, GASNETE_IOP_CNTRS(op,get) GASNETE_THREAD_PASS);
+  gasnetc_rdma_get(node, src, dest, nbytes, GASNETE_IOP_CNTRS(op,get) GASNETI_THREAD_PASS);
  }
 }
 
-extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
+extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
   
-  GASNETI_CHECKPSHM_PUT(ALIGNED,V);
+  GASNETI_CHECKPSHM_PUT(ALIGNED,V,node,dest,src,nbytes);
  {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *op = mythread->current_iop;
   gasnetc_counter_t mem_oust = GASNETC_COUNTER_INITIALIZER;
 
   /* XXX check error returns */ 
-  gasnetc_rdma_put(node, src, dest, nbytes, &mem_oust, GASNETE_IOP_CNTRS(op,put) GASNETE_THREAD_PASS);
+  gasnetc_rdma_put(node, src, dest, nbytes, &mem_oust, GASNETE_IOP_CNTRS(op,put) GASNETI_THREAD_PASS);
   gasnetc_counter_wait(&mem_oust, 0);
  }
 }
 
-extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,V);
+extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,V,node,dest,src,nbytes);
  {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *op = mythread->current_iop;
 
   /* XXX check error returns */ 
-  gasnetc_rdma_put(node, src, dest, nbytes, NULL, GASNETE_IOP_CNTRS(op,put) GASNETE_THREAD_PASS);
+  gasnetc_rdma_put(node, src, dest, nbytes, NULL, GASNETE_IOP_CNTRS(op,put) GASNETI_THREAD_PASS);
  }
 }
 
@@ -549,13 +549,13 @@ extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, siz
   ===========================================================
 */
 
-extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_gets(GASNETI_THREAD_FARG_ALONE) {
 #if 0
  /* polling for syncnbi now happens in header file to avoid duplication */
  GASNETI_SAFE(gasneti_AMPoll());
 #endif
  {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop;
   gasneti_assert(iop->threadidx == mythread->threadidx);
   gasneti_assert(OPTYPE(iop) == OPTYPE_IMPLICIT);
@@ -571,13 +571,13 @@ extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
  }
 }
 
-extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_puts(GASNETI_THREAD_FARG_ALONE) {
 #if 0
  /* polling for syncnbi now happens in header file to avoid duplication */
  GASNETI_SAFE(gasneti_AMPoll());
 #endif
  {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop;
   gasneti_assert(iop->threadidx == mythread->threadidx);
   gasneti_assert(iop->next == NULL);
@@ -602,8 +602,8 @@ extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
 */
 /*  This implementation allows recursive access regions, although the spec does not require that */
 /*  operations are associated with the most immediately enclosing access region */
-extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = gasnete_iop_new(mythread); /*  push an iop */
   GASNETI_TRACE_PRINTF(S,("BEGIN_NBI_ACCESSREGION"));
   #if GASNET_DEBUG
@@ -614,8 +614,8 @@ extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE
   mythread->current_iop = iop;
 }
 
-extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETI_THREAD_FARG_ALONE) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop; /*  pop an iop */
   GASNETI_TRACE_EVENT_VAL(S,END_NBI_ACCESSREGION,iop->initiated_get_cnt + iop->initiated_put_cnt);
   #if GASNET_DEBUG
@@ -637,21 +637,21 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
         &(_req).initiated, &(_req).completed
 
 extern void gasnete_get_bulk (void *dest, gasnet_node_t node, void *src,
-			      size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+			      size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
  {
   gasnetc_counter_t req_oust = GASNETC_COUNTER_INITIALIZER;
-  gasnetc_rdma_get(node, src, dest, nbytes, GASNETE_REQ_CNTRS(req_oust) GASNETE_THREAD_PASS);
+  gasnetc_rdma_get(node, src, dest, nbytes, GASNETE_REQ_CNTRS(req_oust) GASNETI_THREAD_PASS);
   gasnetc_counter_wait(&req_oust, 0);
  }
 }
 
 extern void gasnete_put_bulk (gasnet_node_t node, void* dest, void *src,
-			      size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,V);
+			      size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,V,node,dest,src,nbytes);
  {
   gasnetc_counter_t req_oust = GASNETC_COUNTER_INITIALIZER;
-  gasnetc_rdma_put(node, src, dest, nbytes, NULL, GASNETE_REQ_CNTRS(req_oust) GASNETE_THREAD_PASS);
+  gasnetc_rdma_put(node, src, dest, nbytes, NULL, GASNETE_REQ_CNTRS(req_oust) GASNETI_THREAD_PASS);
   gasnetc_counter_wait(&req_oust, 0);
  }
 }   
@@ -707,7 +707,6 @@ static int gasnete_conduit_rdmabarrier(const char *barrier, gasneti_auxseg_reque
     for (steps=0, i=1; i<size; ++steps, i*=2) /* empty */ ;
 
     request = 2 * steps * GASNETE_RDMABARRIER_INBOX_SZ;
-    gasneti_assert_always(GASNETE_RDMABARRIER_INBOX_SZ >= sizeof(uint64_t));
     gasneti_assert_always(request <= result->optimalsz);
     result->minsz = request;
     result->optimalsz = request;
@@ -719,83 +718,53 @@ static int gasnete_conduit_rdmabarrier(const char *barrier, gasneti_auxseg_reque
 
 /* ------------------------------------------------------------------------------------ */
 /* IB-specific RDMA-based Dissemination implementation of barrier
- * This is an adaptation of the "rmd" barrier in exteneded-ref.
+ * This is a minor variation on the "rmd" barrier in extended-ref.
  * Key differences:
  *  + no complications due to thread-specific handles
- *  + simple (normally inline) 64-bit put
  *  + no handle completion latency
+ * TODO: factor the common elements
  */
 
-#if !GASNETI_THREADS
-  #define GASNETE_IBDBARRIER_LOCK(_var)		/* empty */
-  #define gasnete_ibdbarrier_lock_init(_var)	((void)0)
-  #define gasnete_ibdbarrier_trylock(_var)	(0/*success*/)
-  #define gasnete_ibdbarrier_unlock(_var)	((void)0)
-#elif GASNETI_HAVE_SPINLOCK
-  #define GASNETE_IBDBARRIER_LOCK(_var)		gasneti_atomic_t _var;
-  #define gasnete_ibdbarrier_lock_init(_var)	gasneti_spinlock_init(_var)
-  #define gasnete_ibdbarrier_trylock(_var)	gasneti_spinlock_trylock(_var)
-  #define gasnete_ibdbarrier_unlock(_var)	gasneti_spinlock_unlock(_var)
-#else
-  #define GASNETE_IBDBARRIER_LOCK(_var)		gasneti_mutex_t _var;
-  #define gasnete_ibdbarrier_lock_init(_var)	gasneti_mutex_init(_var)
-  #define gasnete_ibdbarrier_trylock(_var)	gasneti_mutex_trylock(_var)
-  #define gasnete_ibdbarrier_unlock(_var)	gasneti_mutex_unlock(_var)
-#endif
+/* Reusing gasnete_coll_rmdbarrier_inbox_t from the reference implementation */
 
 typedef struct {
-  GASNETE_IBDBARRIER_LOCK(barrier_lock) /* no semicolon */
+  GASNETE_RMDBARRIER_LOCK(barrier_lock) /* no semicolon */
   struct {
     gasnet_node_t node;
-    uint64_t      *addr;
+    uintptr_t     addr;
   } *barrier_peers;           /*  precomputed list of peers to communicate with */
 #if GASNETI_PSHM_BARRIER_HIER
   gasnete_pshmbarrier_data_t *barrier_pshm; /* non-NULL if using hierarchical code */
   int barrier_passive;        /*  2 if some other node makes progress for me, 0 otherwise */
 #endif
+  int barrier_size;           /*  ceil(lg(nodes)) */
   int barrier_goal;           /*  (1+ceil(lg(nodes)) << 1) == final barrier_state for phase=0 */
   int volatile barrier_state; /*  (step << 1) | phase, where step is 1-based (0 is pshm notify) */
   int volatile barrier_value; /*  barrier value (evolves from local value) */
   int volatile barrier_flags; /*  barrier flags (evolves from local value) */
-  volatile uint64_t *barrier_inbox; /*  in-segment memory to recv notifications */
+  void *barrier_inbox;        /*  in-segment memory to recv notifications */
 } gasnete_coll_ibdbarrier_t;
 
-/* Unlike the extended-ref version we CAN assume that a 64-bit write
- * will be atomic, and so can use a "present" bit in flags.
- */
-#define GASNETE_IBDBARRIER_PRESENT GASNETE_BARRIERFLAG_CONDUIT0
-#define GASNETE_IBDBARRIER_VALUE(_u64) GASNETI_HIWORD(_u64)
-#define GASNETE_IBDBARRIER_FLAGS(_u64) GASNETI_LOWORD(_u64)
-#define GASNETE_IBDBARRIER_BUILD(_value,_flags) \
-                GASNETI_MAKEWORD((_value),GASNETE_IBDBARRIER_PRESENT|(_flags))
-  
-/* Pad struct to a specfic size and interleave */
-#define GASNETE_IBDBARRIER_INBOX_WORDS (GASNETE_RDMABARRIER_INBOX_SZ/sizeof(uint64_t))
-#define GASNETE_IBDBARRIER_INBOX(_bd,_state)     \
-            (((_bd)->barrier_inbox) \
-                       + (unsigned)((_state)-2) * GASNETE_IBDBARRIER_INBOX_WORDS)
-#define GASNETE_IBDBARRIER_INBOX_REMOTE(_bd,_step,_state)  \
-            (((_bd)->barrier_peers[(unsigned)(_step)].addr \
-                       + (unsigned)((_state)-2) * GASNETE_IBDBARRIER_INBOX_WORDS))
-#define GASNETE_IBDBARRIER_INBOX_NEXT(_addr)    \
-            ((_addr) + 2U * GASNETE_IBDBARRIER_INBOX_WORDS)
 
 GASNETI_INLINE(gasnete_ibdbarrier_send)
 void gasnete_ibdbarrier_send(gasnete_coll_ibdbarrier_t *barrier_data,
                              int numsteps, unsigned int state,
                              gasnet_handlerarg_t value, gasnet_handlerarg_t flags) {
-  GASNETE_THREAD_LOOKUP
+  GASNETI_THREAD_LOOKUP /* XXX: can we remove/avoid this lookup? */
   unsigned int step = state >> 1;
+  gasnete_coll_rmdbarrier_inbox_t *payload;
   int i;
 
   /* Use the upper half (padding) an "other phase" inbox as an in-segment temporary.
    * This has sufficient lifetime for bulk and sufficient alignment for non-bulk.
    * Use of opposite phase prevents cacheline contention with arrivals.
    */
-  const uint64_t msg = GASNETE_IBDBARRIER_BUILD(value, flags);
-  volatile uint64_t *payload = GASNETE_IBDBARRIER_INBOX(barrier_data, (state^1))
-                                    + (GASNETE_IBDBARRIER_INBOX_WORDS/2);
-  *payload = msg;
+  const unsigned int stride = GASNETE_RDMABARRIER_INBOX_SZ / sizeof(gasnete_coll_rmdbarrier_inbox_t);
+  payload = (stride/2) + GASNETE_RDMABARRIER_INBOX(barrier_data, (state^1));
+  payload->value  = value;
+  payload->flags  = flags;
+  payload->flags2 = ~flags;
+  payload->value2 = ~value;
 
   /* TODO:
    *   Reduce latency by pre-computing sr_desc and rkey_index at init time
@@ -805,13 +774,14 @@ void gasnete_ibdbarrier_send(gasnete_coll_ibdbarrier_t *barrier_data,
 
   for (i = 0; i < numsteps; ++i, state += 2, step += 1) {
     const gasnet_node_t node = barrier_data->barrier_peers[step].node;
-    uint64_t * const dst = GASNETE_IBDBARRIER_INBOX_REMOTE(barrier_data, step, state);
+    void * const addr = GASNETE_RDMABARRIER_INBOX_REMOTE(barrier_data, step, state);
+
 #if GASNET_PSHM
     if (gasneti_pshm_in_supernode(node)) {
-      *(uint64_t*)gasneti_pshm_addr2local(node, dst) = msg;
+      *(volatile gasnete_coll_rmdbarrier_inbox_t *)addr = *payload;
     } else
 #endif
-    (void) gasnetc_rdma_put(node, (void*)payload, dst, sizeof(*payload), NULL, NULL, NULL GASNETE_THREAD_PASS);
+    (void) gasnetc_rdma_put(node, (void*)payload, addr, sizeof(*payload), NULL, NULL, NULL GASNETI_THREAD_PASS);
   }
 }
 
@@ -820,7 +790,7 @@ static int gasnete_ibdbarrier_kick_pshm(gasnete_coll_team_t team) {
   gasnete_coll_ibdbarrier_t *barrier_data = team->barrier_data;
   int done = (barrier_data->barrier_state > 1);
 
-  if (!done && !gasnete_ibdbarrier_trylock(&barrier_data->barrier_lock)) {
+  if (!done && !gasnete_rmdbarrier_trylock(&barrier_data->barrier_lock)) {
     const int state = barrier_data->barrier_state;
     done = (state > 1);
     if (!done) {
@@ -832,8 +802,8 @@ static int gasnete_ibdbarrier_kick_pshm(gasnete_coll_team_t team) {
         barrier_data->barrier_flags = flags;
         gasneti_sync_writes();
         barrier_data->barrier_state = state + 2;
-        gasnete_ibdbarrier_unlock(&barrier_data->barrier_lock); /* Cannot send while holding HSL */
-        if ((barrier_data->barrier_goal > 2) && !barrier_data->barrier_passive) {
+        gasnete_rmdbarrier_unlock(&barrier_data->barrier_lock); /* Cannot send while holding HSL */
+        if (barrier_data->barrier_size && !barrier_data->barrier_passive) {
           gasnete_ibdbarrier_send(barrier_data, 1, state+2, value, flags);
         } else {
           gasnete_barrier_pf_disable(team);
@@ -841,7 +811,7 @@ static int gasnete_ibdbarrier_kick_pshm(gasnete_coll_team_t team) {
         return 1;
       }
     }
-    gasnete_ibdbarrier_unlock(&barrier_data->barrier_lock);
+    gasnete_rmdbarrier_unlock(&barrier_data->barrier_lock);
   }
 
   return done;
@@ -850,8 +820,7 @@ static int gasnete_ibdbarrier_kick_pshm(gasnete_coll_team_t team) {
 
 void gasnete_ibdbarrier_kick(gasnete_coll_team_t team) {
   gasnete_coll_ibdbarrier_t *barrier_data = team->barrier_data;
-  volatile uint64_t *inbox;
-  uint64_t result;
+  gasnete_coll_rmdbarrier_inbox_t *inbox;
   int numsteps = 0;
   int state, new_state;
   int flags, value;
@@ -870,7 +839,7 @@ void gasnete_ibdbarrier_kick(gasnete_coll_team_t team) {
   }
 #endif
 
-  if (gasnete_ibdbarrier_trylock(&barrier_data->barrier_lock))
+  if (gasnete_rmdbarrier_trylock(&barrier_data->barrier_lock))
     return; /* another thread is currently in kick */
 
   /* reread w/ lock held and/or because kick_pshm may have advanced it */
@@ -878,11 +847,11 @@ void gasnete_ibdbarrier_kick(gasnete_coll_team_t team) {
 
 #if GASNETI_PSHM_BARRIER_HIER
   if_pf (state < 2) { /* local notify has not completed */
-    gasnete_ibdbarrier_unlock(&barrier_data->barrier_lock);
+    gasnete_rmdbarrier_unlock(&barrier_data->barrier_lock);
     return;
   } else if (barrier_data->barrier_passive) {
     gasnete_barrier_pf_disable(team);
-    gasnete_ibdbarrier_unlock(&barrier_data->barrier_lock);
+    gasnete_rmdbarrier_unlock(&barrier_data->barrier_lock);
     return;
   }
   gasneti_assert(!barrier_data->barrier_passive);
@@ -898,11 +867,32 @@ void gasnete_ibdbarrier_kick(gasnete_coll_team_t team) {
   flags = barrier_data->barrier_flags;
 
   /* process all consecutive steps which have arrived since we last ran */
-  inbox = GASNETE_IBDBARRIER_INBOX(barrier_data, state);
-  for (new_state = state; new_state < barrier_data->barrier_goal && (0 != (result = *inbox)); new_state+=2) {
-    const int step_value = GASNETE_IBDBARRIER_VALUE(result);
-    const int step_flags = GASNETE_IBDBARRIER_FLAGS(result);
-    *inbox = 0;
+  inbox = GASNETE_RDMABARRIER_INBOX(barrier_data, state);
+  for (new_state = state; new_state < barrier_data->barrier_goal && gasnete_rmdbarrier_poll(inbox); new_state+=2) {
+    const int step_value = inbox->value;
+    const int step_flags = inbox->flags;
+
+    gasneti_assert(inbox->value2 == ~step_value);
+    gasneti_assert(inbox->flags2 == ~step_flags);
+
+    /* "reset" the inbox
+     *
+     * We need to protect against "late arrivals", which are bytes written by the NIC
+     * after we reset.  These late arrivals are possible if we've passed the _poll
+     * check "too early" because one or more bytes already contained the proper value
+     * by chance.  The risk is that the late arrivial byte(s) might match their peer
+     * (e.g. value vs value2) BEFORE the next barrier leads to an actual arrival.
+     * So, we must ensure that every (non-padding) byte is changed by the reset.
+     */
+    {
+  #if SIZEOF_INT == 8
+      unsigned int mask = 0x0101010101010101;
+  #else
+      unsigned int mask = 0x01010101;
+  #endif
+      inbox->value = inbox->value2 = step_value ^ mask;
+      inbox->flags = inbox->flags2 = step_flags ^ mask;
+    }
 
     if ((flags | step_flags) & GASNET_BARRIERFLAG_MISMATCH) {
       flags = GASNET_BARRIERFLAG_MISMATCH; 
@@ -914,7 +904,7 @@ void gasnete_ibdbarrier_kick(gasnete_coll_team_t team) {
     }
 
     ++numsteps;
-    inbox = GASNETE_IBDBARRIER_INBOX_NEXT(inbox);
+    inbox = GASNETE_RDMABARRIER_INBOX_NEXT(inbox);
   }
 
   if (numsteps) { /* completed one or more steps */
@@ -933,7 +923,7 @@ void gasnete_ibdbarrier_kick(gasnete_coll_team_t team) {
     barrier_data->barrier_state = new_state;
   } 
 
-  gasnete_ibdbarrier_unlock(&barrier_data->barrier_lock);
+  gasnete_rmdbarrier_unlock(&barrier_data->barrier_lock);
 
   if (numsteps) { /* need to issue one or more Puts */
     gasnete_ibdbarrier_send(barrier_data, numsteps, state+2, value, flags);
@@ -1159,26 +1149,34 @@ static void gasnete_ibdbarrier_init(gasnete_coll_team_t team) {
 
   gasneti_assert(team == GASNET_TEAM_ALL); /* TODO: deal w/ in-segment allocation */
 
-  gasnete_ibdbarrier_lock_init(&barrier_data->barrier_lock);
+  gasnete_rmdbarrier_lock_init(&barrier_data->barrier_lock);
 
+  /* determine barrier size (number of steps) */
   steps = peers->num;
+  barrier_data->barrier_size = steps;
   barrier_data->barrier_goal = (1+steps) << 1;
 
   if (steps) {
     int step;
 
     gasneti_assert(gasnete_rdmabarrier_auxseg);
+    gasneti_assert_always(2 * sizeof(gasnete_coll_rmdbarrier_inbox_t) <= GASNETE_RDMABARRIER_INBOX_SZ);
     barrier_data->barrier_inbox = gasnete_rdmabarrier_auxseg[gasneti_mynode].addr;
-    gasneti_assert(GASNETE_IBDBARRIER_INBOX_WORDS > 1);
 
     barrier_data->barrier_peers = gasneti_malloc((1+steps) * sizeof(* barrier_data->barrier_peers));
     gasneti_leak(barrier_data->barrier_peers);
   
     for (step = 0; step < steps; ++step) {
       gasnet_node_t node = peers->fwd[step];
+      void *addr = gasnete_rdmabarrier_auxseg[node].addr;
       barrier_data->barrier_peers[1+step].node = node;
-      barrier_data->barrier_peers[1+step].addr = gasnete_rdmabarrier_auxseg[node].addr;
-    } 
+    #if GASNET_PSHM
+      if (gasneti_pshm_in_supernode(node)) {
+        barrier_data->barrier_peers[1+step].addr = (uintptr_t)gasneti_pshm_addr2local(node, addr);
+      } else
+    #endif
+      barrier_data->barrier_peers[1+step].addr = (uintptr_t)addr;
+    }
   } else {
     barrier_data->barrier_state = barrier_data->barrier_goal;
   }
@@ -1212,7 +1210,7 @@ static void gasnete_ibdbarrier_init(gasnete_coll_team_t team) {
 */
 
 /* use reference implementation of scatter/gather and strided */
-#include "gasnet_extended_refvis.h"
+#include "gasnet_refvis.h"
            
 /* ------------------------------------------------------------------------------------ */
 /*
@@ -1221,7 +1219,7 @@ static void gasnete_ibdbarrier_init(gasnete_coll_team_t team) {
 */
 
 /* use reference implementation of collectives */
-#include "gasnet_extended_refcoll.h"
+#include "gasnet_refcoll.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
