@@ -89,7 +89,7 @@ sub gasnet_encode($) {
     my $platform    = $ENV{'GASNET_PLATFORM'};
     #print "using platform hint: $platform\n";
     my $is_lam      = ($mpirun_help =~ m|LAM/MPI|);
-    my $is_ompi     = ($mpirun_help =~ m|OpenRTE|);
+    my $is_ompi     = ($mpirun_help =~ m|Open ?RTE|);
     my $is_mpich2   = ($mpirun_help =~ m|MPICH1 compatibility|);
     my $is_mpiexec  = ($mpirun_help =~ m|mpiexec|);
     my $is_mpiexec_nt = ($mpirun_help =~ m|mpiexec| && $uname =~ m|cygwin|i );
@@ -119,6 +119,7 @@ sub gasnet_encode($) {
                       $mpirun_help =~ m|Usage: srun |);
     my $is_prun    = ($mpirun_help =~ m|railmask|);
     my $is_pam     = ($mpirun_help =~ m|TaskStarter|);
+    my $is_jsrun   = ($mpirun_help =~ m|jsrun --usage|);
     my $envprog = $ENV{'GASNET_ENVCMD'};
     if (! -x $envprog) { # SuperUX has broken "which" implementation, so avoid if possible
       $envprog = `which env`;
@@ -143,8 +144,8 @@ sub gasnet_encode($) {
 	%envfmt = ( 'pre' => '-x',
 		    'inter' => '-x'
 		  );
-        # Seen to crash 1.4.2
-        #$ppn_opt = '-npernode' if ($mpirun_help =~ m/\bnpernode\b/);
+        # Seen to crash 1.4.2, but OK for a long time
+        $ppn_opt = '-npernode' if ($mpirun_help =~ m/\bnpernode\b/);
     } elsif ($is_mpich2) {
 	$spawner_desc = "MPICH2/mpiexec";
 	# pass env as "-envlist A,B,C"
@@ -354,6 +355,11 @@ sub gasnet_encode($) {
 	$encode_args = 1;
 	$encode_env = 1;
 	@verbose_opt = ("-v");
+    } elsif ($is_jsrun) {
+	$spawner_desc = "jsrun - IBM Job Step Manager";
+	%envfmt = ( 'pre' => '-E', 'inter' => '-E');
+	$ppn_opt = '-r';
+	$encode_args = 1;
     } else {
 	$spawner_desc = "unknown program (using generic MPI spawner)";
 	# assume the OS will not propagate the environment
@@ -501,9 +507,10 @@ sub expand {
 	  $exename = `cygpath -a -m $exename`;
 	  chomp($exename);
 	}
-        die("gasnetrun: unable to locate program '$exebase'\n")
-		    unless (defined($exename) && -x $exename);
+        die("gasnetrun: unable to locate program '$exebase'\n") unless defined($exename);
         print("gasnetrun: located executable '$exename'\n") if ($verbose);
+        die("gasnetrun: missing execute permissions for '$exename'\n") unless -x $exename;
+
     } else {
         $exename = $exebase;
     }
@@ -655,8 +662,8 @@ EOF
 	@envargs = ();
      }
 
-# Process LSF host list to ensure it conforms to our request
-if (exists($ENV{'LSB_MCPU_HOSTS'})) {
+# Process LSF host list to ensure it conforms to our request (non-jsrun)
+if (exists($ENV{'LSB_MCPU_HOSTS'}) && !$is_jsrun) {
   my @tmp = split(" ", $ENV{'LSB_MCPU_HOSTS'});
   my %tmp;
   while (@tmp) {
@@ -694,6 +701,23 @@ if (exists($ENV{'LSB_MCPU_HOSTS'})) {
   $dashN_ok = 1;
 }
 
+# With jsrun we (attempt to) default to job size if -N was not given.
+# The "attempt" is a heutistic to exclude the login node which will
+# appear in LSM_MCPU_HOSTS, based on its CPU count of 1.  In the case
+# that *all* hosts are listed as single CPUs this code will not result
+# in any change to $numnode.
+if ($is_jsrun && !defined($numnode)) {
+  my @tmp = split(" ", $ENV{'LSB_MCPU_HOSTS'});
+  my %tmp;
+  while (@tmp) {
+    my $h = shift @tmp; # Host
+    my $n = shift @tmp; # Numcpus
+    $tmp{$h} += $n;
+  }
+  my $count = grep { $tmp{$_} > 1 } keys %tmp;  # counts hosts w/ >1 CPU
+  if ($count) { $numnode = $count; }
+}
+
 # LAM-specific preprocessing of $numproc in the presence of $numnode
 if ($is_lam && $numnode) {
   my @tmp = (0..($numnode-1));
@@ -721,6 +745,8 @@ if ($is_aprun || $is_yod) {
             close(QSTAT);
         } elsif (exists($ENV{'SLURM_JOB_ID'}) && exists($ENV{'SLURM_NNODES'})) {
             $numnode = $ENV{'SLURM_NNODES'};
+        } elsif (exists($ENV{'COBALT_JOBID'}) && exists($ENV{'COBALT_JOBSIZE'})) {
+            $numnode = $ENV{'COBALT_JOBSIZE'};
         }
   }
 
