@@ -95,6 +95,17 @@ void gasnetc_bootstrapBroadcast(void *src, size_t len, void *dest, int rootnode)
   GASNETI_AM_SAFE_NORETURN(retval,AMMPI_SPMDBroadcast(dest, len, rootnode));
   if_pf (retval) gasneti_fatalerror("failure in gasnetc_bootstrapBroadcast()");
 }
+#if GASNET_PSHM /* Used only in call to gasneti_pshm_init() */
+/* Naive (poorly scaling) "reference" implementation via gasnetc_bootstrapExchange() */
+static void gasnetc_bootstrapSNodeBroadcast(void *src, size_t len, void *dest, int rootnode) {
+  void *tmp = gasneti_malloc(len * gasneti_nodes);
+  void *self = src ? src : gasneti_malloc(len); /* Ensure never NULL */
+  gasnetc_bootstrapExchange(self, len, tmp);
+  memcpy(dest, (void*)((uintptr_t)tmp + (len * rootnode)), len);
+  if (self != src) gasneti_free(self);
+  gasneti_free(tmp);
+}
+#endif
 
 #define INITERR(type, reason) do {                                      \
    if (gasneti_VerboseErrors) {                                         \
@@ -131,36 +142,32 @@ static int gasnetc_init(int *argc, char ***argv) {
     networkdepth = gasnett_getenv_int_withdefault("GASNET_NETWORKDEPTH", GASNETC_DEFAULT_NETWORKDEPTH, 0);
     if (networkdepth <= 1) networkdepth = GASNETC_DEFAULT_NETWORKDEPTH;
 
-    AMMPI_VerboseErrors = gasneti_VerboseErrors;
+    AMX_VerboseErrors = gasneti_VerboseErrors;
     AMMPI_SPMDkillmyprocess = gasneti_killmyprocess;
-    #if !defined(GASNETI_DISABLE_MPI_INIT_THREAD) 
-      { int res; 
+    #if !GASNETI_DISABLE_MPI_INIT_THREAD
+    { // this scope silences a warning on Cray C about INITERR bypassing this initialization:
       #if GASNETI_THREADS
-        /* tell MPI to be thread-safe */
-        res = AMMPI_SPMDSetThreadMode(1, &pstr, argc, argv);
+        int usingthreads = 1;
       #else
-        res = AMMPI_SPMDSetThreadMode(0, &pstr, argc, argv);
+        int usingthreads = 0;
       #endif
-        if (!res) { 
-          #if GASNETI_THREADS
-          { static char tmsg[255];
-            snprintf(tmsg, sizeof(tmsg),
-                          "*** WARNING: The pthreaded version of mpi-conduit requires an MPI implementation "
-                          "which supports threading mode MPI_THREAD_SERIALIZED, "
-                          "but this implementation reports it can only support %s\n", pstr);
-            #if GASNET_DEBUG_VERBOSE
-              /* only show this in verbose mode, because some versions of MPICH (eg Quadrics version)
-                 lie and report THREAD_SINGLE, when in actuality MPI_THREAD_SERIALIZED works just fine */
-              if (!gasneti_getenv_yesno_withdefault("GASNET_QUIET",0)) fprintf(stderr, "%s", tmsg);
-            #else
-              tmsgstr = tmsg;
-            #endif
-          }
-          #else
-            fprintf(stderr,"unknown failure in AMMPI_SPMDSetThreadMode() => %s\n",pstr);
-          #endif
-        }
+      // for verbose documentation only:
+      gasnett_getenv_withdefault("GASNET_MPI_THREAD", (usingthreads?"MPI_THREAD_SERIALIZED":"MPI_THREAD_SINGLE"));
+      if (!AMMPI_SPMDSetThreadMode(usingthreads, &pstr, argc, argv)) { 
+        // Some versions of MPI lie and report THREAD_SINGLE, when in actuality MPI_THREAD_SERIALIZED seems to work just fine.
+        // User can ignore this warning or hide it by setting GASNET_MPI_THREAD or GASNET_QUIET if they want to "live dangerously".
+        static char tmsg[1024];
+        snprintf(tmsg, sizeof(tmsg),
+                      "*** WARNING: This MPI implementation reports it can only support %s.\n"
+                    #if GASNETI_THREADS
+                      "*** WARNING: The thread-safe version of mpi-conduit recommends an MPI implementation\n"
+                      "*** WARNING: which supports at least MPI_THREAD_SERIALIZED to ensure correct operation.\n"
+                    #endif
+                      "*** WARNING: You can override the requested thread mode by setting GASNET_MPI_THREAD.\n"
+                      , pstr);
+        tmsgstr = tmsg;
       }
+    }
     #endif
 
     /*  perform job spawn */
@@ -175,11 +182,16 @@ static int gasnetc_init(int *argc, char ***argv) {
     gasneti_setupGlobalEnvironment(gasneti_nodes, gasneti_mynode, 
                                    gasnetc_bootstrapExchange, gasnetc_bootstrapBroadcast);
 
+    /* Must init timers after global env, and preferably before tracing */
+    GASNETI_TICKS_INIT();
+
     /* enable tracing */
     gasneti_trace_init(argc, argv);
     GASNETI_AM_SAFE(AMMPI_SPMDSetExitCallback(gasnetc_traceoutput));
     if (pstr)    GASNETI_TRACE_PRINTF(C,("AMMPI_SPMDSetThreadMode/MPI_Init_thread()=>%s",pstr));
     if (tmsgstr) GASNETI_TRACE_PRINTF(I,("%s",tmsgstr));
+    if (tmsgstr && !gasneti_mynode &&
+        !gasneti_getenv_yesno_withdefault("GASNET_QUIET",0)) { fprintf(stderr, "%s", tmsgstr); fflush(stderr); }
 
     #if GASNET_DEBUG_VERBOSE
       fprintf(stderr,"gasnetc_init(): spawn successful - node %i/%i starting...\n", 
@@ -189,7 +201,7 @@ static int gasnetc_init(int *argc, char ***argv) {
     gasneti_nodemapInit(&gasnetc_bootstrapExchange, NULL, 0, 0);
 
     #if GASNET_PSHM
-      gasneti_pshm_init(&gasnetc_bootstrapExchange, 0);
+      gasneti_pshm_init(&gasnetc_bootstrapSNodeBroadcast, 0);
     #endif
  
     #if GASNET_SEGMENT_FAST || GASNET_SEGMENT_LARGE
@@ -232,56 +244,14 @@ extern int gasnet_init(int *argc, char ***argv) {
   return GASNET_OK;
 }
 /* ------------------------------------------------------------------------------------ */
-static char checkuniqhandler[256] = { 0 };
-static int gasnetc_reghandlers(gasnet_handlerentry_t *table, int numentries,
-                               int lowlimit, int highlimit,
-                               int dontcare, int *numregistered) {
-  int i;
-  *numregistered = 0;
-  for (i = 0; i < numentries; i++) {
-    int newindex;
-
-    if ((table[i].index == 0 && !dontcare) || 
-        (table[i].index && dontcare)) continue;
-    else if (table[i].index) newindex = table[i].index;
-    else { /* deterministic assignment of dontcare indexes */
-      for (newindex = lowlimit; newindex <= highlimit; newindex++) {
-        if (!checkuniqhandler[newindex]) break;
-      }
-      if (newindex > highlimit) {
-        char s[255];
-        snprintf(s, sizeof(s), "Too many handlers. (limit=%i)", highlimit - lowlimit + 1);
-        GASNETI_RETURN_ERRR(BAD_ARG, s);
-      }
-    }
-
-    /*  ensure handlers fall into the proper range of pre-assigned values */
-    if (newindex < lowlimit || newindex > highlimit) {
-      char s[255];
-      snprintf(s, sizeof(s), "handler index (%i) out of range [%i..%i]", newindex, lowlimit, highlimit);
-      GASNETI_RETURN_ERRR(BAD_ARG, s);
-    }
-
-    /* discover duplicates */
-    if (checkuniqhandler[newindex] != 0) 
-      GASNETI_RETURN_ERRR(BAD_ARG, "handler index not unique");
-    checkuniqhandler[newindex] = 1;
-
-    /* register the handler */
-    if (AM_SetHandler(gasnetc_endpoint, (handler_t)newindex, table[i].fnptr) != AM_OK) 
-      GASNETI_RETURN_ERRR(RESOURCE, "AM_SetHandler() failed while registering handlers");
+extern int gasnetc_amregister(gasnet_handler_t index, gasneti_handler_fn_t fnptr) {
+  if (AM_SetHandler(gasnetc_endpoint, (handler_t)index, fnptr) != AM_OK)
+    GASNETI_RETURN_ERRR(RESOURCE, "AM_SetHandler() failed while registering handlers");
 #if GASNET_PSHM
-    /* Maintain a shadown handler table for AMPSHM */
-    gasnetc_handler[(gasnet_handler_t)newindex] = (gasneti_handler_fn_t)table[i].fnptr;
+  /* Maintain a shadown handler table for AMPSHM */
+  gasneti_assert(gasnetc_handler[index] == gasneti_defaultAMHandler);
+  gasnetc_handler[index] = fnptr;
 #endif
-
-    /* The check below for !table[i].index is redundant and present
-     * only to defeat the over-aggressive optimizer in pathcc 2.1
-     */
-    if (dontcare && !table[i].index) table[i].index = newindex;
-
-    (*numregistered)++;
-  }
   return GASNET_OK;
 }
 /* ------------------------------------------------------------------------------------ */
@@ -290,8 +260,8 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
   int retval = GASNET_OK;
   void *segbase = NULL;
   
-  GASNETI_TRACE_PRINTF(C,("gasnetc_attach(table (%i entries), segsize=%lu, minheapoffset=%lu)",
-                          numentries, (unsigned long)segsize, (unsigned long)minheapoffset));
+  GASNETI_TRACE_PRINTF(C,("gasnetc_attach(table (%i entries), segsize=%"PRIuPTR", minheapoffset=%"PRIuPTR")",
+                          numentries, segsize, minheapoffset));
   AMLOCK();
     if (!gasneti_init_done) 
       INITERR(NOT_INIT, "GASNet attach called before init");
@@ -334,7 +304,7 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
       int numreg = 0;
       gasneti_assert(ctable);
       while (ctable[len].fnptr) len++; /* calc len */
-      if (gasnetc_reghandlers(ctable, len, 1, 63, 0, &numreg) != GASNET_OK)
+      if (gasneti_amregister(ctable, len, 1, 63, 0, &numreg) != GASNET_OK)
         INITERR(RESOURCE,"Error registering core API handlers");
       gasneti_assert(numreg == len);
     }
@@ -345,7 +315,7 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
       int numreg = 0;
       gasneti_assert(etable);
       while (etable[len].fnptr) len++; /* calc len */
-      if (gasnetc_reghandlers(etable, len, 64, 127, 0, &numreg) != GASNET_OK)
+      if (gasneti_amregister(etable, len, 64, 127, 0, &numreg) != GASNET_OK)
         INITERR(RESOURCE,"Error registering extended API handlers");
       gasneti_assert(numreg == len);
     }
@@ -355,11 +325,11 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
       int numreg2 = 0;
 
       /*  first pass - assign all fixed-index handlers */
-      if (gasnetc_reghandlers(table, numentries, 128, 255, 0, &numreg1) != GASNET_OK)
+      if (gasneti_amregister(table, numentries, 128, 255, 0, &numreg1) != GASNET_OK)
         INITERR(RESOURCE,"Error registering fixed-index client handlers");
 
       /*  second pass - fill in dontcare-index handlers */
-      if (gasnetc_reghandlers(table, numentries, 128, 255, 1, &numreg2) != GASNET_OK)
+      if (gasneti_amregister(table, numentries, 128, 255, 1, &numreg2) != GASNET_OK)
         INITERR(RESOURCE,"Error registering variable-index client handlers");
 
       gasneti_assert(numreg1 + numreg2 == numentries);
@@ -428,7 +398,10 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
 
   GASNETI_TRACE_PRINTF(C,("gasnetc_attach(): primary attach complete\n"));
 
-  gasneti_auxseg_attach(); /* provide auxseg */
+  /* (###) exchange_fn is optional (may be NULL) and is only used with GASNET_SEGMENT_EVERYTHING
+           if your conduit has an optimized bootstrapExchange pass it in place of NULL
+   */
+  gasneti_auxseg_attach(gasnetc_bootstrapExchange); /* provide auxseg */
 
   gasnete_init(); /* init the extended API */
 
@@ -515,6 +488,10 @@ extern void gasnetc_exit(int exitcode) {
    }
   }
 
+  #if GASNET_PSHM
+    gasneti_pshm_fini();
+  #endif
+
   AMMPI_SPMDExit(exitcode);
   gasneti_fatalerror("AMMPI_SPMDExit failed");
 }
@@ -567,7 +544,14 @@ extern int gasnetc_AMPoll(void) {
   gasneti_AMPSHMPoll(0);
 #endif
   AMLOCK();
+  static int cntr;
+  // In single-supernode case never need to poll the network for client AMs.
+  // However, we'll still AM_Poll() every 256th call for orderly exit handling.
+  if ((gasneti_mysupernode.grp_count > 1) || !(0xff & cntr++)) {
     GASNETI_AM_SAFE_NORETURN(retval, AM_Poll(gasnetc_bundle));
+  } else {
+    retval = 0;
+  }
   AMUNLOCK();
   if_pf (retval) GASNETI_RETURN_ERR(RESOURCE);
   else return GASNET_OK;
@@ -590,6 +574,7 @@ extern int gasnetc_AMRequestShortM(
   va_start(argptr, numargs); /*  pass in last argument */
 #if GASNET_PSHM
   if_pt (gasneti_pshm_in_supernode(dest)) {
+    gasneti_AMPoll(); /* poll at least once, to assure forward progress */
     retval = gasneti_AMPSHM_RequestGeneric(gasnetc_Short, dest, handler, 
                                            0, 0, 0,
                                            numargs, argptr); 
@@ -619,14 +604,13 @@ extern int gasnetc_AMRequestMediumM(
   va_start(argptr, numargs); /*  pass in last argument */
 #if GASNET_PSHM
   if_pt (gasneti_pshm_in_supernode(dest)) {
+    gasneti_AMPoll(); /* poll at least once, to assure forward progress */
     retval = gasneti_AMPSHM_RequestGeneric(gasnetc_Medium, dest, handler, 
                                            source_addr, nbytes, 0,
                                            numargs, argptr);
   } else
 #endif
   { 
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AMLOCK_TOSEND();
       GASNETI_AM_SAFE_NORETURN(retval,
                AMMPI_RequestIVA(gasnetc_endpoint, dest, handler, 
@@ -652,6 +636,7 @@ extern int gasnetc_AMRequestLongM( gasnet_node_t dest,        /* destination nod
   va_start(argptr, numargs); /*  pass in last argument */
 #if GASNET_PSHM
   if_pt (gasneti_pshm_in_supernode(dest)) {
+      gasneti_AMPoll(); /* poll at least once, to assure forward progress */
       retval = gasneti_AMPSHM_RequestGeneric(gasnetc_Long, dest, handler, 
                                              source_addr, nbytes, dest_addr,
                                              numargs, argptr);
@@ -661,8 +646,6 @@ extern int gasnetc_AMRequestLongM( gasnet_node_t dest,        /* destination nod
     uintptr_t dest_offset;
     dest_offset = ((uintptr_t)dest_addr) - ((uintptr_t)gasneti_seginfo[dest].addr);
 
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AMLOCK_TOSEND();
       GASNETI_AM_SAFE_NORETURN(retval,
                AMMPI_RequestXferVA(gasnetc_endpoint, dest, handler, 
@@ -720,8 +703,6 @@ extern int gasnetc_AMReplyMediumM(
   } else
 #endif
   {
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AM_ASSERT_LOCKED();
     GASNETI_AM_SAFE_NORETURN(retval,
               AMMPI_ReplyIVA(token, handler, source_addr, nbytes, numargs, argptr));
@@ -758,8 +739,6 @@ extern int gasnetc_AMReplyLongM(
     GASNETI_SAFE_PROPAGATE(gasnet_AMGetMsgSource(token, &dest));
     dest_offset = ((uintptr_t)dest_addr) - ((uintptr_t)gasneti_seginfo[dest].addr);
 
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AM_ASSERT_LOCKED();
     GASNETI_AM_SAFE_NORETURN(retval,
               AMMPI_ReplyXferVA(token, handler, source_addr, nbytes, dest_offset, numargs, argptr));
