@@ -14,8 +14,6 @@
 
 #include <gasnet_core_help.h>
 
-GASNETI_BEGIN_EXTERNC
-
 /* ------------------------------------------------------------------------------------ */
 /*
   Initialization
@@ -53,8 +51,8 @@ extern void gasnetc_resume_interrupts(void);
 #define gasnet_hold_interrupts    gasnetc_hold_interrupts
 #define gasnet_resume_interrupts  gasnetc_resume_interrupts
 #else
-#define gasnet_hold_interrupts()
-#define gasnet_resume_interrupts()
+#define gasnet_hold_interrupts()   ((void)0)
+#define gasnet_resume_interrupts() ((void)0)
 #endif
 
 /* ------------------------------------------------------------------------------------ */
@@ -73,7 +71,7 @@ typedef struct _gasnet_hsl_t {
     /* more state may be required for conduits using interrupts */
 #error interrupts not implemented
 #endif
-} gasnet_hsl_t GASNETI_THREAD_TYPEDEF;
+} gasnet_hsl_t;
 
 #if GASNETI_STATS_OR_TRACE
 #define GASNETC_LOCK_STAT_INIT ,0
@@ -172,15 +170,7 @@ size_t gasneti_AMMaxMedium();
 #endif
 
 #define gasnet_AMMaxArgs()          ((size_t)GASNETC_MAX_ARGS)
-#if GASNET_PSHM
-/* (###) If supporting PSHM a conduit must "negotiate" the maximum size of a
- * Medium message.  This can either be done by lowering the conduit's value to
- * the default PSHM value (as shown here), or GASNETI_MAX_MEDIUM_PSHM can be
- * defined in gasnet_core_fwd.h to give the conduit complete control. */
-#define gasnet_AMMaxMedium()      ((size_t)MIN(GASNETC_MAX_MEDIUM, GASNETI_MAX_MEDIUM_PSHM))
-#else
-#define gasnet_AMMaxMedium()      ((size_t)GASNETC_MAX_MEDIUM)
-#endif
+#define gasnet_AMMaxMedium()        ((size_t)MIN(GASNETC_MAX_MEDIUM, GASNETC_MAX_MEDIUM_PSHM_DFLTMAX))
 
 #define gasnet_AMMaxLongRequest()   ((size_t)GASNETC_MAX_LONG_REQ)
 #define gasnet_AMMaxLongReply()     ((size_t)GASNETC_MAX_LONG_REP)
@@ -205,8 +195,6 @@ extern void gasnetc_barrier_fence(void);
 
 /* ------------------------------------------------------------------------------------ */
 
-GASNETI_END_EXTERNC
-
 #endif
 
 #include <gasnet_ammacros.h>
