@@ -3,65 +3,149 @@
  * Copyright 2000, Dan Bonachea <bonachea@cs.berkeley.edu>
  */
 
-#include <amudp_internal.h>
-#include <amudp_spmd.h>
+#undef _PORTABLE_PLATFORM_H
+#include <amx_portable_platform.h>
 
 #include <errno.h>
-#if PLATFORM_OS_MSWINDOWS
-  #include <winsock2.h>
-  #include <windows.h>  
-  #define sleep(x) Sleep(1000*x)
-  #include <process.h>
-  #include <io.h>
-  #include <direct.h>
+#if PLATFORM_OS_CYGWIN
+  #include <windows.h>  // CreateProcess
 #endif
 #if PLATFORM_ARCH_CRAYX1
   #include <unistd.h>
   #include <sys/prctl.h>
   extern char **environ; 
 #endif
+#include <string.h>
+
+#include <amudp_spmd.h>
+
+#include "amudp_internal.h" // must come after any other headers
 
 amudp_spawnfn_desc_t const AMUDP_Spawnfn_Desc[] = {
   { 'S',  "Spawn jobs using ssh remote shells", 
     (amudp_spawnfn_t)AMUDP_SPMDSshSpawn },
   { 'L',  "Spawn jobs using fork()/exec() on the local machine (good for SMP's)", 
     (amudp_spawnfn_t)AMUDP_SPMDLocalSpawn },
-#ifdef REXEC
-  { 'R',  "Spawn jobs using rexec (Berkeley Millennium only)", 
-    (amudp_spawnfn_t)AMUDP_SPMDRexecSpawn },
-#endif
-#ifdef GLUNIX
-  { 'G',  "Spawn jobs using GLUNIX (Berkeley NOW only)", 
-    (amudp_spawnfn_t)AMUDP_SPMDGlunixSpawn },
-#endif
-  { 'C',  "Spawn jobs using custom job spawner (" AMUDP_ENV_PREFIX_STR"_CSPAWN_CMD)", 
+  { 'C',  "Spawn jobs using custom job spawner (" AMX_ENV_PREFIX_STR"_CSPAWN_CMD)", 
     (amudp_spawnfn_t)AMUDP_SPMDCustomSpawn },
   { '\0', NULL, (amudp_spawnfn_t)NULL }
 };
 
+/* Utility functions */
+// Apply single quotes to prevent shell from evaluating special chars
+char *quote_for_remote(const char *arg) {
+  const char *p;
+  int c;
+
+  for (p=arg, c=0; NULL != (p = strchr(p, '\'')); ++c, ++p) { /*empty*/ }
+
+  size_t old_len = strlen(arg);
+  size_t new_len = old_len + 2 + 3 * c;
+  char *result = (char *)AMX_malloc(1 + new_len);
+  result[0] = '\'';
+  result[1] = '\0';
+  char *end = result+1;
+
+  if (c) {
+    char *dup = (char *)AMX_malloc(1 + old_len);
+    p = strcpy(dup, arg);
+    while (c--) {
+      char *q = strchr((char *)p, '\'');
+      *q = '\0';
+      strcpy(end, p);       end += q-p;
+      strcpy(end, "'\\''"); end += 4;
+      p = q + 1;
+    }
+    AMX_free(dup);
+  } else {
+    p = arg;
+  }
+  strcpy(end, p);
+
+  AMX_assert(strlen(result) == new_len-1);
+  result[new_len-1] = '\'';
+  result[new_len]   = '\0';
+
+  return result;
+}
+// Apply quotes to protect against the shell spawned by system()
+// Note this is for use *inside* the double quotes we place around a command run by system.
+// shell docs tells us:
+//   Inside double-quotes, '$' and '`' remain special unconditionally,
+//   and '\' remains special only if followed by another special
+char *quote_for_local(const char *arg) {
+  const char specials[] = "$`\\\"";
+
+  char *tmp = quote_for_remote(arg);
+
+  // count specials (assume every backslash is special)
+  const char *p;
+  int c;
+  for (p=tmp, c=0; NULL != (p = strpbrk(p, specials)); ++c, ++p) { /*empty*/ }
+
+  if (!c) return tmp; // Nothing to protect from local shell
+
+  size_t old_len = strlen(tmp);
+  size_t new_len = old_len + c;
+  char *result = (char *)AMX_malloc(1 + new_len);
+  char *end = result;
+
+  char *q, *dup = (char *)AMX_malloc(1 + old_len);
+  p = strcpy(dup, tmp);
+  while (NULL != (q = strpbrk((char *)p, specials))) {
+    size_t len = q-p;
+    strncpy(end, p, len);   end += len;
+    end[0] = '\\';          end += 1;
+    if (q[0] != '\\' || strchr(specials, q[1])) {
+      end[0] = q[0];        end += 1;
+    }
+    p = q + 1;
+  }
+  strcpy(end, p);
+  AMX_free(dup);
+  AMX_free(tmp);
+
+  AMX_assert(strlen(result) <= new_len);
+
+  return result;
+}
+
 /* ------------------------------------------------------------------------------------ 
  *  spawn jobs on local machine
  * ------------------------------------------------------------------------------------ */
-extern int AMUDP_SPMDLocalSpawn(int nproc, int argc, char **argv) {
+extern int AMUDP_SPMDLocalSpawn(int nproc, int argc, char **argv, char **extra_env) {
   /* just a simple fork/exec */
   int i;
 
   if (!AMUDP_SPMDSpawnRunning) {
-    AMUDP_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
+    AMX_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
     return FALSE;
   }
 
+  /* Temporarily assume values from extra_env[] (which we modify in-place) */
+  char **save_env = NULL;
+  int envc = 0;
+  if (extra_env && extra_env[0]) {
+    for (envc=0; extra_env[envc]; ++envc) {/*empty*/}
+    save_env = (char **)AMX_malloc(sizeof(char *)*envc);
+    for (i=0;i<envc;++i) {
+      char *var = extra_env[i];
+      char *delim = strchr(var,'=');
+      AMX_assert(delim);
+      *delim = '\0';
+      save_env[i] = getenv(var);
+      setenv(var,delim+1,1);
+    }
+  }
+
   for (i = 0; i < nproc; i++) {
-    #if PLATFORM_OS_MSWINDOWS && !PLATFORM_OS_CYGWIN
-      if (_spawnv(_P_NOWAIT, argv[0], argv) == -1)
-        AMUDP_FatalErr("failed _spawnv()");
-    #elif PLATFORM_ARCH_CRAYX1
-      { char **nargv = (char **)AMUDP_malloc(sizeof(char *)*(argc+2));
+    #if PLATFORM_ARCH_CRAYX1
+      { char **nargv = (char **)AMX_malloc(sizeof(char *)*(argc+2));
         nargv[0] = argv[0];
         memcpy(nargv+1,argv,argc*sizeof(char *));
         nargv[argc+1] = NULL;
         if (execsp(nargv, environ, NULL) == -1)
-          AMUDP_FatalErr("failed execsp()");
+          AMX_FatalErr("failed execsp()");
       }
     #else
       int forkRet = fork();
@@ -86,125 +170,22 @@ extern int AMUDP_SPMDLocalSpawn(int nproc, int argc, char **argv) {
       } /*  child */
     #endif
   }
+
+  /* Restore saved environment var(s) */
+  for (i=0;i<envc;++i) {
+    char *name = extra_env[i];
+    if (save_env[i]) {
+      setenv(name,save_env[i],1);
+    } else {
+      unsetenv(name);
+    }
+    /* Revert our in-place modification of extra_env[]: */
+    name[strlen(name)] = '=';
+  }
+  AMX_free(save_env);
+
   return TRUE;
 }
-/* ------------------------------------------------------------------------------------ 
- *  spawn jobs using Glunix (requires GLUNIX be defined when library is built)
- * ------------------------------------------------------------------------------------ */
-#ifdef GLUNIX
-  #include <glib.h>
-  extern int AMUDP_SPMDGlunixSpawn(int nproc, int argc, char **argv) {
-    /* GLUNIX has good built-in facilities for remote spawn */
-    int forkRet;
-
-    if (!AMUDP_SPMDSpawnRunning) {
-      AMUDP_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
-      return FALSE;
-    }
-
-    forkRet = fork();
-    if (forkRet == -1) {
-      perror("fork");
-      return FALSE;
-    } else if (forkRet != 0) {
-      AMUDP_SPMDRedirectStdsockets = FALSE; /* GLUNIX has it's own console redirection facilities */
-      return TRUE;  
-    } else {  /*  this is the child - exec the new process */
-      if (!Glib_Initialize())
-        AMUDP_FatalErr("failed to Glib_Initialize()")
-      /*AMUDP_assert(Glib_AmIStartup() > 0);*/
-      Glib_Spawn(nproc, argv[0], argv); /* should never return */
-      AMUDP_FatalErr("failed to Glib_Spawn()")
-      return FALSE;
-    }
-  }
-#else
-  extern int AMUDP_SPMDGlunixSpawn(int nproc, int argc, char **argv) {
-    AMUDP_Err("AMUDP_SPMDGlunixSpawn() cannot be called because the AMUDP library was compiled without -DGLUNIX.\n"
-             "  Please recompile libAMUDP.a with -DGLUNIX, relink your application and try again.");
-    return FALSE;
-  }
-#endif
-/* ------------------------------------------------------------------------------------ 
- *  spawn jobs using REXEC utility (Berkeley Millennium project)
- * ------------------------------------------------------------------------------------ */
-/* rexec provides decent support for remote spawn
- * see http://www.millennium.berkeley.edu/rexec/ for instructions on setting up your environment
- */
-/* rexec spawn uses the following environment variables:
- *
- *  option         default       description
- * --------------------------------------------------
- * REXEC_SVRS      none          list of servers to use
- * REXEC_CMD       "rexec"       rexec command to use
- * REXEC_OPTIONS   "-q"            additional options to give rexec 
- * (any environment variable may be specified with an optional prefix 
- *  of AMUDP_ or AMUDP_ENV_PREFIX##_)
- */
-#ifdef REXEC
-  int AMUDP_SPMDRexecSpawn(int nproc, int argc, char **argv) {
-    int i;
-    char *rexec_cmd;
-    char *rexec_options;
-    char cmd1[1024],cmd2[1024];
-    int pid;
-
-    if (!AMUDP_SPMDSpawnRunning) {
-      AMUDP_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
-      return FALSE;
-    }
-
-    pid = getpid();
-
-    rexec_cmd = AMUDP_getenv_prefixed_withdefault("REXEC_CMD", "rexec");
-    rexec_options = AMUDP_getenv_prefixed_withdefault("REXEC_OPTIONS", "-q");
-
-    cmd1[0] = '\0';
-    for (i = 0; i < argc; i++) {
-      AMUDP_assert(argv[i]);
-      strcat(cmd1,"'");
-      strcat(cmd1,argv[i]);
-      strcat(cmd1,"' ");
-    }
-    AMUDP_assert(!argv[i]);
-
-    /* build the rexec command */
-    sprintf(cmd2, "%s %s -n %i /bin/sh -c \"%s%s\" " /* shell wrapper required because of crappy rexec implementation */
-     " || ( echo \"rexec spawn failed.\" ; kill %i ) ",
-      rexec_cmd, rexec_options, nproc,
-
-      (AMUDP_SilentMode?"":"echo connected to \\`uname -n\\`... ; "),
-
-      cmd1, pid
-
-    );
-
-    { int forkRet;
-      forkRet = fork(); /* fork a new process to hold rexec master */
-
-      if (forkRet == -1) {
-        perror("fork");
-        return FALSE;
-      } else if (forkRet != 0) {
-        AMUDP_SPMDRedirectStdsockets = FALSE; /* REXEC has it's own console redirection facilities */
-        return TRUE;  
-      } else {  /*  this is the child - exec the new process */
-        if (!AMUDP_SilentMode) 
-          printf("system(%s)\n", cmd2); fflush(stdout);
-        if (system(cmd2) == -1)
-           AMUDP_FatalErr("Failed to call system() to spawn rexec");
-        exit(0);
-      }
-    }
-    return TRUE;
-}
-#else
-  extern int AMUDP_SPMDRexecSpawn(int nproc, int argc, char **argv) {
-    AMUDP_Err("AMUDP_SPMDRexecSpawn() cannot be called because the AMUDP library was compiled without -DREXEC.\n"
-             "  Please recompile libAMUDP.a with -DREXEC, relink your application and try again.");
-    return FALSE;
-  }
-#endif
 /* ------------------------------------------------------------------------------------ 
  *  spawn jobs using ssh remote shell
  * ------------------------------------------------------------------------------------ */
@@ -227,10 +208,10 @@ extern int AMUDP_SPMDLocalSpawn(int nproc, int argc, char **argv) {
  * SSH_OPTIONS  ""                          additional options to give ssh
  * SSH_REMOTE_PATH  current working dir.    the directory to use on remote machine
  * (any environment variable may be specified with an optional prefix 
- *  of AMUDP_ or AMUDP_ENV_PREFIX##_)
+ *  of AMUDP_ or AMX_ENV_PREFIX##_)
  */
 
-int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
+int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv, char **extra_env) {
   int i;
   const char *ssh_servers;
   const char *ssh_cmd;
@@ -243,7 +224,7 @@ int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
   int pid;
 
   if (!AMUDP_SPMDSpawnRunning) {
-    AMUDP_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
+    AMX_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
     return FALSE;
   }
 
@@ -251,21 +232,21 @@ int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
 
   ssh_servers = AMUDP_getenv_prefixed_withdefault("SSH_SERVERS","");
   if (!strlen(ssh_servers)) {
-    printf("Environment variable SSH_SERVERS is missing.\n");
+    AMX_Err("Environment variable SSH_SERVERS is missing.");
     return FALSE;
   }
 
 
   if (!getcwd(cwd, 1024)) {
-    printf("Error calling getcwd()\n");
+    AMX_Err("Error calling getcwd()");
     return FALSE;
   }
-  ssh_remote_path = AMUDP_getenv_prefixed_withdefault("SSH_REMOTE_PATH", cwd);
+  ssh_remote_path = quote_for_local(AMUDP_getenv_prefixed_withdefault("SSH_REMOTE_PATH", cwd));
   ssh_cmd = AMUDP_getenv_prefixed_withdefault("SSH_CMD", "ssh");
 
   int isOpenSSH = 0; /* figure out if we're using OpenSSH */
   { char cmdtmp[1024];
-    sprintf(cmdtmp,"%s -v 2>&1 | grep OpenSSH", ssh_cmd);
+    sprintf(cmdtmp,"%s -V 2>&1 | grep OpenSSH", ssh_cmd);
     FILE *pop = popen(cmdtmp,"r");
     while (!feof(pop) && !ferror(pop)) {
       int next = fgetc(pop);
@@ -285,30 +266,49 @@ int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
     while (*p && strchr(SSH_SERVERS_DELIM_CHARS, *p)) p++;
     end = p + strcspn(p, SSH_SERVERS_DELIM_CHARS);
     if (p == end) {
-      printf("Not enough machines in environment variable SSH_SERVERS to satisfy request for (%i).\n"
-       "Only (%i) machines available: %s\n", nproc, i, ssh_servers);
+      AMX_Err("Not enough machines in environment variable SSH_SERVERS to satisfy request for (%i).\n"
+       "Only (%i) machines available: %s", nproc, i, ssh_servers);
       return FALSE;
     }
     if (*end) p = end+1;
     else p = end;
   } 
 
+  const char *envcmd = AMUDP_getenv_prefixed_withdefault("ENV_CMD", "env");
+
   cmd1_sz = 1; /* terminating nul */
-  for (i = 0; i < argc; i++) {
-    cmd1_sz += strlen(argv[i]) + 3; /* "'%s' " */
-  }
-  cmd1 = (char *)AMUDP_malloc(cmd1_sz);
-  { char *tmp = cmd1;
-    for (i = 0; i < argc; i++) {
-      AMUDP_assert(argv[i]);
-      tmp += sprintf(tmp, "'%s' ", argv[i]);
+  if (extra_env && extra_env[0]) {
+    cmd1_sz += 1 + strlen(envcmd); // "env "
+    for (i=0; extra_env[i]; ++i) {
+      cmd1_sz += strlen(extra_env[i]) + 3; // "'%s' "
     }
-    AMUDP_assert(!argv[i]);
-    *tmp = '\0';
-    AMUDP_assert(strlen(cmd1) == cmd1_sz - 1);
   }
-  cmd2_sz = cmd1_sz + 1024; /* estimated */
-  cmd2 =  (char *)AMUDP_malloc(cmd2_sz);
+  char **quoted_args = (char**)AMX_malloc(argc * sizeof(char*));
+  for (i = 0; i < argc; i++) {
+    AMX_assert(argv[i]);
+    quoted_args[i] = quote_for_local(argv[i]);
+    cmd1_sz += strlen(quoted_args[i]) + 1; // +1 for trailing space
+  }
+  cmd1 = (char *)AMX_malloc(cmd1_sz);
+  { char *tmp = cmd1;
+    if (extra_env && extra_env[0]) {
+      tmp += sprintf(tmp, "%s ", envcmd);
+      for (i=0; extra_env[i]; ++i) {
+        tmp += sprintf(tmp, "'%s' ", extra_env[i]);
+      }
+    }
+    for (i = 0; i < argc; i++) {
+      AMX_assert(quoted_args[i]);
+      tmp += sprintf(tmp, "%s ", quoted_args[i]);
+      AMX_free(quoted_args[i]);
+    }
+    AMX_free(quoted_args);
+    AMX_assert(!argv[i]);
+    *tmp = '\0';
+    AMX_assert(strlen(cmd1) == cmd1_sz - 1);
+  }
+  cmd2_sz = cmd1_sz + strlen(ssh_remote_path) + 1024; /* estimated */
+  cmd2 =  (char *)AMX_malloc(cmd2_sz);
 
   p = ssh_servers;
   for (i = 0; i < nproc; i++) {
@@ -316,34 +316,34 @@ int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
     const char *end;
     while (*p && strchr(SSH_SERVERS_DELIM_CHARS, *p)) p++;
     end = p + strcspn(p, SSH_SERVERS_DELIM_CHARS);
-    AMUDP_assert(p != end);
+    AMX_assert(p != end);
 
     strncpy(ssh_server, p, (end-p));
     ssh_server[end-p] = '\0'; 
 
     /* build the ssh command */
-    snprintf(cmd2, cmd2_sz, "%s %s %s %s %s %s \" %s cd '%s' ; %s\" "
+    snprintf(cmd2, cmd2_sz, "%s %s%s%s%s %s \"%scd %s ; %s\" "
       " || ( echo \"connection to %s failed.\" ; kill %i ) "
       "%s", 
       ssh_cmd,
 
-      (isOpenSSH?"-f":""),    /* go into background and nullify stdin */
+      (isOpenSSH?"-f ":""),    /* go into background and nullify stdin */
 
       #if SSH_SUPRESSNEWKEYPROMPT
-        (isOpenSSH?"-o 'StrictHostKeyChecking no'":""),
+        (isOpenSSH?"-o 'StrictHostKeyChecking no' ":""),
       #else 
         "",
       #endif
 
       #if SSH_PREVENTRSHFALLBACK
-        (isOpenSSH?"-o 'FallBackToRsh no'":""),
+        (isOpenSSH?"-o 'FallBackToRsh no' ":""),
       #else 
         "",
       #endif
 
       ssh_options, ssh_server, 
       
-      (AMUDP_SilentMode?"":"echo connected to \\$HOST... ;"),
+      (AMX_SilentMode?"":"echo connected to \\$HOST... ; "),
 
       ssh_remote_path, cmd1, ssh_server, pid,
 
@@ -354,20 +354,20 @@ int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
       #endif
     );
 
-    if (!AMUDP_SilentMode) 
-      printf("system(%s)\n", cmd2); fflush(stdout);
+    if (!AMX_SilentMode) 
+      AMX_Info("system(%s)", cmd2);
     if (system(cmd2) == -1) {
-      printf("Failed to call system() to spawn");
-      AMUDP_free(cmd1);
-      AMUDP_free(cmd2);
+      AMX_Err("Failed to call system() to spawn");
+      AMX_free(cmd1);
+      AMX_free(cmd2);
       return FALSE;
     }
     if (*end) p = end+1;
     else p = end;
   } 
 
-  AMUDP_free(cmd1);
-  AMUDP_free(cmd2);
+  AMX_free(cmd1);
+  AMX_free(cmd2);
   return TRUE;
 }
 /* ------------------------------------------------------------------------------------ 
@@ -384,12 +384,12 @@ int AMUDP_SPMDSshSpawn(int nproc, int argc, char **argv) {
  *                              %C => AMUDP command that should be run by each worker node
  *                              %D => the current working directory
  *                              %% => %
- * CSPAWN_ROUTE_OUTPUT <not set>   set this variable to request stdout/stderr routing of workers
+ * CSPAWN_ROUTE_OUTPUT 0   set this variable to request stdout/stderr routing of workers
  * (any environment variable may be specified with an optional prefix 
- *  of AMUDP_ or AMUDP_ENV_PREFIX##_)
+ *  of AMUDP_ or AMX_ENV_PREFIX##_)
  */
 
-int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
+int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv, char **extra_env) {
   int i;
   char *spawn_cmd = NULL;
   char *spawn_servers = NULL;
@@ -398,14 +398,14 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
   char nproc_str[10];
   char cwd[1024];
   char cmd[1024];
-#if PLATFORM_OS_MSWINDOWS || PLATFORM_OS_CYGWIN
+#if PLATFORM_OS_CYGWIN
   int spawn_use_create_process;
 #endif
   int spawn_route_output;
   int pid;
 
   if (!AMUDP_SPMDSpawnRunning) {
-    AMUDP_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
+    AMX_Err("Spawn functions should never be run directly - only passed to AMUDP_SPMDStartup()"); 
     return FALSE;
   }
 
@@ -413,14 +413,14 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
 
   spawn_cmd = AMUDP_getenv_prefixed_withdefault("CSPAWN_CMD","");
   if (!strlen(spawn_cmd)) {
-    AMUDP_Err("You must set the " AMUDP_ENV_PREFIX_STR"_CSPAWN_CMD environment variable to use the custom spawn function"); 
+    AMX_Err("You must set the " AMX_ENV_PREFIX_STR"_CSPAWN_CMD environment variable to use the custom spawn function"); 
     return FALSE;
   }
   spawn_servers = AMUDP_getenv_prefixed_withdefault("CSPAWN_SERVERS","");
   if (!strlen(spawn_servers)) spawn_servers = NULL;
   spawn_route_output = 
     strcmp(AMUDP_getenv_prefixed_withdefault("CSPAWN_ROUTE_OUTPUT","0"),"0");
-#if PLATFORM_OS_MSWINDOWS || PLATFORM_OS_CYGWIN
+#if PLATFORM_OS_CYGWIN
   spawn_use_create_process =
     strcmp(AMUDP_getenv_prefixed_withdefault("CSPAWN_USE_CREATE_PROCESS","0"),"0");
 #endif
@@ -437,8 +437,8 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
       while (*p && strchr(SSH_SERVERS_DELIM_CHARS, *p)) p++;
       end = p + strcspn(p, SSH_SERVERS_DELIM_CHARS);
       if (p == end) {
-        printf("Not enough machines in environment variable " AMUDP_ENV_PREFIX_STR"_CSPAWN_SERVERS to satisfy request for (%i).\n"
-          "Only (%i) machines available: %s\n", nproc, i, spawn_servers);
+        AMX_Err("Not enough machines in environment variable " AMX_ENV_PREFIX_STR"_CSPAWN_SERVERS to satisfy request for (%i).\n"
+          "Only (%i) machines available: %s", nproc, i, spawn_servers);
         return FALSE;
       }
       strncpy(servername, p, end-p);
@@ -454,29 +454,42 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
   sprintf(nproc_str, "%i", nproc);
 
   if (!getcwd(cwd, 1024)) {
-    printf("Error calling getcwd()\n");
+    AMX_Err("Error calling getcwd()");
     return FALSE;
   }
 
     /* build the worker command */
-  { char temp[1024];
+  { char temp[2048]; // TODO: allocate dynamically
     temp[0] = '\0';
-    for (i = 0; i < argc; i++) {
-      AMUDP_assert(argv[i] != NULL);
-      strcat(temp,"'");
-      strcat(temp,argv[i]);
-      strcat(temp,"' ");
+    if (extra_env && extra_env[0]) {
+      const char *envcmd = AMUDP_getenv_prefixed_withdefault("ENV_CMD", "env");
+      strcat(temp,envcmd);
+      strcat(temp," ");
+      for (i=0; extra_env[i]; ++i) {
+        strcat(temp,"'");
+        strcat(temp,extra_env[i]);
+        strcat(temp,"' ");
+      }
     }
-    AMUDP_assert(!argv[i]);
+    for (i = 0; i < argc; i++) {
+      AMX_assert(argv[i] != NULL);
+      char *qarg = quote_for_local(argv[i]);
+      strcat(temp,qarg);
+      strcat(temp," ");
+      AMX_free(qarg);
+    }
+    AMX_assert(!argv[i]);
 
-  #if PLATFORM_OS_MSWINDOWS || PLATFORM_OS_CYGWIN
-    strcpy(workercmd, temp);
-  #else
+  #if PLATFORM_OS_CYGWIN
+    if (spawn_use_create_process) strcpy(workercmd, temp);
+    else
+  #endif
+   {
     sprintf(workercmd, "/bin/sh -c \"%s%s\" || ( echo \"spawn failed.\" ; kill %i ) ",
-      (AMUDP_SilentMode?"":"echo connected to `uname -n`... ; "),
+      (AMX_SilentMode?"":"echo connected to `uname -n`... ; "),
       temp, pid
     );
-  #endif
+   }
   }
 
   strcpy(cmd, spawn_cmd);
@@ -487,14 +500,15 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
       switch (*(p+1)) {
         case 'M': case 'm': 
           if (!spawn_servers) { /* user failed to provide servers and now is asking for them */
-            AMUDP_Err("You must set the " AMUDP_ENV_PREFIX_STR"_CSPAWN_SERVERS environment "
-                       "variable to use the %%M option in " AMUDP_ENV_PREFIX_STR"_CSPAWN_CMD");
+            AMX_Err("You must set the " AMX_ENV_PREFIX_STR"_CSPAWN_SERVERS environment "
+                       "variable to use the %%M option in " AMX_ENV_PREFIX_STR"_CSPAWN_CMD");
           }
           replacement = workerservers; 
           break;
         case 'N': case 'n': replacement = nproc_str; break;
         case 'C': case 'c': replacement = workercmd; break;
         case 'D': case 'd': replacement = cwd; break;
+        case 'P': case 'p': replacement = argv[0]; break;
         case '%': replacement = "%"; break;
         default: replacement = "";
       }
@@ -510,7 +524,6 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
   AMUDP_SPMDRedirectStdsockets = spawn_route_output; 
 
   {
-  #if !PLATFORM_OS_MSWINDOWS
     int forkRet;
     forkRet = fork(); /* fork a new process to hold cmd master */
 
@@ -522,11 +535,10 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
       return TRUE;  
     }
     else 
-  #endif
     {  /*  this is the child - exec the new process */
-      if (!AMUDP_SilentMode) 
-        printf("system(%s)\n", cmd); fflush(stdout);
-    #if PLATFORM_OS_MSWINDOWS || PLATFORM_OS_CYGWIN
+      if (!AMX_SilentMode) 
+        AMX_Info("system(%s)\n", cmd);
+    #if PLATFORM_OS_CYGWIN
       if (spawn_use_create_process) {
         STARTUPINFO si;
         PROCESS_INFORMATION pi;
@@ -538,15 +550,15 @@ int AMUDP_SPMDCustomSpawn(int nproc, int argc, char **argv) {
         si.hStdOutput = GetStdHandle(STD_ERROR_HANDLE); 
         si.dwFlags |= STARTF_USESTDHANDLES;
 
-        if (   TRUE != CreateProcess(0, strdup(cmd), 0, 0, TRUE, 0, 0, 0, &si, &pi)
+        if (   TRUE != CreateProcess(0, AMX_strdup(cmd), 0, 0, TRUE, 0, 0, 0, &si, &pi)
             || WAIT_FAILED == WaitForSingleObject(pi.hProcess, INFINITE)
             || 0 == GetExitCodeProcess(pi.hProcess, &code)
             || 0 != code)
-          AMUDP_FatalErr("Failed while calling CreateProcess() with custom spawn command:\n%s", cmd);
+          AMX_FatalErr("Failed while calling CreateProcess() with custom spawn command:\n%s", cmd);
       } else
     #endif
       if (system(cmd) != 0)
-         AMUDP_FatalErr("Failed while calling system() with custom spawn command:\n%s", cmd);
+         AMX_FatalErr("Failed while calling system() with custom spawn command:\n%s", cmd);
       exit(0);
     }
   }
