@@ -1262,16 +1262,21 @@ fh_init_plugin(uintptr_t max_pinnable_memory,
                const firehose_region_t *regions, size_t num_prepinned,
 	       firehose_info_t *fhinfo)
 {
-	unsigned long param_M, param_VM;
-	unsigned long param_R, param_VR;
-	unsigned long param_RS;
+	/* Memory sizes: */
+	uintptr_t param_M, param_VM;
+	uintptr_t m_prepinned;
+	/* Region size: */
+	size_t param_RS;
+	/* Region counts: */
+	int param_R, param_VR;
+	int firehoses;
+
 	int i, j;
-	unsigned long firehoses, m_prepinned;
 #if 0  /* UNUSED - see param_RS computation for explanation */
 	int med_regions;
 #endif
 	int b_prepinned = 0;
-	int num_nodes = gasneti_nodes;
+	gasnet_node_t num_nodes = gasneti_nodes;
 	int dflt_M, dflt_VM;
 	int dflt_R, dflt_VR;
 	int dflt_RS;
@@ -1311,11 +1316,11 @@ fh_init_plugin(uintptr_t max_pinnable_memory,
 	param_RS = fh_getenv("GASNET_FIREHOSE_MAXREGION_SIZE", (1<<20));
 	dflt_RS  = !param_RS;
 	GASNETI_TRACE_PRINTF(C, 
-	    ("ENV: Firehose M=%ld, MAXVICTIM_M=%ld", param_M, param_VM));
+	    ("ENV: Firehose M=%"PRIuPTR", MAXVICTIM_M=%"PRIuPTR, param_M, param_VM));
 	GASNETI_TRACE_PRINTF(C, 
-	    ("ENV: Firehose R=%ld, MAXVICTIM_R=%ld", param_R, param_VR));
+	    ("ENV: Firehose R=%d, MAXVICTIM_R=%d", param_R, param_VR));
 	GASNETI_TRACE_PRINTF(C, 
-	    ("ENV: Firehose max region size=%ld", param_RS));
+	    ("ENV: Firehose max region size=%"PRIuPTR, (uintptr_t)param_RS));
 
 	/* Now assign decent "M" defaults based on physical memory */
 	if (param_M == 0 && param_VM == 0) {
@@ -1323,9 +1328,9 @@ fh_init_plugin(uintptr_t max_pinnable_memory,
 			param_M  = m_prepinned;
 			param_VM = max_pinnable_memory - param_M;
 		} else {
-			param_M  = (unsigned long) max_pinnable_memory *
+			param_M  = max_pinnable_memory *
 				(1-FH_MAXVICTIM_TO_PHYSMEM_RATIO);
-			param_VM = (unsigned long) max_pinnable_memory *
+			param_VM = max_pinnable_memory *
 				    FH_MAXVICTIM_TO_PHYSMEM_RATIO;
 		}
 	}
@@ -1398,110 +1403,110 @@ fh_init_plugin(uintptr_t max_pinnable_memory,
 	param_VM = param_RS * param_VR;
 
 	/* Report final values */
-	GASNETI_TRACE_PRINTF(C, ("param_M=%ld param_VM=%ld", param_M, param_VM));
-	GASNETI_TRACE_PRINTF(C, ("param_RS=%ld", param_RS));
-	GASNETI_TRACE_PRINTF(C, ("param_R=%ld param_VR=%ld", param_R, param_VR));
+	GASNETI_TRACE_PRINTF(C, ("param_M=%"PRIuPTR" param_VM=%"PRIuPTR, param_M, param_VM));
+	GASNETI_TRACE_PRINTF(C, ("param_RS=%"PRIuPTR, (uintptr_t)param_RS));
+	GASNETI_TRACE_PRINTF(C, ("param_R=%d param_VR=%d", param_R, param_VR));
 	gasneti_envint_display("GASNET_FIREHOSE_M", param_M, dflt_M, 1);
 	gasneti_envint_display("GASNET_FIREHOSE_MAXVICTIM_M", param_VM, dflt_VM, 1);
 	gasneti_envint_display("GASNET_FIREHOSE_R", param_R, dflt_R, 0);
 	gasneti_envint_display("GASNET_FIREHOSE_MAXVICTIM_R", param_VR, dflt_VR, 0);
-	gasneti_envint_display("GASNET_FIREHOSE_MAXREGION_SIZE", (int)param_RS, dflt_RS, 1);
+	gasneti_envint_display("GASNET_FIREHOSE_MAXREGION_SIZE", param_RS, dflt_RS, 1);
 
 	/* 
 	 * Validate firehose parameters parameters 
 	 */ 
 	if ((fhi_InitFlags & FIREHOSE_INIT_FLAG_LOCAL_ONLY)) {
 		/* Want at least 16MB worth of buckets in victim FIFO */
-		unsigned long	VM_min = (16*1024*1024) / FH_BUCKET_SIZE;
+		uintptr_t	VM_min = (16*1024*1024) / FH_BUCKET_SIZE;
 
 		/* Want at least 32 regions of FIFO */
-		unsigned long	VR_min = 32;
+		int		VR_min = 32;
 
 		if_pf (param_RS < FH_BUCKET_SIZE)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXREGION_SIZE (%ld) "
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXREGION_SIZE (%d) "
 			    "is less than the minimum %d",
-			    param_RS, FH_BUCKET_SIZE); 
+			    (int)param_RS, FH_BUCKET_SIZE); 
 
 		if_pf (param_VM < VM_min)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_M (%ld) is less than "
-			    "the minimum %ld (%ld buckets)",
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_M (%"PRIuPTR") is less than "
+			    "the minimum %"PRIuPTR" (%"PRIuPTR" buckets)",
 			    param_VM, VM_min, VM_min >> FH_BUCKET_SHIFT);
 
 		if_pf (param_VR < VR_min)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_R (%ld) is less than "
-			    "the minimum %ld", param_VR, VR_min);
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_R (%d) is less than "
+			    "the minimum %d", param_VR, VR_min);
 
 		if_pf (param_M < m_prepinned)	/* XXX: need this check? */
 			gasneti_fatalerror("Too many bytes in initial"
-			    " pinned regions list (%ld) for current "
-			    "GASNET_FIREHOSE_M parameter (%ld)", 
+			    " pinned regions list (%"PRIuPTR") for current "
+			    "GASNET_FIREHOSE_M parameter (%"PRIuPTR")", 
 			    m_prepinned, param_M);
 
 		if_pf (param_R < num_prepinned)	/* XXX: need this check? */
 			gasneti_fatalerror("Too many regions passed on initial"
-			    " pinned bucket list (%ld) for current "
-			    "GASNET_FIREHOSE_R parameter (%ld)", 
-			    (unsigned long)num_prepinned, param_R);
+			    " pinned bucket list (%"PRIuPTR") for current "
+			    "GASNET_FIREHOSE_R parameter (%d)", 
+			    (uintptr_t)num_prepinned, param_R);
 	} else {
 		/* Want at least 32 buckets per node */
-		unsigned long	M_min = FH_BUCKET_SIZE * num_nodes * 32;
+		uintptr_t	M_min = FH_BUCKET_SIZE * num_nodes * 32;
 
 		/* Want at least 256 buckets of victim FIFO */
-		unsigned long	VM_min = FH_BUCKET_SIZE * 256;
+		uintptr_t	VM_min = FH_BUCKET_SIZE * 256;
 
 		/* Want at least 1 region per node */
 		/* XXX/PHH THIS IS REALLY A BARE MINIMUM */
-		unsigned long	R_min = num_nodes;
+		int		R_min = num_nodes;
 
 		/* Want at least 2 regions of FIFO */
 		/* XXX/PHH THIS IS REALLY A BARE MINIMUM */
-		unsigned long	VR_min = 2;
+		int		VR_min = 2;
 
 		if_pf (param_RS < FH_BUCKET_SIZE)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXREGION_SIZE (%ld) "
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXREGION_SIZE (%d) "
 			    "is less than the minimum %d",
-			    param_RS, FH_BUCKET_SIZE); 
+			    (int)param_RS, FH_BUCKET_SIZE); 
 
 /* We don't (yet) do accounting against param_R, so the worst case
  * is unpining FH_MAX_UNPIN_REM regions, rather than param_RS/FH_BUCKET_SIZE.
  */
 #if 0
 		if_pf (param_RS > (med_regions-1)*FH_BUCKET_SIZE)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXREGION_SIZE (%ld) "
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXREGION_SIZE (%"PRIuPTR") "
 			    "is too large to encode in an AM Medium payload "
 			    "(%d bytes max)",
-			    param_RS, FH_BUCKET_SIZE*(med_regions-1));
+			    (uintptr_t)param_RS, FH_BUCKET_SIZE*(med_regions-1));
 #endif
 
 		if_pf (param_M < M_min)
-			gasneti_fatalerror("GASNET_FIREHOSE_M (%ld) is less "
-			    "than the minimum %ld (%ld buckets)",
+			gasneti_fatalerror("GASNET_FIREHOSE_M (%"PRIuPTR") is less "
+			    "than the minimum %"PRIuPTR" (%"PRIuPTR" buckets)",
 			    param_M, M_min, M_min >> FH_BUCKET_SHIFT);
 
 		if_pf (param_VM < VM_min)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_M (%ld) is less than "
-			    "the minimum %ld (%ld buckets)",
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_M (%"PRIuPTR") is less than "
+			    "the minimum %"PRIuPTR" (%"PRIuPTR" buckets)",
 			    param_VM, VM_min, VM_min >> FH_BUCKET_SHIFT);
 
 		if_pf (param_M - m_prepinned < M_min)
 			gasneti_fatalerror("Too many bytes in initial"
-			    " pinned regions list (%ld) for current "
-			    "GASNET_FIREHOSE_M parameter (%ld)", 
+			    " pinned regions list (%"PRIuPTR") for current "
+			    "GASNET_FIREHOSE_M parameter (%"PRIuPTR")", 
 			    m_prepinned, param_M);
 
 		if_pf (param_R < R_min)
-			gasneti_fatalerror("GASNET_FIREHOSE_R (%ld) is less"
-			    "than the minimum %ld", param_R, R_min);
+			gasneti_fatalerror("GASNET_FIREHOSE_R (%d) is less"
+			    "than the minimum %d", param_R, R_min);
 
 		if_pf (param_VR < VR_min)
-			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_R (%ld) is less than "
-			    "the minimum %ld", param_VR, VR_min);
+			gasneti_fatalerror("GASNET_FIREHOSE_MAXVICTIM_R (%d) is less than "
+			    "the minimum %d", param_VR, VR_min);
 
 		if_pf (param_R - num_prepinned < R_min)
 			gasneti_fatalerror("Too many regions passed on initial"
-			    " pinned bucket list (%ld) for current "
-			    "GASNET_FIREHOSE_R parameter (%ld)", 
-			    (unsigned long)num_prepinned, param_R);
+			    " pinned bucket list (%"PRIuPTR") for current "
+			    "GASNET_FIREHOSE_R parameter (%d)", 
+			    (uintptr_t)num_prepinned, param_R);
 	}
 
 	/* 
@@ -1524,9 +1529,9 @@ fh_init_plugin(uintptr_t max_pinnable_memory,
 	}
 
 	GASNETI_TRACE_PRINTF(C, 
-		    ("Maximum pinnable=%ld\tMax allowed=%ld", 
+		    ("Maximum pinnable=%"PRIuPTR"\tMax allowed=%"PRIuPTR, 
 		     (firehoses + param_VR) * param_RS + m_prepinned,
-		     (unsigned long)max_pinnable_memory));
+		     max_pinnable_memory));
 	gasneti_assert((firehoses + param_VR) * param_RS + m_prepinned
 						<= max_pinnable_memory);
 
@@ -1583,19 +1588,19 @@ fh_init_plugin(uintptr_t max_pinnable_memory,
 		}
 
 		GASNETI_TRACE_PRINTF(C, 
-		    ("Firehose M=%ld (fh=%ld)\tprepinned=%ld (buckets=%d)",
+		    ("Firehose M=%"PRIuPTR" (fh=%d)\tprepinned=%"PRIuPTR" (buckets=%d)",
 		    param_M, firehoses, m_prepinned, b_prepinned));
-		GASNETI_TRACE_PRINTF(C, ("Firehose Maxvictim=%ld (fh=%d)",
+		GASNETI_TRACE_PRINTF(C, ("Firehose Maxvictim=%"PRIuPTR" (fh=%d)",
 		    param_VM, fhc_MaxVictimBuckets));
 
 		GASNETI_TRACE_PRINTF(C, 
-		    ("MaxLocalPinSize=%d\tMaxRemotePinSize=%d", 
-		    (int) fhinfo->max_LocalPinSize,
-                    (int) fhinfo->max_RemotePinSize));
+		    ("MaxLocalPinSize=%"PRIuPTR"\tMaxRemotePinSize=%"PRIuPTR, 
+		    (uintptr_t) fhinfo->max_LocalPinSize,
+                    (uintptr_t) fhinfo->max_RemotePinSize));
 		GASNETI_TRACE_PRINTF(C, 
-		    ("MaxLocalRegions=%d\tMaxRemoteRegions=%d", 
-		    (int) fhinfo->max_LocalRegions,
-                    (int) fhinfo->max_RemoteRegions));
+		    ("MaxLocalRegions=%"PRIuPTR"\tMaxRemoteRegions=%"PRIuPTR, 
+		    (uintptr_t) fhinfo->max_LocalRegions,
+                    (uintptr_t) fhinfo->max_RemoteRegions));
 	}
 
 	return;
