@@ -220,7 +220,6 @@ void smp_coll_barrier_cond_var(smp_coll_t handle, int flags){
 #else
 void smp_coll_barrier_cond_var(smp_coll_t handle, int flags){
   /* cond variables must be phased on some OS's (HPUX) */
-  GASNETI_UNUSED_UNLESS_THREADS
   static struct {
     gasnett_cond_t cond;
     gasnett_mutex_t mutex;
@@ -233,7 +232,7 @@ void smp_coll_barrier_cond_var(smp_coll_t handle, int flags){
   barrier_count++;
   if (barrier_count != handle->THREADS) {
     /* CAUTION: changing the "do-while" to a "while" triggers a bug in the SunStudio 2006-08
-     * compiler for x86_64.  See http://upc-bugs.lbl.gov/bugzilla/show_bug.cgi?id=1858
+     * compiler for x86_64.  See http://gasnet-bugs.lbl.gov/bugzilla/show_bug.cgi?id=1858
      * which includes a link to Sun's own database entry for this issue.
      */
     do {
