@@ -97,7 +97,7 @@ typedef struct {
 } gasnetc_longmsg_t;
 
 #define GASNETC_ARGSEND_AUX(s,nargs) \
-        (offsetof(s,args)+(nargs*sizeof(gasnet_handlerarg_t)))
+        gasneti_offsetof(s,args[nargs])
 #define GASNETC_ARGSEND(cat,nargs) \
         GASNETC_ARGSEND_AUX(gasnetc_##cat##msg_t,(nargs))
 
@@ -189,4 +189,64 @@ extern void gasnetc_fast_barrier(void);
 extern void gasnetc_bootstrapBarrier(void);
 extern void gasnetc_bootstrapExchange(void *src, size_t len, void *dst);
 
+/* Hooks for conduit-specific collectives */
+#if !defined(GASNET_NO_PAMI_COLL)
+  /* NOTE: Focus is on blocking collectives because they are simpler, and are all that UPCR uses */
+  #define GASNET_PAMI_NATIVE_COLL 1
+  #define gasnete_coll_init_conduit gasnete_coll_init_pami
+  #define gasnete_coll_team_init_conduit gasnete_coll_team_init_pami
+
+  #define gasnete_coll_broadcast  gasnete_coll_broadcast_pami
+  #define gasnete_coll_broadcastM gasnete_coll_broadcastM_pami
+
+  #define gasnete_coll_exchange   gasnete_coll_exchange_pami
+  #define gasnete_coll_exchangeM  gasnete_coll_exchangeM_pami
+
+  #define gasnete_coll_gather     gasnete_coll_gather_pami
+  #define gasnete_coll_gatherM    gasnete_coll_gatherM_pami
+
+  #define gasnete_coll_gather_all  gasnete_coll_gather_all_pami
+  #define gasnete_coll_gather_allM gasnete_coll_gather_allM_pami
+
+  #define gasnete_coll_scatter    gasnete_coll_scatter_pami
+  #define gasnete_coll_scatterM   gasnete_coll_scatterM_pami
+
+  #if GASNET_PAR
+    #define GASNETE_COLL_TEAM_EXTRA struct {  \
+        /* collective geom & algorithms: */   \
+        pami_geometry_t geom;                 \
+        pami_algorithm_t allga_alg;           \
+        pami_algorithm_t allto_alg;           \
+        pami_algorithm_t bcast_alg;           \
+        pami_algorithm_t gathr_alg;           \
+        pami_algorithm_t scatt_alg;           \
+        pami_algorithm_t allgavi_alg;         \
+        pami_algorithm_t alltovi_alg;         \
+        pami_algorithm_t gathrvi_alg;         \
+        pami_algorithm_t scattvi_alg;         \
+        /* for multi-image intermediates: */  \
+        size_t scratch_max_nbytes;            \
+        size_t scratch_max_nbytes_allto;      \
+        void * scratch_space;                 \
+        int * counts;                         \
+        int * displs;                         \
+        size_t prev_nbytes;                   \
+        /* for syncronization: */             \
+        void * volatile tmp_addr;             \
+        volatile int barrier_phase;           \
+        char _pad[GASNETI_CACHE_LINE_BYTES];  \
+        gasneti_atomic_t barrier_counter[2];  \
+    } pami;
+  #else
+    #define GASNETE_COLL_TEAM_EXTRA struct {  \
+        pami_geometry_t geom;                 \
+        pami_algorithm_t allga_alg;           \
+        pami_algorithm_t allto_alg;           \
+        pami_algorithm_t bcast_alg;           \
+        pami_algorithm_t gathr_alg;           \
+        pami_algorithm_t scatt_alg;           \
+    } pami;
+  #endif
+#endif
+
 #endif
