@@ -9,6 +9,7 @@
 #include <gasnet_core_internal.h>
 #include <gasnet_extended_internal.h>
 #include <gasnet_handler.h>
+#include <gasnet_coll.h>
 
 static pami_send_hint_t gasnete_null_send_hint;
 static const gasnete_eopaddr_t EOPADDR_NIL = { { 0xFF, 0xFF } };
@@ -43,7 +44,7 @@ void gasnete_op_clr_lc(gasnete_op_t *op) {
 
 /*  allocate more eops */
 GASNETI_NEVER_INLINE(gasnete_eop_alloc,
-static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
+static void gasnete_eop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_eopaddr_t addr;
     int bufidx = thread->eop_num_bufs;
     gasnete_eop_t *buf;
@@ -129,7 +130,7 @@ static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  allocate a new iop */
 GASNETI_NEVER_INLINE(gasnete_iop_alloc,
-static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
+static gasnete_iop_t *gasnete_iop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_iop_t *iop = (gasnete_iop_t *)gasneti_malloc(sizeof(gasnete_iop_t));
     gasneti_leak(iop);
     #if GASNET_DEBUG
@@ -151,7 +152,7 @@ static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  get a new op */
 static
-gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *_gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eopaddr_t head = thread->eop_free;
   if_pf (gasnete_eopaddr_isnil(head)) {
     gasnete_eop_alloc(thread);
@@ -177,7 +178,7 @@ gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new op AND mark it in flight */
 GASNETI_INLINE(gasnete_eop_new)
-gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eop_t *eop = _gasnete_eop_new(thread);
 #if GASNETE_EOP_COUNTED
   eop->initiated_cnt++;
@@ -187,7 +188,7 @@ gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new iop */
 static
-gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
+gasnete_iop_t *gasnete_iop_new(gasneti_threaddata_t * const thread) {
   gasnete_iop_t *iop = thread->iop_free;
   if_pt (iop) {
     thread->iop_free = iop->next;
@@ -213,7 +214,7 @@ gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
 /*  query an eop for completeness */
 static
 int gasnete_eop_isdone(gasnete_eop_t *eop) {
-  gasneti_assert(eop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(eop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_eop_check(eop);
   return GASNETE_EOP_DONE(eop);
 }
@@ -221,7 +222,7 @@ int gasnete_eop_isdone(gasnete_eop_t *eop) {
 /*  query an iop for completeness - this means both puts and gets */
 static
 int gasnete_iop_isdone(gasnete_iop_t *iop) {
-  gasneti_assert(iop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(iop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_iop_check(iop);
   return (GASNETE_IOP_CNTDONE(iop,get) && GASNETE_IOP_CNTDONE(iop,put));
 }
@@ -284,9 +285,9 @@ static void gasnete_cb_op_lc(pami_context_t context, void *cookie, pami_result_t
 /*  free an eop */
 static
 void gasnete_eop_free(gasnete_eop_t *eop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
+  gasneti_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
   gasnete_eopaddr_t addr = eop->addr;
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_eop_check(eop);
   gasneti_assert(GASNETE_EOP_DONE(eop));
 #if GASNET_DEBUG
@@ -299,8 +300,8 @@ void gasnete_eop_free(gasnete_eop_t *eop) {
 /*  free an iop */
 static
 void gasnete_iop_free(gasnete_iop_t *iop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_iop_check(iop);
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,get));
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,put));
@@ -332,7 +333,6 @@ static void gasnete_check_config(void) {
 }
 
 extern void gasnete_init(void) {
-  GASNETI_UNUSED_UNLESS_DEBUG
   static int firstcall = 1;
   GASNETI_TRACE_PRINTF(C,("gasnete_init()"));
   gasneti_assert(firstcall); /*  make sure we haven't been called before */
@@ -342,11 +342,11 @@ extern void gasnete_init(void) {
 
   gasneti_assert(gasneti_nodes >= 1 && gasneti_mynode < gasneti_nodes);
 
-  { gasnete_threaddata_t *threaddata = NULL;
+  { gasneti_threaddata_t *threaddata = NULL;
     gasnete_eop_t *eop = NULL;
     #if GASNETI_MAX_THREADS > 1
       /* register first thread (optimization) */
-      threaddata = gasnete_mythread(); 
+      threaddata = _gasneti_mythread_slow(); 
     #else
       /* register only thread (required) */
       threaddata = gasnete_new_threaddata();
@@ -391,12 +391,12 @@ extern void gasnete_init(void) {
 
 /* ------------------------------------------------------------------------------------ */
 /* GASNET-Internal OP Interface */
-gasneti_eop_t *gasneti_eop_create(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_eop_t *op = gasnete_eop_new(GASNETE_MYTHREAD);
+gasneti_eop_t *gasneti_eop_create(GASNETI_THREAD_FARG_ALONE) {
+  gasnete_eop_t *op = gasnete_eop_new(GASNETI_MYTHREAD);
   return (gasneti_eop_t *)op;
 }
-gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t * const op = mythread->current_iop;
   gasnete_iop_check(op);
   if (isget) op->initiated_get_cnt += noperations;
@@ -618,29 +618,29 @@ void gasnete_get_common(void *dest, gasnet_node_t node, void *src, size_t nbytes
      gasnete_memset_nb
 */
 
-extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,H);
+extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,H,dest,node,src,nbytes);
   {
-    gasnete_eop_t * op = gasnete_eop_new(GASNETE_MYTHREAD);
+    gasnete_eop_t * op = gasnete_eop_new(GASNETI_MYTHREAD);
     gasnete_get_common(dest, node, src, nbytes, (gasnete_op_t *)op, 1);
     return (gasnet_handle_t)op;
   }
 }
 
-extern gasnet_handle_t gasnete_put_nb      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(ALIGNED,H);
+extern gasnet_handle_t gasnete_put_nb      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(ALIGNED,H,node,dest,src,nbytes);
   {
-    gasnete_eop_t * op = gasnete_eop_new(GASNETE_MYTHREAD);
+    gasnete_eop_t * op = gasnete_eop_new(GASNETI_MYTHREAD);
     gasnete_put_common(node, dest, src, nbytes, (gasnete_op_t *)op, 1, 1);
     gasneti_assert(gasnete_op_read_lc((gasnete_op_t *)op));
     return (gasnet_handle_t)op;
   }
 }
 
-extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
+extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
   {
-    gasnete_eop_t * op = gasnete_eop_new(GASNETE_MYTHREAD);
+    gasnete_eop_t * op = gasnete_eop_new(GASNETI_MYTHREAD);
     gasnete_put_common(node, dest, src, nbytes, (gasnete_op_t *)op, 0, 1);
     return (gasnet_handle_t)op;
   }
@@ -659,7 +659,7 @@ GASNETI_INLINE(gasnete_op_try_free)
 int gasnete_op_try_free(gasnet_handle_t handle) {
   gasnete_op_t *op = (gasnete_op_t *)handle;
 
-  gasneti_assert(op->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(op->threadidx == _gasneti_mythread_slow()->threadidx);
   if_pt (OPTYPE(op) == OPTYPE_EXPLICIT) {
     gasnete_eop_t *eop = (gasnete_eop_t*)op;
 
@@ -758,20 +758,20 @@ extern int  gasnete_try_syncnb_all (gasnet_handle_t *phandle, size_t numhandles)
      gasnete_memset_nb
 */
 
-extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const op = mythread->current_iop;
     op->initiated_get_cnt++;
     gasnete_get_common(dest, node, src, nbytes, (gasnete_op_t *)op, 0);
   }
 }
 
-extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(ALIGNED,V);
+extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(ALIGNED,V,node,dest,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const op = mythread->current_iop;
     op->initiated_put_cnt++;
     gasnete_put_common(node, dest, src, nbytes, (gasnete_op_t *)op, 1, 0);
@@ -785,10 +785,10 @@ extern void gasnete_put_nbi      (gasnet_node_t node, void *dest, void *src, siz
   }
 }
 
-extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,V);
+extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,V,node,dest,src,nbytes);
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const op = mythread->current_iop;
     op->initiated_put_cnt++;
     gasnete_put_common(node, dest, src, nbytes, (gasnete_op_t *)op, 0, 0);
@@ -801,13 +801,13 @@ extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src, siz
   ===========================================================
 */
 
-extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_gets(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(OPTYPE(iop) == OPTYPE_IMPLICIT);
@@ -824,13 +824,13 @@ extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
   }
 }
 
-extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_puts(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(iop->next == NULL);
@@ -856,8 +856,8 @@ extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
 */
 /*  This implementation allows recursive access regions, although the spec does not require that */
 /*  operations are associated with the most immediately enclosing access region */
-extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = gasnete_iop_new(mythread); /*  push an iop  */
   GASNETI_TRACE_PRINTF(S,("BEGIN_NBI_ACCESSREGION"));
   #if GASNET_DEBUG
@@ -868,8 +868,8 @@ extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE
   mythread->current_iop = iop;
 }
 
-extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETI_THREAD_FARG_ALONE) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop; /*  pop an iop */
   GASNETI_TRACE_EVENT_VAL(S,END_NBI_ACCESSREGION,iop->initiated_get_cnt + iop->initiated_put_cnt);
   #if GASNET_DEBUG
@@ -889,8 +889,8 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
 
 #if GASNETI_DIRECT_GET_BULK
 extern void gasnete_get_bulk (void *dest, gasnet_node_t node, void *src,
-			      size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+			      size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
   {
     volatile gasnete_eop_t op = { OPSTATE_INFLIGHT, };
     gasnete_get_common(dest, node, src, nbytes, (gasnete_op_t *)&op, 1);
@@ -901,8 +901,8 @@ extern void gasnete_get_bulk (void *dest, gasnet_node_t node, void *src,
 
 #if GASNETI_DIRECT_PUT_BULK
 extern void gasnete_put_bulk (gasnet_node_t node, void* dest, void *src,
-			      size_t nbytes GASNETE_THREAD_FARG) {
-  GASNETI_CHECKPSHM_PUT(UNALIGNED,V);
+			      size_t nbytes GASNETI_THREAD_FARG) {
+  GASNETI_CHECKPSHM_PUT(UNALIGNED,V,node,dest,src,nbytes);
   {
     volatile gasnete_eop_t op = { OPSTATE_INFLIGHT, };
     gasnete_put_common(node, dest, src, nbytes, (gasnete_op_t *)&op, 0, 1);
@@ -941,11 +941,11 @@ static void gasnete_pdbarrier_init(gasnete_coll_team_t team);
     gasnete_coll_default_barrier_type = GASNETE_COLL_BARRIER_PAMIDISSEM;    \
 } while (0)
 
-#define GASNETE_BARRIER_INIT(TEAM, BARRIER_TYPE) do {            \
-    if ((BARRIER_TYPE) == GASNETE_COLL_BARRIER_PAMIALLREDUCE) {  \
+#define GASNETE_BARRIER_INIT(TEAM, TYPE, NODES, SUPERNODES) do { \
+    if ((TYPE) == GASNETE_COLL_BARRIER_PAMIALLREDUCE) {          \
       gasnete_parbarrier_init(TEAM);                             \
     } else                                                       \
-    if ((BARRIER_TYPE) == GASNETE_COLL_BARRIER_PAMIDISSEM) {     \
+    if ((TYPE) == GASNETE_COLL_BARRIER_PAMIDISSEM) {             \
       gasnete_pdbarrier_init(TEAM);                              \
     }                                                            \
   } while (0)
@@ -1130,10 +1130,10 @@ static void gasnete_parbarrier_init(gasnete_coll_team_t team) {
 
 typedef struct {
   gasnet_node_t *peer_list;
+  gasnet_node_t peer_count;
   volatile int phase;
 #if GASNETI_PSHM_BARRIER_HIER
   gasnete_pshmbarrier_data_t *pshm_data; /* non-NULL if using hierarchical code */
-  gasnet_node_t pshm_rep;                /* the "representative" member of my supernode */
   int pshm_shift;                        /* "shift" if this node is not the representative */
 #endif
   volatile int prev_value, prev_flags;   /* for gasnet_barrier_result() */
@@ -1142,6 +1142,9 @@ typedef struct {
     uint64_t arrived;   /* Which messages have arrived (bit map) need 33 bits */
     volatile int next;  /* Which message index are we waiting for */
     int busy;           /* Prevent recursion */
+#if GASNETI_PSHM_BARRIER_HIER
+    int notify_done;    /* Need to kick pshm until non-zero */
+#endif
   } state[2]; /* per-phase */
   pami_xfer_t barrier_op;
 } gasnete_pdbarrier_t;
@@ -1149,9 +1152,9 @@ typedef struct {
 typedef struct {
   uint32_t teamid;
   uint32_t value;
-#if ((GASNET_BARRIERFLAG_MISMATCH|GASNET_BARRIERFLAG_ANONYMOUS|0xff) == 0xff)
+#if ((GASNETE_BARRIERFLAGS_CLIENT_ALL|0xff) == 0xff)
   uint8_t  flags;
-#elif ((GASNET_BARRIERFLAG_MISMATCH|GASNET_BARRIERFLAG_ANONYMOUS|0xffff) == 0xffff)
+#elif ((GASNETE_BARRIERFLAGS_CLIENT_ALL|0xffff) == 0xffff)
   uint16_t flags;
 #else
   uint32_t flags;
@@ -1162,6 +1165,13 @@ typedef struct {
 } gasnete_pdbarrier_msg_t;
 #define GASNETE_PDBARRIER_MSG_T offsetof(gasnete_pdbarrier_msg_t, end)
 
+#if 0 // DISABLED due to (not yet explained) errorneous behavior - see bug 3764
+#define GASNETE_PDBARRIER_UNNAMED(_flags,_barr) \
+  (((_flags) & GASNETE_BARRIERFLAG_UNNAMED) && (_barr)->barrier_op.cb_done)
+#else
+#define GASNETE_PDBARRIER_UNNAMED(_flags,_barr) 0
+#endif
+
 static pami_send_hint_t gasnete_pdbarrier_send_hint;
 
 static void gasnete_cb_pdbarr_done(pami_context_t context, void *cookie, pami_result_t status) {
@@ -1172,7 +1182,7 @@ static void gasnete_cb_pdbarr_done(pami_context_t context, void *cookie, pami_re
 }
 
 /* Called only w/ context lock held */
-GASNETI_ALWAYS_INLINE(gasnete_pdbarr_send)
+GASNETI_INLINE(gasnete_pdbarr_send)
 void gasnete_pdbarr_send(
         gasnet_node_t peer,
         uint32_t teamid,
@@ -1209,6 +1219,9 @@ static void gasnete_pdbarr_advance(
    */
   register int value = state->value;
   register int flags = state->flags;
+#if GASNETI_PSHM_BARRIER_HIER
+again:
+#endif
   if ((flags | msg_flags) & GASNET_BARRIERFLAG_MISMATCH) {
     flags = GASNET_BARRIERFLAG_MISMATCH;
   } else if (flags & GASNET_BARRIERFLAG_ANONYMOUS) {
@@ -1221,6 +1234,23 @@ static void gasnete_pdbarr_advance(
   state->flags = flags;
   state->arrived |= (1 << msg_index);
 
+#if GASNETI_PSHM_BARRIER_HIER
+  /* may need to advance local notify */
+  if (barr->pshm_data && !state->notify_done) {
+    PSHM_BDATA_DECL(pshm_bdata, barr->pshm_data);
+    if (gasnete_pshmbarrier_kick(pshm_bdata)) {
+      /* "simulate" arrival of the notify */
+      state->notify_done = 1;
+      msg_value = pshm_bdata->shared->value,
+      msg_flags = pshm_bdata->shared->flags,
+      msg_index = 0;
+      goto again;
+    } else {
+      return;
+    }
+  }
+#endif
+
   /* Avoid recursion if run from below gasnete_pdbarr_send() */
   if (state->busy) return;
   else state->busy = 1;
@@ -1231,17 +1261,15 @@ static void gasnete_pdbarr_advance(
 
     /* Note: state->{arrived,value,flags} may change within each send */
     while (state->arrived & distance) {
-      const gasnet_node_t peer = barr->peer_list[index];
-
-      if (peer == gasneti_nodes) {
+      if (index == barr->peer_count) {
         gasneti_sync_writes();
         index = -1; /* DONE! */
         break;
+      } else {
+        const gasnet_node_t peer = barr->peer_list[index];
+        gasnete_pdbarr_send(peer, teamid, state->value, state->flags, msg_phase, ++index);
+        distance <<= 1;
       }
-
-      gasnete_pdbarr_send(peer, teamid, state->value, state->flags, msg_phase, ++index);
-
-      distance <<= 1;
     }
 
     state->next = index;
@@ -1262,41 +1290,74 @@ static void gasnete_pdbarr_dispatch(
   gasnete_pdbarr_advance(barr, msg->teamid, msg->value, msg->flags, msg->phase, msg->index);
 }
 
+#if GASNETI_PSHM_BARRIER_HIER
+static int gasnete_pdbarrier_kick_pshm(gasnete_coll_team_t team) {
+  gasnete_pdbarrier_t *barr = team->barrier_data;
+  int done = barr->state[barr->phase].notify_done;
+
+  if (!done && !GASNETC_PAMI_TRYLOCK(gasnetc_context)) {
+    const int phase = barr->phase;
+    done = barr->state[phase].notify_done;
+    if (!done) {
+      PSHM_BDATA_DECL(pshm_bdata, barr->pshm_data);
+      if (gasnete_pshmbarrier_kick(pshm_bdata)) {
+        done = barr->state[phase].notify_done = 1;
+        if (!barr->pshm_shift) {
+          gasnete_pdbarr_advance(barr, team->team_id,
+                                 pshm_bdata->shared->value,
+                                 pshm_bdata->shared->flags,
+                                 phase, 0);
+        }
+      }
+    }
+    GASNETC_PAMI_UNLOCK(gasnetc_context);
+  }
+
+  return done;
+}
+#endif
+
 static void gasnete_pdbarrier_notify(gasnete_coll_team_t team, int id, int flags) {
   gasnete_pdbarrier_t *barr = team->barrier_data;
-  const int is_unnamed = ((flags & GASNETE_BARRIERFLAG_UNNAMED) && barr->barrier_op.cb_done);
-  int do_send = 1;
   int phase;
   
   gasneti_sync_reads();
   phase = barr->phase ^ 1;
   GASNETE_SPLITSTATE_NOTIFY_ENTER(team);
 
-#if GASNETI_PSHM_BARRIER_HIER
-  if (barr->pshm_data && !is_unnamed) {
-    PSHM_BDATA_DECL(pshm_bdata, barr->pshm_data);
-    (void)gasnete_pshmbarrier_notify_inner(pshm_bdata, id, flags);
-    do_send = !barr->pshm_shift;
-    id = pshm_bdata->shared->value;
-    flags = pshm_bdata->shared->flags;
-  }
-#endif
-
-  if (do_send) {
+  if (GASNETE_PDBARRIER_UNNAMED(flags, barr)) {
+    /* UNNAMED barrier - use a non-blocking PAMI barrier on the geometry */
+    struct gasnete_pdbarrier_state * const state = &barr->state[phase];
+    gasneti_assert(state->flags == GASNET_BARRIERFLAG_ANONYMOUS);
+    barr->barrier_op.cookie = (void *)&state->next;
     GASNETC_PAMI_LOCK(gasnetc_context);
-    if (is_unnamed) {
-      /* launch a non-blocking barrier on the geometry */
-      struct gasnete_pdbarrier_state * const state = &barr->state[phase];
-      pami_result_t rc;
-      gasneti_assert(state->flags == GASNET_BARRIERFLAG_ANONYMOUS);
-      barr->barrier_op.cookie = (void *)&state->next;
-      rc = PAMI_Collective(gasnetc_context, &barr->barrier_op);
+    { pami_result_t rc = PAMI_Collective(gasnetc_context, &barr->barrier_op);
       GASNETC_PAMI_CHECK(rc, "initiating collective for barrier");
-    } else {
-      /* Merge w/ any earlier arrivals and send the notify: */
-      gasnete_pdbarr_advance(barr, team->team_id, id, flags, phase, 0);
     }
     GASNETC_PAMI_UNLOCK(gasnetc_context);
+  } else {
+    int do_send = 1;
+
+#if GASNETI_PSHM_BARRIER_HIER
+    if (barr->pshm_data) {
+      struct gasnete_pdbarrier_state * const state = &barr->state[phase];
+      PSHM_BDATA_DECL(pshm_bdata, barr->pshm_data);
+      do_send = 0;
+      state->notify_done = gasnete_pshmbarrier_notify_inner(pshm_bdata, id, flags);
+      if (state->notify_done) {
+        id = pshm_bdata->shared->value;
+        flags = pshm_bdata->shared->flags;
+        do_send = !barr->pshm_shift;
+      }
+    }
+#endif
+
+    if (do_send) {
+      GASNETC_PAMI_LOCK(gasnetc_context);
+      /* Merge w/ any earlier arrivals and send the notify: */
+      gasnete_pdbarr_advance(barr, team->team_id, id, flags, phase, 0);
+      GASNETC_PAMI_UNLOCK(gasnetc_context);
+    }
   }
   
   barr->phase = phase;
@@ -1330,7 +1391,7 @@ int gasnete_pdbarrier_finish(
   GASNETE_SPLITSTATE_LEAVE(team);
 
 #if GASNETI_PSHM_BARRIER_HIER
-  if (barr->pshm_data && !((flags & GASNETE_BARRIERFLAG_UNNAMED) && barr->barrier_op.cb_done)) {
+  if (barr->pshm_data && !GASNETE_PDBARRIER_UNNAMED(flags, barr)) {
     /* Signal any passive peers w/ the final result */
     const PSHM_BDATA_DECL(pshm_bdata, barr->pshm_data);
     pshm_bdata->shared->value = final_value;
@@ -1353,8 +1414,9 @@ static int gasnete_pdbarrier_wait(gasnete_coll_team_t team, int id, int flags) {
   GASNETE_SPLITSTATE_WAIT(team);
 
 #if GASNETI_PSHM_BARRIER_HIER
-  if (barr->pshm_data && !((flags & GASNETE_BARRIERFLAG_UNNAMED) && barr->barrier_op.cb_done)) {
+  if (barr->pshm_data && !GASNETE_PDBARRIER_UNNAMED(flags, barr)) {
     const int passive_shift = barr->pshm_shift;
+    gasneti_polluntil(gasnete_pdbarrier_kick_pshm(team));
     retval = gasnete_pshmbarrier_wait_inner(barr->pshm_data, id, flags, passive_shift);
     if (passive_shift) {
       /* Once the active peer signals done, we can return */
@@ -1385,9 +1447,10 @@ static int gasnete_pdbarrier_try(gasnete_coll_team_t team, int id, int flags) {
   GASNETI_SAFE(gasneti_AMPoll());
 
 #if GASNETI_PSHM_BARRIER_HIER
-  if (barr->pshm_data && !((flags & GASNETE_BARRIERFLAG_UNNAMED) && barr->barrier_op.cb_done)) {
+  if (barr->pshm_data && !GASNETE_PDBARRIER_UNNAMED(flags, barr)) {
     const int pshm_shift = barr->pshm_shift;
-    if (!gasnete_pshmbarrier_try_inner(barr->pshm_data, pshm_shift))
+    if (!gasnete_pdbarrier_kick_pshm(team) ||
+        !gasnete_pshmbarrier_try_inner(barr->pshm_data, pshm_shift))
       return GASNET_ERR_NOT_READY;
     if (pshm_shift)
       return gasnete_pdbarrier_wait(team, id, flags);
@@ -1417,9 +1480,10 @@ static void gasnete_pdbarrier_init(gasnete_coll_team_t team) {
   static int is_init = 0;
   int total_ranks = team->total_ranks;
   int myrank = team->myrank;
+  gasnete_coll_peer_list_t *peers = &team->peers;
+
 #if GASNETI_PSHM_BARRIER_HIER
-  gasnet_node_t *supernode_reps = NULL;
-  PSHM_BDATA_DECL(pshm_bdata, gasnete_pshmbarrier_init_hier(team, &total_ranks, &myrank, &supernode_reps));
+  PSHM_BDATA_DECL(pshm_bdata, gasnete_pshmbarrier_init_hier(team, &total_ranks, &myrank, &peers));
 #endif
 
   /* Allocate memory */
@@ -1445,42 +1509,15 @@ static void gasnete_pdbarrier_init(gasnete_coll_team_t team) {
   }
 
 #if GASNETI_PSHM_BARRIER_HIER
+  barr->pshm_data = pshm_bdata;
   if (pshm_bdata) {
-    barr->pshm_data = pshm_bdata;
     barr->pshm_shift = pshm_bdata->private.rank ? 2 : 0;
-    if (supernode_reps) {
-      barr->pshm_rep = supernode_reps[myrank];
-    } else {
-      barr->pshm_rep = gasneti_pshm_firsts[myrank];
-    }
   }
 #endif
 
-  /* Build the peer list */
-  { int step, steps;
-    uint64_t distance;
-#if GASNETI_PSHM_BARRIER_HIER
-    gasnet_node_t *nodes = supernode_reps ? supernode_reps : gasneti_pshm_firsts;
-#endif
-
-    for (steps=0, distance=1; distance < total_ranks; ++steps, distance*=2) { /* empty */ }
-
-    barr->peer_list = gasneti_calloc(steps+1, sizeof(gasnet_node_t));
-    gasneti_leak(barr->peer_list);
-
-    for (step=0, distance=1; step < steps; ++step, distance*=2) {
-      gasnet_node_t peer = (myrank < distance)
-                              ? (myrank + (total_ranks - distance))
-                              : (myrank - distance);
-#if GASNETI_PSHM_BARRIER_HIER
-      if (pshm_bdata) {
-        barr->peer_list[step] = nodes[peer];
-      } else
-#endif
-      barr->peer_list[step] = GASNETE_COLL_REL2ACT(team, peer);
-    }
-    barr->peer_list[steps] = gasneti_nodes; /* sentinel */
-  }
+  /* The peer list */
+  barr->peer_list = peers->fwd;
+  barr->peer_count = peers->num;
 
   /* TODO: Anything we should hint here? */
   memset(&gasnete_pdbarrier_send_hint, 0, sizeof(pami_send_hint_t));
@@ -1541,7 +1578,7 @@ static void gasnete_pdbarrier_init(gasnete_coll_team_t team) {
 */
 
 /* use reference implementation of scatter/gather and strided */
-#include "gasnet_extended_refvis.h"
+#include "gasnet_refvis.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
@@ -1550,7 +1587,7 @@ static void gasnete_pdbarrier_init(gasnete_coll_team_t team) {
 */
 
 /* use reference implementation of collectives */
-#include "gasnet_extended_refcoll.h"
+#include "gasnet_refcoll.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
