@@ -19,7 +19,7 @@ extern void _gasnete_iop_check(gasnete_iop_t *iop) { gasnete_iop_check(iop); }
 
 /*  allocate more eops */
 GASNETI_NEVER_INLINE(gasnete_eop_alloc,
-static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
+static void gasnete_eop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_eopaddr_t addr;
     int bufidx = thread->eop_num_bufs;
     gasnete_eop_t *buf;
@@ -105,7 +105,7 @@ static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  allocate a new iop */
 GASNETI_NEVER_INLINE(gasnete_iop_alloc,
-static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
+static gasnete_iop_t *gasnete_iop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_iop_t *iop = (gasnete_iop_t *)gasneti_malloc(sizeof(gasnete_iop_t));
     gasneti_leak(iop);
     #if GASNET_DEBUG
@@ -122,7 +122,7 @@ static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  get a new op */
 static
-gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *_gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eopaddr_t head = thread->eop_free;
   if_pf (gasnete_eopaddr_isnil(head)) {
     gasnete_eop_alloc(thread);
@@ -145,7 +145,7 @@ gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new op AND mark it in flight */
 GASNETI_INLINE(gasnete_eop_new)
-gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eop_t *eop = _gasnete_eop_new(thread);
 #if GASNETE_EOP_COUNTED
   eop->initiated_cnt++;
@@ -155,7 +155,7 @@ gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new iop */
 static
-gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
+gasnete_iop_t *gasnete_iop_new(gasneti_threaddata_t * const thread) {
   gasnete_iop_t *iop = thread->iop_free;
   if_pt (iop) {
     thread->iop_free = iop->next;
@@ -180,7 +180,7 @@ gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
 /*  query an eop for completeness */
 static
 int gasnete_eop_isdone(gasnete_eop_t *eop) {
-  gasneti_assert(eop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(eop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_eop_check(eop);
   return GASNETE_EOP_DONE(eop);
 }
@@ -188,7 +188,7 @@ int gasnete_eop_isdone(gasnete_eop_t *eop) {
 /*  query an iop for completeness - this means both puts and gets */
 static
 int gasnete_iop_isdone(gasnete_iop_t *iop) {
-  gasneti_assert(iop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(iop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_iop_check(iop);
   return (GASNETE_IOP_CNTDONE(iop,get) && GASNETE_IOP_CNTDONE(iop,put));
 }
@@ -211,9 +211,9 @@ void gasnete_op_markdone(gasnete_op_t *op, int isget) {
 /*  free an eop */
 static
 void gasnete_eop_free(gasnete_eop_t *eop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
+  gasneti_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
   gasnete_eopaddr_t addr = eop->addr;
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_eop_check(eop);
   gasneti_assert(GASNETE_EOP_DONE(eop));
 #if GASNET_DEBUG
@@ -226,8 +226,8 @@ void gasnete_eop_free(gasnete_eop_t *eop) {
 /*  free an iop */
 static
 void gasnete_iop_free(gasnete_iop_t *iop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_iop_check(iop);
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,get));
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,put));
@@ -259,7 +259,6 @@ static void gasnete_check_config(void) {
 }
 
 extern void gasnete_init(void) {
-  GASNETI_UNUSED_UNLESS_DEBUG
   static int firstcall = 1;
   GASNETI_TRACE_PRINTF(C,("gasnete_init()"));
   gasneti_assert(firstcall); /*  make sure we haven't been called before */
@@ -269,11 +268,11 @@ extern void gasnete_init(void) {
 
   gasneti_assert(gasneti_nodes >= 1 && gasneti_mynode < gasneti_nodes);
 
-  { gasnete_threaddata_t *threaddata = NULL;
+  { gasneti_threaddata_t *threaddata = NULL;
     gasnete_eop_t *eop = NULL;
     #if GASNETI_MAX_THREADS > 1
       /* register first thread (optimization) */
-      threaddata = gasnete_mythread(); 
+      threaddata = _gasneti_mythread_slow(); 
     #else
       /* register only thread (required) */
       threaddata = gasnete_new_threaddata();
@@ -294,12 +293,12 @@ extern void gasnete_init(void) {
 
 /* ------------------------------------------------------------------------------------ */
 /* GASNET-Internal OP Interface */
-gasneti_eop_t *gasneti_eop_create(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_eop_t *op = gasnete_eop_new(GASNETE_MYTHREAD);
+gasneti_eop_t *gasneti_eop_create(GASNETI_THREAD_FARG_ALONE) {
+  gasnete_eop_t *op = gasnete_eop_new(GASNETI_MYTHREAD);
   return (gasneti_eop_t *)op;
 }
-gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t * const op = mythread->current_iop;
   gasnete_iop_check(op);
   if (isget) op->initiated_get_cnt += noperations;
@@ -357,11 +356,11 @@ void gasneti_iop_markdone(gasneti_iop_t *iop, unsigned int noperations, int isge
 */
 
 gasnet_handle_t
-gasnete_get_nb_bulk(void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG)
+gasnete_get_nb_bulk(void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_GET(UNALIGNED,H);
+    GASNETI_CHECKPSHM_GET(UNALIGNED,H,dest,node,src,nbytes);
     {
-        gasnete_eop_t *op = _gasnete_eop_new(GASNETE_MYTHREAD);
+        gasnete_eop_t *op = _gasnete_eop_new(GASNETI_MYTHREAD);
         gasnetc_rdma_get(dest, node, src, nbytes, OP_TYPE_EOP, (gasnet_handle_t) op);
         return (gasnet_handle_t)op;
     }
@@ -369,11 +368,11 @@ gasnete_get_nb_bulk(void *dest, gasnet_node_t node, void *src, size_t nbytes GAS
 
 
 gasnet_handle_t
-gasnete_put_nb(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG)
+gasnete_put_nb(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
+    GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
     {
-        gasnete_eop_t *op = _gasnete_eop_new(GASNETE_MYTHREAD);
+        gasnete_eop_t *op = _gasnete_eop_new(GASNETI_MYTHREAD);
         gasnetc_rdma_put(node, dest, src, nbytes, OP_TYPE_EOP, (gasnet_handle_t) op);
         gasnetc_rdma_put_wait((gasnet_handle_t) op);
         return (gasnet_handle_t)op;
@@ -382,11 +381,11 @@ gasnete_put_nb(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_
 
 
 gasnet_handle_t
-gasnete_put_nb_bulk(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG)
+gasnete_put_nb_bulk(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
+    GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
     {
-        gasnete_eop_t *op = _gasnete_eop_new(GASNETE_MYTHREAD);
+        gasnete_eop_t *op = _gasnete_eop_new(GASNETI_MYTHREAD);
         gasnetc_rdma_put(node, dest, src, nbytes, OP_TYPE_EOP, (gasnet_handle_t) op);
         return (gasnet_handle_t)op;
     }
@@ -406,7 +405,7 @@ GASNETI_INLINE(gasnete_op_try_free)
 int gasnete_op_try_free(gasnet_handle_t handle) {
   gasnete_op_t *op = (gasnete_op_t *)handle;
 
-  gasneti_assert(op->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(op->threadidx == _gasneti_mythread_slow()->threadidx);
   if_pt (OPTYPE(op) == OPTYPE_EXPLICIT) {
     gasnete_eop_t *eop = (gasnete_eop_t*)op;
 
@@ -505,11 +504,11 @@ extern int  gasnete_try_syncnb_all (gasnet_handle_t *phandle, size_t numhandles)
 */
 
 void
-gasnete_get_nbi_bulk(void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG)
+gasnete_get_nbi_bulk(void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+    GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
     {
-        gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+        gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
         gasnete_iop_t *op = mythread->current_iop;
         op->initiated_get_cnt++;
         gasnetc_rdma_get(dest, node, src, nbytes, OP_TYPE_IOP, (gasnet_handle_t) op);
@@ -518,11 +517,11 @@ gasnete_get_nbi_bulk(void *dest, gasnet_node_t node, void *src, size_t nbytes GA
 
 
 void
-gasnete_put_nbi(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG)
+gasnete_put_nbi(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_PUT(ALIGNED,V);
+    GASNETI_CHECKPSHM_PUT(ALIGNED,V,node,dest,src,nbytes);
     {
-        gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+        gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
         gasnete_iop_t *op = mythread->current_iop;
         op->initiated_put_cnt++;
         gasnetc_rdma_put(node, dest, src, nbytes, OP_TYPE_IOP, (gasnet_handle_t) op);
@@ -532,11 +531,11 @@ gasnete_put_nbi(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE
 
 
 void
-gasnete_put_nbi_bulk(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG)
+gasnete_put_nbi_bulk(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_PUT(ALIGNED,V);
+    GASNETI_CHECKPSHM_PUT(ALIGNED,V,node,dest,src,nbytes);
     {
-        gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+        gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
         gasnete_iop_t *op = mythread->current_iop;
         op->initiated_put_cnt++;
         gasnetc_rdma_put(node, dest, src, nbytes, OP_TYPE_IOP, (gasnet_handle_t) op);
@@ -550,13 +549,13 @@ gasnete_put_nbi_bulk(gasnet_node_t node, void *dest, void *src, size_t nbytes GA
   ===========================================================
 */
 
-extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_gets(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(OPTYPE(iop) == OPTYPE_IMPLICIT);
@@ -572,13 +571,13 @@ extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
   }
 }
 
-extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_puts(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(iop->next == NULL);
@@ -603,8 +602,8 @@ extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
 */
 /*  This implementation allows recursive access regions, although the spec does not require that */
 /*  operations are associated with the most immediately enclosing access region */
-extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = gasnete_iop_new(mythread); /*  push an iop  */
   GASNETI_TRACE_PRINTF(S,("BEGIN_NBI_ACCESSREGION"));
   #if GASNET_DEBUG
@@ -615,8 +614,8 @@ extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE
   mythread->current_iop = iop;
 }
 
-extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETI_THREAD_FARG_ALONE) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop; /*  pop an iop */
   GASNETI_TRACE_EVENT_VAL(S,END_NBI_ACCESSREGION,iop->initiated_get_cnt + iop->initiated_put_cnt);
   #if GASNET_DEBUG
@@ -646,7 +645,7 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
 */
 
 /* use reference implementation of scatter/gather and strided */
-#include "gasnet_extended_refvis.h"
+#include "gasnet_refvis.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
@@ -655,7 +654,7 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
 */
 
 /* use reference implementation of collectives */
-#include "gasnet_extended_refcoll.h"
+#include "gasnet_refcoll.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
