@@ -10,7 +10,6 @@
 #include <gasnet_extended_internal.h>
 #include <gasnet_handler.h>
 #include <gasnet_core_internal.h>
-#include <pmi-spawner/gasnet_bootstrap_internal.h>
 
 #include <errno.h>
 #include <unistd.h>
@@ -47,6 +46,8 @@ static ptl_size_t bootstrap_barrier_calls = 0;
 static ptl_handle_ct_t bootstrap_barrier_ct_h;
 static ptl_handle_me_t bootstrap_barrier_me_h;
 
+gasneti_spawnerfn_t const *gasneti_spawner = NULL;
+
 /* 16 blocks of 1MB each for AM reception */
 static int p4_am_size = 1 * 1024 * 1024;
 static int p4_am_num_entries = 16;
@@ -242,8 +243,11 @@ gasnetc_p4_init(gasnet_node_t *rank_p, gasnet_node_t *size_p)
     p4_rdma_eq_len = gasneti_getenv_int_withdefault("GASNET_EXTENDED_EVENT_QUEUE_LENGTH", p4_rdma_eq_len, 0);
     p4_poll_limit = p4_am_eq_len / 8; /* arbitrary */
 
-    ret = gasneti_bootstrapInit_pmi(NULL, NULL, size_p, rank_p);
-    if (GASNET_OK != ret) return ret;
+    gasneti_spawner = gasneti_spawnerInit(NULL, NULL, "PMI", size_p, rank_p);
+    if (!gasneti_spawner) GASNETI_RETURN_ERRR(NOT_INIT, "GASNet job spawn failed");
+
+    /* Must init timers after global env, and preferably before tracing */
+    GASNETI_TICKS_INIT();
 
     /* Setup data structures */
     p4_long_hash = gasnetc_hash_create();
@@ -311,7 +315,7 @@ gasnetc_p4_init(gasnet_node_t *rank_p, gasnet_node_t *size_p)
 
     /* build id map */
     desired = gasneti_malloc(sizeof(ptl_process_t) * gasneti_nodes);
-    gasneti_bootstrapExchange_pmi(&my_id, sizeof(ptl_process_t), desired);
+    gasneti_spawner->Exchange(&my_id, sizeof(ptl_process_t), desired);
 
     gasneti_nodemapInit(NULL, &desired[0].phys.nid,
                         sizeof(desired[0].phys.nid),
@@ -515,9 +519,9 @@ gasnetc_p4_init(gasnet_node_t *rank_p, gasnet_node_t *size_p)
        for the bootstrapBarrier before the info exchange and therefore
        not care about synchronization until the segmentInit later in
        gasnet_init(). */
-    gasneti_bootstrapBarrier_pmi();
+    gasneti_spawner->Barrier();
 
-    gasneti_bootstrapCleanup_pmi(); /* no more PMI-based collectives */
+    gasneti_spawner->Cleanup(); /* no more PMI-based collectives */
 
     return GASNET_OK;
 }
@@ -542,7 +546,7 @@ gasnetc_p4_attach(void *segbase, uintptr_t segsize)
 #elif GASNET_SEGMENT_EVERYTHING
     gasneti_assert((void*) 0 == segbase && (uintptr_t)-1 == segsize);
     me.start = le.start = 0;
-    me.length = le.start = PTL_SIZE_MAX;
+    me.length = le.length = PTL_SIZE_MAX;
 #endif
 
     me.min_free = 0;
@@ -642,7 +646,7 @@ gasnetc_p4_exit(void)
     PtlFini();
 
     /* Currently, regardless whether make this call, non-collective exits hang */
-    gasneti_bootstrapFini_pmi();
+    gasneti_spawner->Fini();
 }
 
 
@@ -887,8 +891,8 @@ p4_poll(const ptl_handle_eq_t *eq_handles, unsigned int size, unsigned int limit
 		        p4_free_long_match(match);
                       }
                     } else {
-                        gasneti_fatalerror("unknown active message type.  MB: 0x%lx",
-                                           (unsigned long)ev.match_bits);
+                        gasneti_fatalerror("unknown active message type.  MB: 0x%"PRIx64,
+                                           (uint64_t)ev.match_bits);
                     }
 #ifdef GASNET_PAR
                     gasneti_weakatomic_add(&block->op_count, 1, 0);
@@ -1120,8 +1124,10 @@ p4_poll(const ptl_handle_eq_t *eq_handles, unsigned int size, unsigned int limit
                        been processed.  Therefore, we need to wait
                        until the other thread finishes processing the
                        events associated with this ME */
-                    fprintf(stderr, "%03d: spinning waiting for block to finish %ld, %ld\n",
-                            gasneti_mynode, ct.success + ct.failure, (long)val);
+  #ifdef P4_DEBUG
+                    fprintf(stderr, "%03d: spinning waiting for block to finish %"PRIu64", %"PRIu64"\n",
+                            gasneti_mynode, (uint64_t)(ct.success + ct.failure), (uint64_t)val);
+  #endif
                     GASNETI_WAITHOOK();
                     val = gasneti_weakatomic_read(&block->op_count, 0);
                 }
@@ -1694,3 +1700,97 @@ gasnetc_bootstrapExchange(void *src, size_t len, void *dest)
     /* should not need a barrier here */
     GASNETI_TRACE_PRINTF(C,("bootExch exit"));
 }
+
+#if GASNET_PSHM /* Used only in call to gasneti_pshm_init() */
+void gasnetc_bootstrapSNodeBroadcast(void *src, size_t len, void *dest, int rootnode) {
+    ptl_md_t md;
+    ptl_me_t me;
+    ptl_handle_me_t me_h;
+    ptl_handle_md_t md_h;
+    ptl_event_t ev;
+    const int isroot = (gasneti_mynode == rootnode);
+    int ret;
+
+    GASNETI_TRACE_PRINTF(C,("bootGrpBcast with len = %d, src = %p dest = %p",(int)len,src,dest));
+
+    if (isroot) {
+      /* MD for src */
+      md.start = src;
+      md.length = len;
+      md.options = PTL_MD_UNORDERED;
+      md.eq_handle = coll_eq_h;
+      md.ct_handle = PTL_CT_NONE;
+      ret = PtlMDBind(matching_ni_h, &md, &md_h);
+      if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlMDBind()");
+    } else {
+      /* ME for dest */
+      me.start = dest;
+      me.length = len;
+      me.ct_handle = PTL_CT_NONE;
+      me.uid = uid;
+      me.options = PTL_ME_OP_PUT;
+      me.match_id.rank = rootnode;
+      me.match_bits = BOOTSTRAP_GRPBCAST_MB;
+      me.ignore_bits = 0;
+      me.min_free = 0;
+      ret = PtlMEAppend(matching_ni_h, bootstrap_idx, &me, PTL_PRIORITY_LIST, NULL, &me_h);
+      if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlMEAppend()");
+      /* ensure ME is linked before the barrier */
+      ret = PtlEQWait(coll_eq_h, &ev);
+      if (ret != PTL_OK) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlEQWait(link)");
+      gasneti_assert( ev.type == PTL_EVENT_LINK );
+    }
+
+    gasnetc_bootstrapBarrier();
+
+    if (isroot) {
+      int i;
+
+      memmove(dest, src, len);
+
+      for (i = 0; i < gasneti_nodemap_local_count; ++i) {
+        ptl_process_t proc;
+        if (i == gasneti_nodemap_local_rank) continue;
+        proc.rank = gasneti_nodemap_local[i];
+        ret = PtlPut(md_h, 
+                     0, 
+                     len, 
+                     PTL_NO_ACK_REQ, 
+                     proc,
+                     bootstrap_idx,
+                     BOOTSTRAP_GRPBCAST_MB,
+                     0,
+                     NULL,
+                     0);
+        if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlPut()");
+      }
+
+      for (i = 1; i < gasneti_nodemap_local_count; ++i) { /* NOTE: start at 1 to skip self */
+        ret = PtlEQWait(coll_eq_h, &ev);
+        if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlEQWait(final)");
+        gasneti_assert(ev.type == PTL_EVENT_SEND);
+      }
+
+      ret = PtlMDRelease(md_h);
+      if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlMDRelease()");
+    } else {
+      ret = PtlEQWait(coll_eq_h, &ev);
+      if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlEQWait(final)");
+      gasneti_assert(ev.type == PTL_EVENT_PUT);
+      gasneti_assert(ev.rlength == ev.mlength);
+      gasneti_assert(ev.rlength == len);
+#if 1 /* see "portals4 issue #28" node in gasnetc_bootstrapExchange */
+      do {
+        ret = PtlMEUnlink(me_h);
+        if (PTL_IN_USE == ret) sleep(1);
+      } while (PTL_IN_USE == ret);
+#else
+      ret = PtlMEUnlink(me_h);
+#endif
+      if_pf (PTL_OK != ret) p4_fatalerror(ret, "gasnetc_bootstrapGrpBcast() PtlMEUnlink()");
+    }
+
+    /* should not need a barrier here */
+    GASNETI_TRACE_PRINTF(C,("bootGrpBcast exit"));
+}
+#endif
