@@ -9,6 +9,7 @@
 #include <gasnet_core_internal.h>
 #include <gasnet_core_list.h>
 #include <gasnet_extended_internal.h>
+#include <gasnet_extended_internal_extra.h>
 
 #include <psm2.h>
 #include <psm2_am.h>
@@ -17,9 +18,9 @@ static const gasnete_eopaddr_t EOPADDR_NIL = { { 0xFF, 0xFF } };
 extern void _gasnete_iop_check(gasnete_iop_t *iop) { gasnete_iop_check(iop); }
 
 void gasnete_put_long(gasnet_node_t node, void *dest, void *src,
-        size_t nbytes, gasnet_handle_t op GASNETE_THREAD_FARG);
+        size_t nbytes, gasnet_handle_t op, uint8_t isbulk GASNETI_THREAD_FARG);
 void gasnete_get_long (void *dest, gasnet_node_t node, void *src,
-        size_t nbytes, gasnet_handle_t op GASNETE_THREAD_FARG);
+        size_t nbytes, gasnet_handle_t op GASNETI_THREAD_FARG);
 
 
 /* ------------------------------------------------------------------------------------ */
@@ -40,7 +41,7 @@ void gasnete_get_long (void *dest, gasnet_node_t node, void *src,
 
 /*  allocate more eops */
 GASNETI_NEVER_INLINE(gasnete_eop_alloc,
-static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
+static void gasnete_eop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_eopaddr_t addr;
     int bufidx = thread->eop_num_bufs;
     gasnete_eop_t *buf;
@@ -126,7 +127,7 @@ static void gasnete_eop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  allocate a new iop */
 GASNETI_NEVER_INLINE(gasnete_iop_alloc,
-static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
+static gasnete_iop_t *gasnete_iop_alloc(gasneti_threaddata_t * const thread)) {
     gasnete_iop_t *iop = (gasnete_iop_t *)gasneti_malloc(sizeof(gasnete_iop_t));
     gasneti_leak(iop);
     #if GASNET_DEBUG
@@ -143,7 +144,7 @@ static gasnete_iop_t *gasnete_iop_alloc(gasnete_threaddata_t * const thread)) {
 
 /*  get a new op */
 static
-gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *_gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eopaddr_t head = thread->eop_free;
   if_pf (gasnete_eopaddr_isnil(head)) {
     gasnete_eop_alloc(thread);
@@ -166,7 +167,7 @@ gasnete_eop_t *_gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new op AND mark it in flight */
 GASNETI_INLINE(gasnete_eop_new)
-gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
+gasnete_eop_t *gasnete_eop_new(gasneti_threaddata_t * const thread) {
   gasnete_eop_t *eop = _gasnete_eop_new(thread);
 #if GASNETE_EOP_COUNTED
   eop->initiated_cnt++;
@@ -176,7 +177,7 @@ gasnete_eop_t *gasnete_eop_new(gasnete_threaddata_t * const thread) {
 
 /*  get a new iop */
 static
-gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
+gasnete_iop_t *gasnete_iop_new(gasneti_threaddata_t * const thread) {
   gasnete_iop_t *iop = thread->iop_free;
   if_pt (iop) {
     thread->iop_free = iop->next;
@@ -201,7 +202,7 @@ gasnete_iop_t *gasnete_iop_new(gasnete_threaddata_t * const thread) {
 /*  query an eop for completeness */
 static
 int gasnete_eop_isdone(gasnete_eop_t *eop) {
-  gasneti_assert(eop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(eop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_eop_check(eop);
   return GASNETE_EOP_DONE(eop);
 }
@@ -209,7 +210,7 @@ int gasnete_eop_isdone(gasnete_eop_t *eop) {
 /*  query an iop for completeness - this means both puts and gets */
 static
 int gasnete_iop_isdone(gasnete_iop_t *iop) {
-  gasneti_assert(iop->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(iop->threadidx == _gasneti_mythread_slow()->threadidx);
   gasnete_iop_check(iop);
   return (GASNETE_IOP_CNTDONE(iop,get) && GASNETE_IOP_CNTDONE(iop,put));
 }
@@ -233,9 +234,9 @@ void gasnete_op_markdone(gasnete_op_t *op, int isget) {
 /*  free an eop */
 static
 void gasnete_eop_free(gasnete_eop_t *eop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
+  gasneti_threaddata_t * const thread = gasnete_threadtable[eop->threadidx];
   gasnete_eopaddr_t addr = eop->addr;
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_eop_check(eop);
   gasneti_assert(GASNETE_EOP_DONE(eop));
 #if GASNET_DEBUG
@@ -248,8 +249,8 @@ void gasnete_eop_free(gasnete_eop_t *eop) {
 /*  free an iop */
 static
 void gasnete_iop_free(gasnete_iop_t *iop) {
-  gasnete_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
-  gasneti_assert(thread == gasnete_mythread());
+  gasneti_threaddata_t * const thread = gasnete_threadtable[iop->threadidx];
+  gasneti_assert(thread == _gasneti_mythread_slow());
   gasnete_iop_check(iop);
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,get));
   gasneti_assert(GASNETE_IOP_CNTDONE(iop,put));
@@ -286,7 +287,6 @@ static void gasnete_check_config(void) {
 
 GASNETI_COLD
 extern void gasnete_init(void) {
-  GASNETI_UNUSED_UNLESS_DEBUG
   static int firstcall = 1;
   GASNETI_TRACE_PRINTF(C,("gasnete_init()"));
   gasneti_assert(firstcall); /*  make sure we haven't been called before */
@@ -296,11 +296,11 @@ extern void gasnete_init(void) {
 
   gasneti_assert(gasneti_nodes >= 1 && gasneti_mynode < gasneti_nodes);
 
-  { gasnete_threaddata_t *threaddata = NULL;
+  { gasneti_threaddata_t *threaddata = NULL;
     gasnete_eop_t *eop = NULL;
     #if GASNETI_MAX_THREADS > 1
       /* register first thread (optimization) */
-      threaddata = gasnete_mythread();
+      threaddata = _gasneti_mythread_slow();
     #else
       /* register only thread (required) */
       threaddata = gasnete_new_threaddata();
@@ -326,12 +326,12 @@ extern void gasnete_init(void) {
 
 /* ------------------------------------------------------------------------------------ */
 /* GASNET-Internal OP Interface */
-gasneti_eop_t *gasneti_eop_create(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_eop_t *op = gasnete_eop_new(GASNETE_MYTHREAD);
+gasneti_eop_t *gasneti_eop_create(GASNETI_THREAD_FARG_ALONE) {
+  gasnete_eop_t *op = gasnete_eop_new(GASNETI_MYTHREAD);
   return (gasneti_eop_t *)op;
 }
-gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+gasneti_iop_t *gasneti_iop_register(unsigned int noperations, int isget GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t * const op = mythread->current_iop;
   gasnete_iop_check(op);
   if (isget) op->initiated_get_cnt += noperations;
@@ -369,42 +369,9 @@ void gasneti_iop_markdone(gasneti_iop_t *iop, unsigned int noperations, int isge
   ===============
 */
 
-/* Use reference implementation of get/put/memset in terms of AMs */
-/* NOTE: Barriers, Collectives, VIS may use GASNETE_USING_REF_* in algorithm selection */
-#define GASNETE_USING_REF_EXTENDED_GET_BULK 0
-#define GASNETE_USING_REF_EXTENDED_PUT_BULK 0
-#define GASNETE_USING_REF_EXTENDED_PUT      0
-#define GASNETE_USING_REF_EXTENDED_MEMSET   1
-
-#if GASNETE_USING_REF_EXTENDED_GET_BULK
-#define GASNETE_BUILD_AMREF_GET_HANDLERS 1
-#define GASNETE_BUILD_AMREF_GET_BULK     1
-#define gasnete_amref_get_nb_bulk   gasnete_get_nb_bulk
-#define gasnete_amref_get_nbi_bulk  gasnete_get_nbi_bulk
-#endif
-
-#if GASNETE_USING_REF_EXTENDED_PUT_BULK
-#define GASNETE_BUILD_AMREF_PUT_HANDLERS 1
-#define GASNETE_BUILD_AMREF_PUT_BULK     1
-#define gasnete_amref_put_nb_bulk   gasnete_put_nb_bulk
-#define gasnete_amref_put_nbi_bulk  gasnete_put_nbi_bulk
-#endif
-
-#if GASNETE_USING_REF_EXTENDED_PUT
-#define GASNETE_BUILD_AMREF_PUT_HANDLERS 1
-#define GASNETE_BUILD_AMREF_PUT     1
-#define gasnete_amref_put_nb        gasnete_put_nb
-#define gasnete_amref_put_nbi       gasnete_put_nbi
-#endif
-
-#if GASNETE_USING_REF_EXTENDED_MEMSET
-#define GASNETE_BUILD_AMREF_MEMSET_HANDLERS 1
-#define GASNETE_BUILD_AMREF_MEMSET  1
-#define gasnete_amref_memset_nb     gasnete_memset_nb
-#define gasnete_amref_memset_nbi    gasnete_memset_nbi
-#endif
-
-#define gasnete_amref_put_nbi       gasnete_put_nbi
+/*
+ * Configuration appears in gasnet_extended_fwd.h
+ */
 #include "gasnet_extended_amref.c"
 
 /* -------------------------------------------------------------------------- */
@@ -580,9 +547,9 @@ int gasnete_handler_get_reply(psm2_am_token_t token,
 */
 
 static void gasnete_put_nbi_inner (gasnet_node_t node, void *dest, void *src,
-                             size_t nbytes GASNETE_THREAD_FARG)
+                             size_t nbytes, uint8_t isbulk GASNETI_THREAD_FARG)
 {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const op = mythread->current_iop;
 
     size_t mtu_size = gasnetc_psm_max_request_len;
@@ -598,7 +565,7 @@ static void gasnete_put_nbi_inner (gasnet_node_t node, void *dest, void *src,
     if(nbytes >= gasnetc_psm_state.long_msg_threshold) {
             op->initiated_put_cnt++;
         gasnete_put_long(node, dest, src, nbytes,
-                (gasnet_handle_t)op GASNETE_THREAD_PASS);
+                (gasnet_handle_t)op, isbulk GASNETI_THREAD_PASS);
         return;
     }
 
@@ -617,6 +584,7 @@ static void gasnete_put_nbi_inner (gasnet_node_t node, void *dest, void *src,
         bytes_remaining -= mtu_size;
     }
 
+    op->initiated_put_cnt++;
     ret = psm2_am_request_short(epaddr, handler,
             (psm2_amarg_t*)&dest_addr, 1, (void*)src_addr, bytes_remaining,
             PSM2_AM_FLAG_NOREPLY,
@@ -627,25 +595,24 @@ static void gasnete_put_nbi_inner (gasnet_node_t node, void *dest, void *src,
                 psm2_error_get_string(ret));
     }
 
-    op->initiated_put_cnt++;
     gasnetc_psm_poll_periodic();
 }
 
 extern void gasnete_put_nbi (gasnet_node_t node, void *dest, void *src,
-                             size_t nbytes GASNETE_THREAD_FARG)
+                             size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_PUT(ALIGNED,V);
-    gasnete_put_nbi_inner(node, dest, src, nbytes GASNETE_THREAD_PASS);
+    GASNETI_CHECKPSHM_PUT(ALIGNED,V,node,dest,src,nbytes);
+    gasnete_put_nbi_inner(node, dest, src, nbytes, 0 GASNETI_THREAD_PASS);
 }
 
 extern void gasnete_put_nbi_bulk (gasnet_node_t node, void *dest, void *src,
-                              size_t nbytes GASNETE_THREAD_FARG) {
-    GASNETI_CHECKPSHM_PUT(UNALIGNED,V);
-    gasnete_put_nbi_inner(node, dest, src, nbytes GASNETE_THREAD_PASS);
+                              size_t nbytes GASNETI_THREAD_FARG) {
+    GASNETI_CHECKPSHM_PUT(UNALIGNED,V,node,dest,src,nbytes);
+    gasnete_put_nbi_inner(node, dest, src, nbytes, 1 GASNETI_THREAD_PASS);
 }
 
-extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t * const op = mythread->current_iop;
     uintptr_t src_addr = (uintptr_t)src;
     uintptr_t dest_addr = (uintptr_t)dest;
@@ -655,13 +622,13 @@ extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, siz
     gasnete_getreq_t* req;
     psm2_error_t ret;
 
-    GASNETI_CHECKPSHM_GET(UNALIGNED,V);
+    GASNETI_CHECKPSHM_GET(UNALIGNED,V,dest,node,src,nbytes);
     gasneti_assert(node < gasneti_nodes);
 
     if(nbytes >= gasnetc_psm_state.long_msg_threshold) {
         op->initiated_get_cnt++;
         gasnete_get_long(dest, node, src, nbytes,
-                (gasnet_handle_t)op GASNETE_THREAD_PASS);
+                (gasnet_handle_t)op GASNETI_THREAD_PASS);
         return;
     }
 
@@ -701,6 +668,7 @@ extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, siz
     args[1].u32w0 = gasnete_getreq_to_offset(req);
     args[1].u32w1 = bytes_remaining;
 
+    op->initiated_get_cnt++;
     ret = psm2_am_request_short(gasnetc_psm_state.peer_epaddrs[node],
             gasnetc_psm_state.am_handlers[AM_HANDLER_GET_REQUEST],
             args, 2, NULL, 0, PSM2_AM_FLAG_NONE, NULL, NULL);
@@ -710,7 +678,6 @@ extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, siz
                 psm2_error_get_string(ret));
     }
 
-    op->initiated_get_cnt++;
     gasnetc_psm_poll_periodic();
 }
 
@@ -729,9 +696,9 @@ extern void gasnete_get_nbi_bulk (void *dest, gasnet_node_t node, void *src, siz
 */
 
 extern gasnet_handle_t gasnete_put_nb_inner(gasnet_node_t node, void *dest,
-                               void *src, size_t nbytes GASNETE_THREAD_FARG)
+                               void *src, size_t nbytes, uint8_t isbulk GASNETI_THREAD_FARG)
 {
-    gasnete_eop_t* op = gasnete_eop_new(GASNETE_MYTHREAD);
+    gasnete_eop_t* op = gasnete_eop_new(GASNETI_MYTHREAD);
     size_t mtu_size = gasnetc_psm_max_request_len;
     size_t bytes_remaining = nbytes;
     uintptr_t src_addr = (uintptr_t)src;
@@ -742,7 +709,7 @@ extern gasnet_handle_t gasnete_put_nb_inner(gasnet_node_t node, void *dest,
 
     if(nbytes >= gasnetc_psm_state.long_msg_threshold) {
         gasnete_put_long(node, dest, src, nbytes,
-                (gasnet_handle_t)op GASNETE_THREAD_PASS);
+                (gasnet_handle_t)op, isbulk GASNETI_THREAD_PASS);
         return (gasnet_handle_t)op;
     }
 
@@ -777,19 +744,19 @@ extern gasnet_handle_t gasnete_put_nb_inner(gasnet_node_t node, void *dest,
     return (gasnet_handle_t)op;
 }
 
-extern gasnet_handle_t gasnete_put_nb(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG)
+extern gasnet_handle_t gasnete_put_nb(gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG)
 {
-    GASNETI_CHECKPSHM_PUT(ALIGNED,H);
-    return gasnete_put_nb_inner(node, dest, src, nbytes GASNETE_THREAD_PASS);
+    GASNETI_CHECKPSHM_PUT(ALIGNED,H,node,dest,src,nbytes);
+    return gasnete_put_nb_inner(node, dest, src, nbytes, 0 GASNETI_THREAD_PASS);
 }
 
-extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETE_THREAD_FARG) {
-    GASNETI_CHECKPSHM_PUT(UNALIGNED,H);
-    return gasnete_put_nb_inner(node, dest, src, nbytes GASNETE_THREAD_PASS);
+extern gasnet_handle_t gasnete_put_nb_bulk (gasnet_node_t node, void *dest, void *src, size_t nbytes GASNETI_THREAD_FARG) {
+    GASNETI_CHECKPSHM_PUT(UNALIGNED,H,node,dest,src,nbytes);
+    return gasnete_put_nb_inner(node, dest, src, nbytes, 1 GASNETI_THREAD_PASS);
 }
 
 
-extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETE_THREAD_FARG) {
+extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void *src, size_t nbytes GASNETI_THREAD_FARG) {
     gasnete_eop_t* op;
     uintptr_t src_addr = (uintptr_t)src;
     uintptr_t dest_addr = (uintptr_t)dest;
@@ -799,15 +766,15 @@ extern gasnet_handle_t gasnete_get_nb_bulk (void *dest, gasnet_node_t node, void
     gasnete_getreq_t* req;
     psm2_error_t ret;
 
-    GASNETI_CHECKPSHM_GET(UNALIGNED,H);
+    GASNETI_CHECKPSHM_GET(UNALIGNED,H,dest,node,src,nbytes);
     gasneti_assert(node < gasneti_nodes);
 
-    op = gasnete_eop_new(GASNETE_MYTHREAD);
+    op = gasnete_eop_new(GASNETI_MYTHREAD);
     mtu_size = gasnetc_psm_max_reply_len;
 
     if(nbytes >= gasnetc_psm_state.long_msg_threshold) {
         gasnete_get_long(dest, node, src, nbytes,
-                (gasnet_handle_t)op GASNETE_THREAD_PASS);
+                (gasnet_handle_t)op GASNETI_THREAD_PASS);
         return (gasnet_handle_t)op;
     }
 
@@ -873,7 +840,7 @@ GASNETI_INLINE(gasnete_op_try_free)
 int gasnete_op_try_free(gasnet_handle_t handle) {
   gasnete_op_t *op = (gasnete_op_t *)handle;
 
-  gasneti_assert(op->threadidx == gasnete_mythread()->threadidx);
+  gasneti_assert(op->threadidx == _gasneti_mythread_slow()->threadidx);
   if_pt (OPTYPE(op) == OPTYPE_EXPLICIT) {
     gasnete_eop_t *eop = (gasnete_eop_t*)op;
 
@@ -977,13 +944,13 @@ extern int  gasnete_try_syncnb_all (gasnet_handle_t *phandle, size_t numhandles)
   ===========================================================
 */
 
-extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_gets(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(OPTYPE(iop) == OPTYPE_IMPLICIT);
@@ -999,13 +966,13 @@ extern int  gasnete_try_syncnbi_gets(GASNETE_THREAD_FARG_ALONE) {
   }
 }
 
-extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
+extern int  gasnete_try_syncnbi_puts(GASNETI_THREAD_FARG_ALONE) {
   #if 0
     /* polling for syncnbi now happens in header file to avoid duplication */
     GASNETI_SAFE(gasneti_AMPoll());
   #endif
   {
-    gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+    gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
     gasnete_iop_t *iop = mythread->current_iop;
     gasneti_assert(iop->threadidx == mythread->threadidx);
     gasneti_assert(iop->next == NULL);
@@ -1030,8 +997,8 @@ extern int  gasnete_try_syncnbi_puts(GASNETE_THREAD_FARG_ALONE) {
 */
 /*  This implementation allows recursive access regions, although the spec does not require that */
 /*  operations are associated with the most immediately enclosing access region */
-extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE_THREAD_FARG) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETI_THREAD_FARG) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = gasnete_iop_new(mythread); /*  push an iop  */
   GASNETI_TRACE_PRINTF(S,("BEGIN_NBI_ACCESSREGION"));
   #if GASNET_DEBUG
@@ -1042,8 +1009,8 @@ extern void            gasnete_begin_nbi_accessregion(int allowrecursion GASNETE
   mythread->current_iop = iop;
 }
 
-extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
-  gasnete_threaddata_t * const mythread = GASNETE_MYTHREAD;
+extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETI_THREAD_FARG_ALONE) {
+  gasneti_threaddata_t * const mythread = GASNETI_MYTHREAD;
   gasnete_iop_t *iop = mythread->current_iop; /*  pop an iop */
   GASNETI_TRACE_EVENT_VAL(S,END_NBI_ACCESSREGION,iop->initiated_get_cnt + iop->initiated_put_cnt);
   #if GASNET_DEBUG
@@ -1073,7 +1040,7 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
 */
 
 /* use reference implementation of scatter/gather and strided */
-#include "gasnet_extended_refvis.h"
+#include "gasnet_refvis.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
@@ -1082,7 +1049,7 @@ extern gasnet_handle_t gasnete_end_nbi_accessregion(GASNETE_THREAD_FARG_ALONE) {
 */
 
 /* use reference implementation of collectives */
-#include "gasnet_extended_refcoll.h"
+#include "gasnet_refcoll.h"
 
 /* ------------------------------------------------------------------------------------ */
 /*
