@@ -13,7 +13,7 @@
 
 #define TEST_GASNET 1
 #define SHORT_REQ_BASE 128
-#include <other/amxtests/testam.h>
+#include <other/amx/testam.h>
 
 /* Define to get one big function that pushes the gcc inliner heursitics */
 #undef TESTGASNET_NO_SPLIT
@@ -23,8 +23,10 @@ TEST_BACKTRACE_DECLS();
 void doit(int partner, int *partnerseg);
 void doit2(int partner, int *partnerseg);
 void doit3(int partner, int *partnerseg);
-void doit4(int partner, int *partnerseg);
+void doit4(int partner, int32_t *partnerseg);
 void doit5(int partner, int *partnerseg);
+void doit6(int partner, int *partnerseg);
+void doit7(int partner, int *partnerseg);
 
 /* ------------------------------------------------------------------------------------ */
 #if GASNET_SEGMENT_EVERYTHING
@@ -39,10 +41,10 @@ void doit5(int partner, int *partnerseg);
   test_everything_seginfo_t myinfo;
   test_everything_seginfo_t partnerinfo;
   int done = 0;
-  void seg_everything_reqh(gasnet_token_t token) {
+  GASNETT_EXTERNC void seg_everything_reqh(gasnet_token_t token) {
     GASNET_Safe(gasnet_AMReplyMedium0(token, 251, &myinfo, sizeof(test_everything_seginfo_t)));
   }
-  void seg_everything_reph(gasnet_token_t token, void *buf, size_t nbytes) {
+  GASNETT_EXTERNC void seg_everything_reph(gasnet_token_t token, void *buf, size_t nbytes) {
     assert(nbytes == sizeof(test_everything_seginfo_t));
     memcpy(&partnerinfo, buf, nbytes);
     gasnett_local_wmb();
@@ -95,19 +97,16 @@ void doit5(int partner, int *partnerseg);
 #endif
 
 #if GASNET_PAR
-#if (TEST_MAXTHREADS < 10)
-  #define NUM_THREADS TEST_MAXTHREADS
+  #define MAX_THREADS 10
 #else
-  #define NUM_THREADS 10
-#endif
-#else
-  #define NUM_THREADS 1
+  #define MAX_THREADS 1
 #endif
+int num_threads = MAX_THREADS;
 
 void test_threadinfo(int threadid, int numthreads) {
   int i;
   gasnet_threadinfo_t my_ti;
-  static gasnet_threadinfo_t all_ti[NUM_THREADS];
+  static gasnet_threadinfo_t all_ti[MAX_THREADS];
 
   { GASNET_BEGIN_FUNCTION();
     my_ti = GASNET_GET_THREADINFO();
@@ -119,7 +118,7 @@ void test_threadinfo(int threadid, int numthreads) {
     gasnet_threadinfo_t ti = GASNET_GET_THREADINFO();
     assert_always(ti == my_ti);
   }
-  assert(threadid < numthreads && numthreads <= NUM_THREADS);
+  assert(threadid < numthreads && numthreads <= MAX_THREADS);
   all_ti[threadid] = my_ti;
   PTHREAD_LOCALBARRIER(numthreads);
   for (i = 0; i < numthreads; i++) {
@@ -136,17 +135,15 @@ void test_threadinfo(int threadid, int numthreads) {
   #endif
   void *test_libgasnetpar_tools(void *p) {
     int idx = (int)(uintptr_t)p;
-    PTHREAD_LOCALBARRIER(NUM_THREADS);
-    test_threadinfo(idx, NUM_THREADS);
-    PTHREAD_LOCALBARRIER(NUM_THREADS);
-  #if GASNETI_ARCH_ALTIX
-    /* Don't pin threads because system is either shared or using cgroups */
-  #elif GASNETI_ARCH_IBMPE
+    PTHREAD_LOCALBARRIER(num_threads);
+    test_threadinfo(idx, num_threads);
+    PTHREAD_LOCALBARRIER(num_threads);
+  #if GASNETI_ARCH_IBMPE
     /* Don't pin threads because system s/w will have already done so */
   #else
     gasnett_set_affinity(idx);
   #endif
-    PTHREAD_LOCALBARRIER(NUM_THREADS);
+    PTHREAD_LOCALBARRIER(num_threads);
     return NULL;
   }
 #endif
@@ -183,7 +180,8 @@ void test_libgasnet_tools(void) {
   }
   #endif
   #if GASNET_PAR
-    test_createandjoin_pthreads(NUM_THREADS, &test_libgasnetpar_tools, NULL, 0);
+    num_threads = test_thread_limit(num_threads);
+    test_createandjoin_pthreads(num_threads, &test_libgasnetpar_tools, NULL, 0);
   #endif
   MSG("*** passed libgasnet_tools test!!");
 }
@@ -435,9 +433,9 @@ void doit3(int partner, int *partnerseg) {
   }
 
 #ifndef TESTGASNET_NO_SPLIT
-  doit4(partner, partnerseg);
+  doit4(partner, (int32_t *)partnerseg);
 }
-void doit4(int partner, int *partnerseg) {
+void doit4(int partner, int32_t *partnerseg) {
   /* int mynode = gasnet_mynode(); UNUSED */
 #endif
 
@@ -446,28 +444,28 @@ void doit4(int partner, int *partnerseg) {
   { /*  memset test */
     GASNET_BEGIN_FUNCTION();
     int i, success=1;
-    int vals[300];
+    int32_t vals[300];
 
-    gasnet_memset(partner, partnerseg, 0x55, 100*sizeof(int));
-    gasnet_wait_syncnb(gasnet_memset_nb(partner, partnerseg+100, 0x66, 100*sizeof(int)));
-    gasnet_memset_nbi(partner, partnerseg+200, 0x77, 100*sizeof(int));
+    gasnet_memset(partner, partnerseg, 0x55, 100*sizeof(int32_t));
+    gasnet_wait_syncnb(gasnet_memset_nb(partner, partnerseg+100, 0x66, 100*sizeof(int32_t)));
+    gasnet_memset_nbi(partner, partnerseg+200, 0x77, 100*sizeof(int32_t));
     gasnet_wait_syncnbi_puts();
 
-    gasnet_get(&vals, partner, partnerseg, 300*sizeof(int));
+    gasnet_get(&vals, partner, partnerseg, 300*sizeof(int32_t));
 
     for (i=0; i < 100; i++) {
-      unsigned long long five  = 0x5555555555555555ull;
-      unsigned long long six   = 0x6666666666666666ull;
-      unsigned long long seven = 0x7777777777777777ull;
-      if (vals[i] != ((int)five)) {
+      const int32_t five  = 0x55555555;
+      const int32_t six   = 0x66666666;
+      const int32_t seven = 0x77777777;
+      if (vals[i] != five) {
         MSG("*** ERROR - FAILED MEMSET TEST!!!");
         success = 0;
       }
-      if (vals[i+100] != ((int)six)) {
+      if (vals[i+100] != six) {
         MSG("*** ERROR - FAILED MEMSET TEST!!!");
         success = 0;
       }
-      if (vals[i+200] != ((int)seven)) {
+      if (vals[i+200] != seven) {
         MSG("*** ERROR - FAILED MEMSET TEST!!!");
         success = 0;
       }
@@ -476,7 +474,7 @@ void doit4(int partner, int *partnerseg) {
   }
 
 #ifndef TESTGASNET_NO_SPLIT
-  doit5(partner, partnerseg);
+  doit5(partner, (int *)partnerseg);
 }
 void doit5(int partner, int *partnerseg) {
   int mynode = gasnet_mynode();
@@ -539,15 +537,15 @@ void doit5(int partner, int *partnerseg) {
           ok = localpos[j] == val;
           if (sz < 8) ok = !memcmp(&(localpos[j]), &val, sz);
           if (!ok) {
-              MSG("*** ERROR - FAILED OUT-OF-SEG PUT_NB/OVERWRITE TEST!!! sz=%i j=%i (got=%016llx expected=%016llx)", (sz), j,
-                  (unsigned long long)localpos[j], (unsigned long long)val);
+              MSG("*** ERROR - FAILED OUT-OF-SEG PUT_NB/OVERWRITE TEST!!! sz=%i j=%i (got=%016" PRIx64 " expected=%016" PRIx64 ")",
+                  sz, j, localpos[j], val);
               success = 0;
           }
           ok = segpos[j] == val;
           if (sz < 8) ok = !memcmp(&(segpos[j]), &val, sz);
           if (!ok) {
-              MSG("*** ERROR - FAILED IN-SEG PUT_NB/OVERWRITE TEST!!! sz=%i j=%i (got=%016llx expected=%016llx)", (sz), j,
-                  (unsigned long long)segpos[j], (unsigned long long)val);
+              MSG("*** ERROR - FAILED IN-SEG PUT_NB/OVERWRITE TEST!!! sz=%i j=%i (got=%016" PRIx64 " expected=%016" PRIx64 ")",
+                  sz, j, segpos[j], val);
               success = 0;
           }
         }
@@ -607,15 +605,15 @@ void doit5(int partner, int *partnerseg) {
           ok = localpos[j] == val;
           if (sz < 8) ok = !memcmp(&(localpos[j]), &val, sz);
           if (!ok) {
-              MSG("*** ERROR - FAILED OUT-OF-SEG PUT_NBI/OVERWRITE TEST!!! sz=%i j=%i (got=%016llx expected=%016llx)", (sz), j,
-                  (unsigned long long)localpos[j], (unsigned long long)val);
+              MSG("*** ERROR - FAILED OUT-OF-SEG PUT_NBI/OVERWRITE TEST!!! sz=%i j=%i (got=%016" PRIx64 " expected=%016" PRIx64 ")",
+                  sz, j, localpos[j], val);
               success = 0;
           }
           ok = segpos[j] == val;
           if (sz < 8) ok = !memcmp(&(segpos[j]), &val, sz);
           if (!ok) {
-              MSG("*** ERROR - FAILED IN-SEG PUT_NBI/OVERWRITE TEST!!! sz=%i j=%i (got=%016llx expected=%016llx)", (sz), j,
-                  (unsigned long long)segpos[j], (unsigned long long)val);
+              MSG("*** ERROR - FAILED IN-SEG PUT_NBI/OVERWRITE TEST!!! sz=%i j=%i (got=%016" PRIx64 " expected=%016" PRIx64 ")",
+                  sz, j, segpos[j], val);
               success = 0;
           }
         }
@@ -625,6 +623,12 @@ void doit5(int partner, int *partnerseg) {
     if (success) MSG("*** passed nbi put/overwrite test!!");
   }
 
+#ifndef TESTGASNET_NO_SPLIT
+  doit6(partner, (int *)partnerseg);
+}
+void doit6(int partner, int *partnerseg) {
+#endif
+
   BARRIER();
 
   { /* all ams test */
@@ -640,6 +644,12 @@ void doit5(int partner, int *partnerseg) {
     MSG("*** passed AM test!!");
   }
 
+#ifndef TESTGASNET_NO_SPLIT
+  doit7(partner, (int *)partnerseg);
+}
+void doit7(int partner, int *partnerseg) {
+#endif
+
   BARRIER();
 
   /* Invoke all the atomics, once each.
@@ -677,6 +687,7 @@ void doit5(int partner, int *partnerseg) {
   } while(0)
   {
     gasnett_atomic_sval_t stmp = gasnett_atomic_signed((gasnett_atomic_val_t)0);
+    gasnett_atomic_increment((gasnett_atomic_t*)&stmp,0);
     TEST_ATOMICS(gasnett_atomic_val_t, atomic);
     TEST_ATOMICS(gasnett_atomic_val_t, strongatomic);
     TEST_ATOMICS(uint32_t, atomic32);
@@ -702,13 +713,15 @@ void doit5(int partner, int *partnerseg) {
       (void)gasnett_atomic64_compare_and_swap(ptr64, 0, 1, 0);
     }
     { double dbl = 1.0;
-      ptr64 = (gasnett_atomic64_t *)(void *)&dbl; /* (void*) suppresses g++ warning (bug 2158) */
+      uintptr_t tmp = (uintptr_t)&dbl;
+      ptr64 = (gasnett_atomic64_t *)tmp; /* conversion suppresses gcc-4 warning (bug 2158) */
       tmp64 = gasnett_atomic64_read(ptr64, 0);
       gasnett_atomic64_set(ptr64, tmp64, 0);
       (void)gasnett_atomic64_compare_and_swap(ptr64, 0, 1, 0);
     }
     { struct { char c; double dbl; } s = {0, 1.0};
-      ptr64 = (gasnett_atomic64_t *)(void *)&s.dbl; /* (void*) suppresses g++ warning (bug 2158) */
+      uintptr_t tmp = (uintptr_t)&s.dbl;
+      ptr64 = (gasnett_atomic64_t *)tmp; /* conversion suppresses gcc-4 warning (bug 2158) */
       tmp64 = gasnett_atomic64_read(ptr64, 0);
       gasnett_atomic64_set(ptr64, tmp64, 0);
       (void)gasnett_atomic64_compare_and_swap(ptr64, 0, 1, 0);
