@@ -176,10 +176,10 @@ void doit1(void) { GASNET_BEGIN_FUNCTION();
       { gasnet_AMRequestMedium0(mynode, hidx_justreply_medhandler, p, 0); });
 
     TIME_OPERATION("Loopback do-nothing gasnet_AMRequestLong0()",
-      { gasnet_AMRequestLong0(mynode, hidx_null_medhandler, p, 0, myseg); });
+      { gasnet_AMRequestLong0(mynode, hidx_null_longhandler, p, 0, myseg); });
 
     TIME_OPERATION("Loopback do nothing AM long request-reply",
-      { gasnet_AMRequestLong0(mynode, hidx_justreply_medhandler, p, 0, myseg); });
+      { gasnet_AMRequestLong0(mynode, hidx_justreply_longhandler, p, 0, myseg); });
 
     doit2();
 }
@@ -195,9 +195,15 @@ void doit2(void) { GASNET_BEGIN_FUNCTION();
 
     #if defined(GASNET_PAR) || defined (GASNET_PARSYNC)
       { static gasnett_mutex_t mutex = GASNETT_MUTEX_INITIALIZER;
-        TIME_OPERATION("lock/unlock uncontended pthread mutex",
+        TIME_OPERATION("lock/unlock uncontended gasnet mutex",
           { gasnett_mutex_lock(&mutex); gasnett_mutex_unlock(&mutex); });
       }
+      { static gasnett_rwlock_t rwlock = GASNETT_RWLOCK_INITIALIZER;
+        TIME_OPERATION("rdlock/unlock uncontended gasnet rwlock",
+          { gasnett_rwlock_rdlock(&rwlock); gasnett_rwlock_unlock(&rwlock); });
+        TIME_OPERATION("wrlock/unlock uncontended gasnet rwlock",
+          { gasnett_rwlock_wrlock(&rwlock); gasnett_rwlock_unlock(&rwlock); });
+      }
     #endif
 
     TIME_OPERATION("lock/unlock uncontended HSL (" _STRINGIFY(TEST_PARSEQ) " mode)",
@@ -288,7 +294,7 @@ GASNETT_THREADKEY_DEFINE(key);
 void doit3(void) { 
   void * volatile x = 0;
   volatile gasnet_threadinfo_t ti;
-  volatile uintptr_t y = 0;
+  static volatile uintptr_t y = 0;
 
   TEST_SECTION_BEGIN();
   { GASNET_BEGIN_FUNCTION();
@@ -312,6 +318,8 @@ void doit3(void) {
    */
   TIME_OPERATION("GASNET_BEGIN_FUNCTION (" _STRINGIFY(TEST_PARSEQ) " mode)", 
       { GASNET_BEGIN_FUNCTION(); });
+  TIME_OPERATION("GASNET_BEGIN_FUNCTION (" _STRINGIFY(TEST_PARSEQ) " mode) w/possible use", 
+      { GASNET_BEGIN_FUNCTION(); if (y) y ^= (uintptr_t)GASNET_GET_THREADINFO(); });
   memset((void *)&ti,0,sizeof(ti));
   TIME_OPERATION("GASNET_POST_THREADINFO (" _STRINGIFY(TEST_PARSEQ) " mode)", 
       { GASNET_POST_THREADINFO(ti); });
@@ -426,7 +434,7 @@ void doit7(void) { GASNET_BEGIN_FUNCTION();
       { gasnet_wait_syncnb(GASNET_INVALID_HANDLE);  });
 
     TIME_OPERATION("do-nothing gasnet_try_syncnb()",
-      { GASNETI_UNUSED int junk = gasnet_try_syncnb(GASNET_INVALID_HANDLE); });
+      { int junk = gasnet_try_syncnb(GASNET_INVALID_HANDLE); });
 
     TIME_OPERATION("do-nothing gasnet_wait_syncnb_all() (8 handles)",
       { gasnet_wait_syncnb_all(handles, 8); });
@@ -450,13 +458,13 @@ void doit7(void) { GASNET_BEGIN_FUNCTION();
       { gasnet_wait_syncnbi_gets(); });
 
     TIME_OPERATION("do-nothing gasnet_try_syncnbi_all()",
-      { GASNETI_UNUSED int junk = gasnet_try_syncnbi_all(); });
+      { int junk = gasnet_try_syncnbi_all(); });
 
     TIME_OPERATION("do-nothing gasnet_try_syncnbi_puts()",
-      { GASNETI_UNUSED int junk = gasnet_try_syncnbi_puts(); });
+      { int junk = gasnet_try_syncnbi_puts(); });
 
     TIME_OPERATION("do-nothing gasnet_try_syncnbi_gets()",
-      { GASNETI_UNUSED int junk = gasnet_try_syncnbi_gets(); });
+      { int junk = gasnet_try_syncnbi_gets(); });
 
     TIME_OPERATION("do-nothing begin/end nbi accessregion",
       { gasnet_begin_nbi_accessregion();
@@ -491,31 +499,31 @@ void doit8(void) { GASNET_BEGIN_FUNCTION();
     }
 
     TIME_OPERATION("1024-byte gasnett_count0s()",
-      { GASNETI_UNUSED int junk = gasnett_count0s(s, 1024); });
+      { int junk = gasnett_count0s(s, 1024); });
     TIME_OPERATION("1024-byte gasnett_count0s_copy()",
-      { GASNETI_UNUSED int junk = gasnett_count0s_copy(d, s, 1024); });
+      { int junk = gasnett_count0s_copy(d, s, 1024); });
     TIME_OPERATION("1024-byte gasnett_count0s() + memcpy()",
-      { GASNETI_UNUSED int junk = gasnett_count0s(s, 1024);
+      { int junk = gasnett_count0s(s, 1024);
         (void)memcpy(d,s,1024);
       });
 
     s += sizeof(void*) / 2;
     d += sizeof(void*) / 2;
     TIME_OPERATION("unaligned 1024-byte gasnett_count0s()",
-      { GASNETI_UNUSED int junk = gasnett_count0s(s, 1024); });
+      { int junk = gasnett_count0s(s, 1024); });
     TIME_OPERATION("unaligned 1024-byte gasnett_count0s_copy()",
-      { GASNETI_UNUSED int junk = gasnett_count0s_copy(d, s, 1024); });
+      { int junk = gasnett_count0s_copy(d, s, 1024); });
     TIME_OPERATION("unaligned 1024-byte gasnett_count0s() + memcpy()",
-      { GASNETI_UNUSED int junk = gasnett_count0s(s, 1024);
+      { int junk = gasnett_count0s(s, 1024);
         (void)memcpy(d,s,1024);
       });
 
     s -= 1;
     d += 1;
     TIME_OPERATION("misaligned 1024-byte gasnett_count0s_copy()",
-      { GASNETI_UNUSED int junk = gasnett_count0s_copy(d, s, 1024); });
+      { int junk = gasnett_count0s_copy(d, s, 1024); });
     TIME_OPERATION("misaligned 1024-byte gasnett_count0s() + memcpy()",
-      { GASNETI_UNUSED int junk = gasnett_count0s(s, 1024);
+      { int junk = gasnett_count0s(s, 1024);
         (void)memcpy(d,s,1024);
       });
 
