@@ -7,6 +7,9 @@
 #include <gasnet_internal.h>
 #include <gasnet_handler.h>
 #include <gasnet_core_internal.h>
+#if GASNET_BLCR
+#include <gasnet_blcr.h>
+#endif
 
 #include <amudp_spmd.h>
 
@@ -26,15 +29,16 @@ static void gasnetc_on_exit(int, void*);
 static void gasnetc_atexit(void);
 #endif
 
+static uint64_t gasnetc_networkpid;
 eb_t gasnetc_bundle;
 ep_t gasnetc_endpoint;
 
 gasneti_mutex_t gasnetc_AMlock = GASNETI_MUTEX_INITIALIZER; /*  protect access to AMUDP */
 volatile int gasnetc_AMLockYield = 0;
 
-#if GASNET_PSHM
+#ifdef GASNETC_MAX_NUMHANDLERS
   gasneti_handler_fn_t gasnetc_handler[GASNETC_MAX_NUMHANDLERS]; /* shadow handler table */
-#endif /* GASNET_PSHM */
+#endif /* GASNETC_MAX_NUMHANDLERS */
 
 #if GASNET_TRACE
   extern void gasnetc_enteringHandler_hook(amudp_category_t cat, int isReq, int handlerId, void *token, 
@@ -45,8 +49,6 @@ volatile int gasnetc_AMLockYield = 0;
 #if defined(GASNET_CSPAWN_CMD)
   #define GASNETC_DEFAULT_SPAWNFN C
   GASNETI_IDENT(gasnetc_IdentString_Default_CSpawnCommand, "$GASNetCSpawnCommand: " GASNET_CSPAWN_CMD " $");
-#elif defined(REXEC)
-  #define GASNETC_DEFAULT_SPAWNFN R
 #else /* AMUDP implicit ssh startup */
   #define GASNETC_DEFAULT_SPAWNFN S
 #endif
@@ -83,6 +85,17 @@ void gasnetc_bootstrapExchange(void *src, size_t len, void *dest) {
   if_pf (retval) gasneti_fatalerror("failure in gasnetc_bootstrapExchange()");
 }
 
+#if GASNET_PSHM /* Used only in call to gasneti_pshm_init() */
+/* Naive (poorly scaling) "reference" implementation via gasnetc_bootstrapExchange() */
+static void gasnetc_bootstrapSNodeBroadcast(void *src, size_t len, void *dest, int rootnode) {
+  void *tmp = gasneti_malloc(len * gasneti_nodes);
+  gasneti_assert(NULL != src);
+  gasnetc_bootstrapExchange(src, len, tmp);
+  memcpy(dest, (void*)((uintptr_t)tmp + (len * rootnode)), len);
+  gasneti_free(tmp);
+}
+#endif
+
 #define INITERR(type, reason) do {                                      \
    if (gasneti_VerboseErrors) {                                         \
      fprintf(stderr, "GASNet initialization encountered an error: %s\n" \
@@ -100,7 +113,7 @@ static int gasnetc_init(int *argc, char ***argv) {
   gasnetc_check_config();
 
   /* --------- begin Master code ------------ */
-  if (!AMUDP_SPMDIsWorker(*argv)) { 
+  if (!AMUDP_SPMDIsWorker(argv?*argv:NULL)) {
     /* assume we're an implicit master 
        (we don't currently support explicit workers spawned 
         without using the AMUDP SPMD API)   
@@ -110,6 +123,10 @@ static int gasnetc_init(int *argc, char ***argv) {
     char spawnfn;
     amudp_spawnfn_t fp = (amudp_spawnfn_t)NULL;
 
+    if (!argv) {
+      gasneti_fatalerror("implicit-master without argv not supported - use amudprun");
+    }
+
     /* pretend we're node 0, for purposes of verbose env reporting */
     gasneti_init_done = 1;
     gasneti_mynode = 0;
@@ -200,20 +217,40 @@ static int gasnetc_init(int *argc, char ***argv) {
 
     gasneti_freezeForDebugger();
 
-    AMUDP_VerboseErrors = gasneti_VerboseErrors;
+    AMX_VerboseErrors = gasneti_VerboseErrors;
     AMUDP_SPMDkillmyprocess = gasneti_killmyprocess;
 
+#if GASNETI_CALIBRATE_TSC
+    // Early x86*/Linux timer initialization before AMUDP_SPMDStartup()
+    //
+    // udp-conduit does not support user-provided values for GASNET_TSC_RATE*
+    // (which fine-tune timer calibration on x86/Linux).  This is partially due
+    // to a dependency cycle at startup with envvar propagation, but more
+    // importantly because the retransmission algorithm (and hence all conduit
+    // comms) rely on gasnet timers to be accurate (at least approximately), so
+    // we don't allow the user to weaken or disable their calibration.
+    gasneti_unsetenv("GASNET_TSC_RATE");
+    gasneti_unsetenv("GASNET_TSC_RATE_TOLERANCE");
+    gasneti_unsetenv("GASNET_TSC_RATE_HARD_TOLERANCE");
+    GASNETI_TICKS_INIT();
+#endif
+
     /*  perform job spawn */
     retval = AMUDP_SPMDStartup(argc, argv, 
       0, 0, NULL, /* dummies */
-      NULL, &gasnetc_bundle, &gasnetc_endpoint);
+      &gasnetc_networkpid, &gasnetc_bundle, &gasnetc_endpoint);
     if (retval != AM_OK) INITERR(RESOURCE, "slave AMUDP_SPMDStartup() failed");
     gasneti_init_done = 1; /* enable early to allow tracing */
 
-    gasneti_conduit_getenv = (/* cast drops const */ gasneti_getenv_fn_t*)&AMUDP_SPMDgetenvMaster;
+    gasneti_getenv_hook = (/* cast drops const */ gasneti_getenv_fn_t*)&AMUDP_SPMDgetenvMaster;
     gasneti_mynode = AMUDP_SPMDMyProc();
     gasneti_nodes = AMUDP_SPMDNumProcs();
 
+#if !GASNETI_CALIBRATE_TSC
+    /* Must init timers after global env, and preferably before tracing */
+    GASNETI_TICKS_INIT();
+#endif
+
     /* enable tracing */
     gasneti_trace_init(argc, argv);
     GASNETI_AM_SAFE(AMUDP_SPMDSetExitCallback(gasnetc_traceoutput));
@@ -232,7 +269,7 @@ static int gasnetc_init(int *argc, char ***argv) {
     gasneti_nodemapInit(&gasnetc_bootstrapExchange, NULL, 0, 0);
 
     #if GASNET_PSHM
-      gasneti_pshm_init(&gasnetc_bootstrapExchange, 0);
+      gasneti_pshm_init(&gasnetc_bootstrapSNodeBroadcast, 0);
     #endif
 
     #if GASNET_SEGMENT_FAST || GASNET_SEGMENT_LARGE
@@ -252,6 +289,11 @@ static int gasnetc_init(int *argc, char ***argv) {
       #error Bad segment config
     #endif
 
+    #if GASNET_BLCR
+      gasneti_checkpoint_guid = gasnetc_networkpid;
+      gasneti_checkpoint_init(NULL);
+    #endif
+
   AMUNLOCK();
 
   gasneti_auxseg_init(); /* adjust max seg values based on auxseg */
@@ -283,56 +325,14 @@ extern void _gasnetc_set_waitmode(int wait_mode) {
   }
 }
 /* ------------------------------------------------------------------------------------ */
-static char checkuniqhandler[256] = { 0 };
-static int gasnetc_reghandlers(gasnet_handlerentry_t *table, int numentries,
-                               int lowlimit, int highlimit,
-                               int dontcare, int *numregistered) {
-  int i;
-  *numregistered = 0;
-  for (i = 0; i < numentries; i++) {
-    int newindex;
-
-    if ((table[i].index == 0 && !dontcare) || 
-        (table[i].index && dontcare)) continue;
-    else if (table[i].index) newindex = table[i].index;
-    else { /* deterministic assignment of dontcare indexes */
-      for (newindex = lowlimit; newindex <= highlimit; newindex++) {
-        if (!checkuniqhandler[newindex]) break;
-      }
-      if (newindex > highlimit) {
-        char s[255];
-        snprintf(s, sizeof(s), "Too many handlers. (limit=%i)", highlimit - lowlimit + 1);
-        GASNETI_RETURN_ERRR(BAD_ARG, s);
-      }
-    }
-
-    /*  ensure handlers fall into the proper range of pre-assigned values */
-    if (newindex < lowlimit || newindex > highlimit) {
-      char s[255];
-      snprintf(s, sizeof(s), "handler index (%i) out of range [%i..%i]", newindex, lowlimit, highlimit);
-      GASNETI_RETURN_ERRR(BAD_ARG, s);
-    }
-
-    /* discover duplicates */
-    if (checkuniqhandler[newindex] != 0) 
-      GASNETI_RETURN_ERRR(BAD_ARG, "handler index not unique");
-    checkuniqhandler[newindex] = 1;
-
-    /* register the handler */
-    if (AM_SetHandler(gasnetc_endpoint, (handler_t)newindex, table[i].fnptr) != AM_OK) 
-      GASNETI_RETURN_ERRR(RESOURCE, "AM_SetHandler() failed while registering handlers");
+extern int gasnetc_amregister(gasnet_handler_t index, gasneti_handler_fn_t fnptr) {
+  if (AM_SetHandler(gasnetc_endpoint, (handler_t)index, fnptr) != AM_OK)
+    GASNETI_RETURN_ERRR(RESOURCE, "AM_SetHandler() failed while registering handlers");
 #if GASNET_PSHM
-    /* Maintain a shadown handler table for AMPSHM */
-    gasnetc_handler[(gasnet_handler_t)newindex] = (gasneti_handler_fn_t)table[i].fnptr;
+  /* Maintain a shadown handler table for AMPSHM */
+  gasneti_assert(gasnetc_handler[index] == gasneti_defaultAMHandler);
+  gasnetc_handler[index] = fnptr;
 #endif
-
-    /* The check below for !table[i].index is redundant and present
-     * only to defeat the over-aggressive optimizer in pathcc 2.1
-     */
-    if (dontcare && !table[i].index) table[i].index = newindex;
-
-    (*numregistered)++;
-  }
   return GASNET_OK;
 }
 /* ------------------------------------------------------------------------------------ */
@@ -341,8 +341,8 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
   int retval = GASNET_OK;
   void *segbase = NULL;
   
-  GASNETI_TRACE_PRINTF(C,("gasnetc_attach(table (%i entries), segsize=%lu, minheapoffset=%lu)",
-                          numentries, (unsigned long)segsize, (unsigned long)minheapoffset));
+  GASNETI_TRACE_PRINTF(C,("gasnetc_attach(table (%i entries), segsize=%"PRIuPTR", minheapoffset=%"PRIuPTR")",
+                          numentries, segsize, minheapoffset));
   AMLOCK();
     if (!gasneti_init_done) 
       INITERR(NOT_INIT, "GASNet attach called before init");
@@ -372,8 +372,8 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
 
     /* ------------------------------------------------------------------------------------ */
     /*  register handlers */
-#if GASNET_PSHM
-    /* Initialize AMPSHM's shadow handler table */
+#ifdef GASNETC_MAX_NUMHANDLERS
+    /* Initialize shadow handler table */
     { int i;
       for (i=0; i<GASNETC_MAX_NUMHANDLERS; i++)
           gasnetc_handler[i]=(gasneti_handler_fn_t)&gasneti_defaultAMHandler;
@@ -385,7 +385,7 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
       int numreg = 0;
       gasneti_assert(ctable);
       while (ctable[len].fnptr) len++; /* calc len */
-      if (gasnetc_reghandlers(ctable, len, 1, 63, 0, &numreg) != GASNET_OK)
+      if (gasneti_amregister(ctable, len, 1, 63, 0, &numreg) != GASNET_OK)
         INITERR(RESOURCE,"Error registering core API handlers");
       gasneti_assert(numreg == len);
     }
@@ -396,7 +396,7 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
       int numreg = 0;
       gasneti_assert(etable);
       while (etable[len].fnptr) len++; /* calc len */
-      if (gasnetc_reghandlers(etable, len, 64, 127, 0, &numreg) != GASNET_OK)
+      if (gasneti_amregister(etable, len, 64, 127, 0, &numreg) != GASNET_OK)
         INITERR(RESOURCE,"Error registering extended API handlers");
       gasneti_assert(numreg == len);
     }
@@ -406,12 +406,12 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
       int numreg2 = 0;
 
       /*  first pass - assign all fixed-index handlers */
-      if (gasnetc_reghandlers(table, numentries, 128, 255, 0, &numreg1) != GASNET_OK)
+      if (gasneti_amregister(table, numentries, 128, 255, 0, &numreg1) != GASNET_OK)
         INITERR(RESOURCE,"Error registering fixed-index client handlers");
 
       /*  second pass - fill in dontcare-index handlers */
-      if (gasnetc_reghandlers(table, numentries, 128, 255, 1, &numreg2) != GASNET_OK)
-        INITERR(RESOURCE,"Error registering fixed-index client handlers");
+      if (gasneti_amregister(table, numentries, 128, 255, 1, &numreg2) != GASNET_OK)
+        INITERR(RESOURCE,"Error registering variable-index client handlers");
 
       gasneti_assert(numreg1 + numreg2 == numentries);
     }
@@ -474,7 +474,10 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
 
   GASNETI_TRACE_PRINTF(C,("gasnetc_attach(): primary attach complete\n"));
 
-  gasneti_auxseg_attach(); /* provide auxseg */
+  /* (###) exchange_fn is optional (may be NULL) and is only used with GASNET_SEGMENT_EVERYTHING
+           if your conduit has an optimized bootstrapExchange pass it in place of NULL
+   */
+  gasneti_auxseg_attach(NULL); /* provide auxseg */
 
   gasnete_init(); /* init the extended API */
 
@@ -507,6 +510,10 @@ static void gasnetc_traceoutput(int exitcode) {
   if (!gasnetc_exitcalled) {
     gasneti_flush_streams();
     gasneti_trace_finish();
+
+  #if GASNET_PSHM
+    gasneti_pshm_fini();
+  #endif
   }
 }
 extern void gasnetc_trace_finish(void) {
@@ -546,8 +553,8 @@ extern void gasnetc_trace_finish(void) {
       GASNETI_STATS_PRINTF(C,("AMUDP Statistics:"));
       if (!isglobal)
         GASNETI_STATS_PRINTF(C,("*** AMUDP stat dump reflects only local node info, because gasnet_exit is non-collective ***"));
-        statdump = AMUDP_DumpStatistics(NULL, &stats, isglobal);
-        GASNETI_STATS_PRINTF(C,("\n%s",statdump)); /* note, dump has embedded '%' chars */
+      statdump = AMUDP_DumpStatistics(NULL, &stats, isglobal);
+      GASNETI_STATS_PRINTF(C,("\n%s",statdump)); /* note, dump has embedded '%' chars */
       GASNETI_STATS_PRINTF(C,("--------------------------------------------------------------------------------"));
     }
   }
@@ -586,6 +593,10 @@ extern void gasnetc_exit(int exitcode) {
   */
   AMLOCK_CAUTIOUS();
 
+  #if GASNET_PSHM
+    gasneti_pshm_fini();
+  #endif
+
   AMUDP_SPMDExit(exitcode);
   gasneti_fatalerror("AMUDP_SPMDExit failed!");
 }
@@ -625,7 +636,14 @@ extern int gasnetc_AMPoll(void) {
   gasneti_AMPSHMPoll(0);
 #endif
   AMLOCK();
+  // In single-supernode case never need to poll the network for client AMs.
+  // However, we'll still check for control traffic for orderly exit handling.
+  if (gasneti_mysupernode.grp_count > 1) {
     GASNETI_AM_SAFE_NORETURN(retval,AM_Poll(gasnetc_bundle));
+  } else {
+    // TODO-EX: a lock-free peek would allow elimination of a lock cycle
+    GASNETI_AM_SAFE_NORETURN(retval,AMUDP_SPMDHandleControlTraffic(NULL));
+  }
   AMUNLOCK();
   if_pf (retval) GASNETI_RETURN_ERR(RESOURCE);
   else return GASNET_OK;
@@ -647,6 +665,7 @@ extern int gasnetc_AMRequestShortM(
   va_start(argptr, numargs); /*  pass in last argument */
 #if GASNET_PSHM
   if_pt (gasneti_pshm_in_supernode(dest)) {
+    gasneti_AMPoll(); /* poll at least once, to assure forward progress */
     retval = gasneti_AMPSHM_RequestGeneric(gasnetc_Short, dest, handler,
                                            0, 0, 0,
                                            numargs, argptr);
@@ -675,14 +694,13 @@ extern int gasnetc_AMRequestMediumM(
   va_start(argptr, numargs); /*  pass in last argument */
 #if GASNET_PSHM
   if_pt (gasneti_pshm_in_supernode(dest)) {
+    gasneti_AMPoll(); /* poll at least once, to assure forward progress */
     retval = gasneti_AMPSHM_RequestGeneric(gasnetc_Medium, dest, handler,
                                            source_addr, nbytes, 0,
                                            numargs, argptr);
   } else
 #endif
   {
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AMLOCK_TOSEND();
       GASNETI_AM_SAFE_NORETURN(retval,
                AMUDP_RequestIVA(gasnetc_endpoint, dest, handler, 
@@ -707,6 +725,7 @@ extern int gasnetc_AMRequestLongM( gasnet_node_t dest,        /* destination nod
   va_start(argptr, numargs); /*  pass in last argument */
 #if GASNET_PSHM
   if_pt (gasneti_pshm_in_supernode(dest)) {
+      gasneti_AMPoll(); /* poll at least once, to assure forward progress */
       retval = gasneti_AMPSHM_RequestGeneric(gasnetc_Long, dest, handler,
                                              source_addr, nbytes, dest_addr,
                                              numargs, argptr);
@@ -716,8 +735,6 @@ extern int gasnetc_AMRequestLongM( gasnet_node_t dest,        /* destination nod
     uintptr_t dest_offset;
     dest_offset = ((uintptr_t)dest_addr) - ((uintptr_t)gasneti_seginfo[dest].addr);
 
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AMLOCK_TOSEND();
       GASNETI_AM_SAFE_NORETURN(retval,
                AMUDP_RequestXferVA(gasnetc_endpoint, dest, handler, 
@@ -773,8 +790,6 @@ extern int gasnetc_AMReplyMediumM(
   } else
 #endif
   {
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AM_ASSERT_LOCKED();
     GASNETI_AM_SAFE_NORETURN(retval,
               AMUDP_ReplyIVA(token, handler, source_addr, nbytes, numargs, argptr));
@@ -810,8 +825,6 @@ extern int gasnetc_AMReplyLongM(
     GASNETI_SAFE_PROPAGATE(gasnet_AMGetMsgSource(token, &dest));
     dest_offset = ((uintptr_t)dest_addr) - ((uintptr_t)gasneti_seginfo[dest].addr);
 
-    if_pf (!nbytes) source_addr = (void*)(uintptr_t)1; /* Bug 2774 - anything but NULL */
-
     AM_ASSERT_LOCKED();
     GASNETI_AM_SAFE_NORETURN(retval,
               AMUDP_ReplyXferVA(token, handler, source_addr, nbytes, dest_offset, numargs, argptr));
@@ -829,7 +842,7 @@ extern int gasnetc_AMReplyLongM(
   See the GASNet spec and http://gasnet.lbl.gov/dist/docs/gasnet.html for
     philosophy and hints on efficiently implementing no-interrupt sections
   Note: the extended-ref implementation provides a thread-specific void* within the 
-    gasnete_threaddata_t data structure which is reserved for use by the core 
+    gasneti_threaddata_t data structure which is reserved for use by the core 
     (and this is one place you'll probably want to use it)
 */
 #if GASNETC_USE_INTERRUPTS
@@ -983,6 +996,67 @@ extern int  gasnetc_hsl_trylock(gasnet_hsl_t *hsl) {
   }
 #endif
 
+/* ------------------------------------------------------------------------------------ */
+/*
+  Checkpoint/restart
+  ==================
+  thin wrappers around AMUDP-level support
+*/
+
+#if GASNET_BLCR
+
+/* NON-collective checkpoint request */
+int gasnet_checkpoint(const char *dir) {
+  int i, rc;
+
+  gasneti_flush_streams();
+
+  AMLOCK();
+  rc = AMUDP_SPMDCheckpoint(&gasnetc_bundle, &gasnetc_endpoint, dir);
+  if (rc > 0) {
+    /* Handlers */
+    for (i=0; i<GASNETC_MAX_NUMHANDLERS; i++) {
+      if (gasnetc_handler[i] != (gasneti_handler_fn_t)&gasneti_defaultAMHandler) {
+        AM_SetHandler(gasnetc_endpoint, (handler_t)i, gasnetc_handler[i]);
+        /* BLCR-TODO: error-checking */
+      }
+    }
+
+    /* Segment */
+    i = AM_SetSeg(gasnetc_endpoint,
+                  gasneti_seginfo[gasneti_mynode].addr,
+                  gasneti_seginfo[gasneti_mynode].size);
+    /* BLCR-TODO: error-checking */
+
+    #if GASNET_TRACE
+      if (GASNETI_TRACE_ENABLED(A))
+        GASNETI_AM_SAFE(AMUDP_SetHandlerCallbacks(gasnetc_endpoint,
+          gasnetc_enteringHandler_hook, gasnetc_leavingHandler_hook));
+    #endif
+  }
+  AMUNLOCK();
+
+#if GASNET_DEBUG_VERBOSE
+  fprintf(stderr, "Node %d %s checkpoint\n", gasneti_mynode, rc?"restart from":"continue after");
+#endif
+
+  return rc;
+}
+
+/* Collective checkpoint request */
+
+int gasnet_all_checkpoint(const char *dir_arg) {
+  const char *dir;
+  int rc;
+  dir = gasneti_checkpoint_dir(dir_arg);
+  gasnet_barrier(0, GASNET_BARRIERFLAG_ANONYMOUS);
+  rc = gasnet_checkpoint(dir);
+  if (!dir_arg) gasneti_free((void*)dir);
+  gasnet_barrier(0, GASNET_BARRIERFLAG_ANONYMOUS);
+  return rc;
+}
+
+#endif /* GASNET_BLCR */
 /* ------------------------------------------------------------------------------------ */
 /*
   Private Handlers:
